# -*- coding: utf-8 -*-
# Stalker a Production Asset Management System
# Copyright (C) 2009-2016 Erkan Ozgur Yilmaz
#
# This file is part of Stalker.
#
# Stalker is free software: you can redistribute it and/or modify
# it under the terms of the Lesser GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License.
#
# Stalker is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# Lesser GNU General Public License for more details.
#
# You should have received a copy of the Lesser GNU General Public License
# along with Stalker.  If not, see <http://www.gnu.org/licenses/>

import unittest

from stalker import Role


class RoleTestCase(unittest.TestCase):
    """testing the Role class
    """

    def setUp(self):
        """set the test up
        """
        # the role class does not have anything to test
        pass

    def test_role_class_generic(self):
        """testing of creation of a Role instance
        """
        r = Role(name='Lead')
        self.assertTrue(isinstance(r, Role))
        self.assertEqual(r.name, 'Lead')
