<!-- Tufts VUE 3.3.0 concept-map (stalker_design.vue) 2016-08-17 -->
<!-- Tufts VUE: http://vue.tufts.edu/ -->
<!-- Do Not Remove: VUE mapping @version(1.1) jar:file:/C:/Program%20Files%20(x86)/VUE/VUE.jar!/tufts/vue/resources/lw_mapping_1_1.xml -->
<!-- Do Not Remove: Saved date Wed Aug 17 11:56:58 EEST 2016 by eoyilmaz on platform Windows 8 6.2 in JVM 1.7.0_21-b11 -->
<!-- Do Not Remove: Saving version @(#)VUE: built October 8 2015 at 1658 by tomadm on Linux 2.6.32-504.23.4.el6.x86_64 i386 JVM 1.7.0_21-b11(bits=32) -->
<?xml version="1.0" encoding="US-ASCII"?>
<LW-MAP xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xsi:noNamespaceSchemaLocation="none" ID="0"
    label="stalker_design.vue" created="0" x="0.0" y="0.0"
    width="12530.668" height="3696.0002" strokeWidth="0.0" autoSized="false">
    <resource referenceCreated="1471424218530" size="531873"
        spec="C:\Users\eoyilmaz\Documents\development\stalker\stalker\docs\source\_static\images\stalker_design.vue"
        type="1" xsi:type="URLResource">
        <title>stalker_design.vue</title>
        <property key="File" value="C:\Users\eoyilmaz\Documents\development\stalker\stalker\docs\source\_static\images\stalker_design.vue"/>
    </resource>
    <fillColor>#FFFFFF</fillColor>
    <strokeColor>#404040</strokeColor>
    <textColor>#000000</textColor>
    <font>SansSerif-plain-14</font>
    <URIString>http://vue.tufts.edu/rdf/resource/5d104b14c00007d601b277f03c449819</URIString>
    <child ID="4225" label="Budget" layerID="1" created="1471422397174"
        x="9346.564" y="2293.409" width="160.5" height="165.2019"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/97ae2d14c0a82a4a019efdbdfb2a0477</URIString>
        <child ID="4232" label="ProjectMixin" created="1471422481990"
            x="34.0" y="23.0" width="82.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/97ae2d14c0a82a4a019efdbde18ed0a7</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4248" label="DAGMixin" created="1471423050069"
            x="34.0" y="43.25" width="65.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/97ae2d14c0a82a4a019efdbd0fb6ad9c</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4226" label="entries" created="1471422397175"
            x="34.0" y="63.5" width="49.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/97ae2d15c0a82a4a019efdbdad4b79ff</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4227" label="custom_template | STRING"
            created="1471422397175" x="34.0" y="83.75" width="161.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/97ae2d15c0a82a4a019efdbd06132a49</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4228" created="1471422397175" x="34.0" y="104.0"
            width="30.333334" height="25.602621" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#FEFB03</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/97ae2d15c0a82a4a019efdbd9fd9c70a</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <child ID="4229" created="1471422397176" x="34.0" y="126.201965"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#AF55F4</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/97ae2d15c0a82a4a019efdbd6059f1d9</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <child ID="4230" created="1471422397176" x="34.0" y="144.20193"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/97ae2d15c0a82a4a019efdbd669ca722</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="3382" label="Message" layerID="1" created="1295883012242"
        x="9785.448" y="1508.0773" width="199.5" height="226.24997"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#D4FF00</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/b8a673297f0001017e6ab533f3ced4f1</URIString>
        <child ID="3467" label="StatusMixin" created="1296050031611"
            x="34.0" y="23.0" width="78.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/c2bbcec17f0001013611c755dc3f00fa</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3383" label="from --> created_by"
            created="1295883012242" x="34.0" y="43.25" width="123.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/b8a673297f0001017e6ab533fe3db766</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3384" label="to | MANY | USER"
            created="1295883012242" x="34.0" y="63.5" width="105.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#DAA9FF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/b8a673297f0001017e6ab533c72f66e7</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3385" label="subject | STRING"
            created="1295883012242" x="34.0" y="83.75" width="105.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/b8a673297f0001017e6ab533ddf68e57</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3386" label="body | STRING" created="1295883012243"
            x="34.0" y="104.0" width="91.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/b8a6732a7f0001017e6ab533785e3d58</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3387" label="in_reply_to | INTEGER | MESSAGE.id"
            created="1295883012243" x="34.0" y="124.25" width="212.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/b8a6732a7f0001017e6ab5335ce88fa0</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3388" label="replies | MANY | MESSAGE"
            created="1295883012243" x="34.0" y="144.5" width="156.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#DAA9FF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/b8a6732a7f0001017e6ab533df4fafbf</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3390" label="attachments | MANY | SIMPLEENTTIY"
            created="1295883012243" x="34.0" y="164.75" width="213.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/b8a6732a7f0001017e6ab5337661773f</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3391" created="1295883012243" x="34.0" y="185.0"
            width="29.0" height="23.0" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/b8a6732a7f0001017e6ab533aa4e4f71</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <child ID="4052" created="1358149001263" x="34.0" y="205.25"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#30D643</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/37fd2d78c0a8000435fa83793037fe30</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="1088" label="Asset" layerID="1" created="1272403764881"
        x="8932.898" y="2475.277" width="178.5" height="142.99995"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/4137afd47f00010135ea5711d86d98a9</URIString>
        <child ID="4016" label="CodeMixin" created="1358081793429"
            x="34.0" y="23.0" width="70.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/33fc5650c0a8000435fa8379675e6ac8</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1512" label="shots | MANY | SHOT"
            created="1277558085545" x="34.0" y="43.25" width="125.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/746640447f0001014a6b2ab7fc9c5221</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1137" label="sequences | shots -> sequence"
            created="1272404571518" x="34.0" y="63.5" width="185.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/4140ccdd7f00010135ea57116e5577a9</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2499" created="1295294796978" x="34.0" y="83.75"
            width="29.0" height="23.0" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#FEFB03</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/9597b7657f0001015ee819a8aafadfdf</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <child ID="3787" created="1310511119720" x="34.0" y="104.0"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#AF55F4</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/208db1db7f000101264d107e3c78088c</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <child ID="4039" created="1358111943627" x="34.0" y="121.99998"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35c8aec5c0a8000435fa837926646ede</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="1131" layerID="1" created="1272404424409" x="8775.111"
        y="751.1875" width="1115.1855" height="757.40625"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/4137afe67f00010135ea57113e9099fb</URIString>
        <point1 x="8775.611" y="751.6875"/>
        <point2 x="9889.797" y="1508.0938"/>
        <ID1 xsi:type="node">1887</ID1>
        <ID2 xsi:type="node">3382</ID2>
        <ctrlPoint0 x="8769.293" y="1480.0895" xsi:type="point"/>
        <ctrlPoint1 x="9894.276" y="1397.921" xsi:type="point"/>
    </child>
    <child ID="1145" label="Link" layerID="1" created="1272404675030"
        x="9597.522" y="828.225" width="168.75" height="185.4519"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/4140cd037f00010135ea57112130f8c4</URIString>
        <child ID="1147" label="full_path | UNICODE"
            created="1272404693670" x="34.0" y="23.0" width="123.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/4140cd037f00010135ea57117ab47786</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3989" label="original_filename | UNICODE"
            created="1352553321865" x="34.0" y="43.25" width="172.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/ec507e437f00010143bf3d773ab1cc57</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4141" label="path" created="1369384313981" x="34.0"
            y="63.5" width="36.0" height="23.0" strokeWidth="1.0"
            autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/d5acab167b6a9fda5a40fe82ce930ac9</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4156" label="filename" created="1369385659119"
            x="34.0" y="83.75" width="60.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/d5bfbb467b6a9fda5a40fe82ac8cdeb8</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4157" label="extension" created="1369385672521"
            x="34.0" y="104.0" width="67.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/d5bfbb477b6a9fda5a40fe824a2cb1f4</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4143" created="1369384327440" x="34.0" y="124.25"
            width="30.333334" height="25.602621" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#FEFB03</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/d5acab0d7b6a9fda5a40fe82688b31f3</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <child ID="3788" created="1310511178848" x="34.0" y="146.45197"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#AF55F4</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/208db1ed7f000101264d107ef3bb38ae</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <child ID="4044" created="1358112017696" x="34.0" y="164.45193"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#30D643</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/3624ae36c0a8000435fa837938f1c1e7</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="1157" label="Playlist" layerID="1"
        created="1272404832018" x="9051.126" y="1514.2256" width="123.0"
        height="66.5" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#ECFFD4</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/4140cd0a7f00010135ea571155c75028</URIString>
        <child ID="1160" label="files | MANY | LINK"
            created="1272404845966" x="34.0" y="23.0" width="111.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/4140cd0b7f00010135ea5711602370dd</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2491" created="1295294705784" x="34.0" y="43.25"
            width="29.0" height="23.0" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/9597b7687f0001015ee819a82b2a236f</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="1166" label="Project" layerID="1" created="1272404937838"
        x="8377.885" y="1858.0773" width="201.75" height="437.74994"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/4140cd0d7f00010135ea5711f07153c7</URIString>
        <child ID="3461" label="StatusMixin" created="1296050001344"
            x="34.0" y="23.0" width="78.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/c2bbced07f0001013611c75577191d5f</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3486" label="ScheduleMixin" created="1297430760778"
            x="34.0" y="43.25" width="94.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/14e83dd27f0001017490649e82b3b87a</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4013" label="CodeMixin" created="1358081773550"
            x="34.0" y="63.5" width="70.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/33fc5655c0a8000435fa8379368c5321</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2221" label="lead | ONE | USER"
            created="1291042857476" x="34.0" y="83.75" width="113.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/98294d057f000101247aa859e3c11b58</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1799" label="image_format | ONE | IMAGEFORMAT"
            created="1279573849214" x="34.0" y="104.0" width="216.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/ec8bebda7f0001010147e4f04ea33efd</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1423" label="fps | FLOAT" created="1277505694849"
            x="34.0" y="124.25" width="74.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/7146b48d7f0001010a530461cc109033</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2007" label="structure | ONE | STRUCTURE"
            created="1289689925944" x="34.0" y="144.5" width="178.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/47830e6e7f0001015a3184d85f950ed5</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2195" label="repository | ONE | REPOSITORY"
            created="1290434059609" x="34.0" y="164.75" width="186.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/73dd535d7f0001010b45ad3dcc59d99c</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2011" label="is_stereoscopic | BOOL"
            created="1290254126593" x="34.0" y="185.0" width="141.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/6924820d7f00010168376cc3eaa57d6b</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1962" label="display_width | FLOAT"
            created="1289153600481" x="34.0" y="205.25" width="132.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/278b9a9a7f000101775978e75d5e32b6</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4059" label="working_hours | dictionary"
            created="1363733530407" x="34.0" y="225.5" width="154.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/8c9f23e0eead3b3354aea17a122ae7af</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4063" label="tasks | MANY | TASK"
            created="1363733707277" x="34.0" y="245.75" width="121.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/8c9f23e1eead3b3354aea17a64edbd9c</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3812" label="tickets | MANY | TICKET"
            created="1315471223904" x="34.0" y="266.0" width="138.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/48326e217f0001017a8ff303fe1e74b3</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1171" label="users | MANY | USER"
            created="1272404979390" x="34.0" y="286.25" width="127.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/4140cd107f00010135ea57114a508f5b</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1506" label="sequences | MANY | SEQUENCE"
            created="1277546655591" x="34.0" y="306.5" width="191.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/73b722e97f000101783f472b90b90716</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1520" label="assets | MANY | ASSET"
            created="1277849783149" x="34.0" y="326.75" width="137.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/85ca6d117f0001010cdfa65c929e6a1b</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3997"
            label="project_tasks -> resources (for&#xa;all TaskableEntities.tasks)"
            created="1357765406294" x="34.0" y="347.0" width="179.0"
            height="38.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/2120bb40c0a80004173b812d2e40c79a</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2496" created="1295294753204" x="34.0" y="378.5"
            width="29.0" height="23.0" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#FEFB03</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/9597b76a7f0001015ee819a8fe024cea</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <child ID="3786" created="1310511115547" x="34.0" y="398.75"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#AF55F4</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/208db2207f000101264d107e85978e7a</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <child ID="4036" created="1358111937208" x="34.0" y="416.74997"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35c8aec9c0a8000435fa83792ea2bbbd</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="1176" label="Sequence" layerID="1"
        created="1272405154343" x="8520.342" y="2475.277" width="133.5"
        height="145.24997" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/4144a3f27f00010135ea57117ebd1aac</URIString>
        <child ID="4014" label="CodeMixin" created="1358081784556"
            x="34.0" y="23.0" width="70.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/33fc565ac0a8000435fa8379e65a22be</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1182" label="shots | MANY | SHOT"
            created="1272405273586" x="34.0" y="43.25" width="125.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/4144a3f47f00010135ea57118b03d3ed</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2008" label="offline | LINK" created="1289702947670"
            x="34.0" y="63.5" width="80.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/484975a57f0001015a3184d81211a0d2</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2010" label="stoaryboard | LINK"
            created="1289703052278" x="34.0" y="83.75" width="112.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/484b00917f0001015a3184d88131b25d</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2497" created="1295294780293" x="34.0" y="104.0"
            width="29.0" height="23.0" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#FEFB03</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/9597b76b7f0001015ee819a858e7cc58</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <child ID="4037" created="1358111939638" x="34.0" y="124.25"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35c8aecbc0a8000435fa837918244a2e</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="1185" label="Shot" layerID="1" created="1272405338854"
        x="8701.075" y="2475.277" width="201.75" height="286.99997"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/414f1a427f00010135ea57117bdc53d1</URIString>
        <child ID="4015" label="CodeMixin" created="1358081789140"
            x="34.0" y="23.0" width="70.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/33fc565dc0a8000435fa83794c989a62</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1519" label="sequences | MANY | SEQUENCE"
            created="1277849616279" x="34.0" y="43.25" width="191.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/85ca6d1d7f0001010cdfa65c7007b57b</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4139" label="scenes | MANY | SCENES"
            created="1368620381286" x="34.0" y="63.5" width="152.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/a8223ca7a62df9af4916575120e6ee34</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1188" label="cut_in | INTEGER"
            created="1272405364822" x="34.0" y="83.75" width="106.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/414f1a427f00010135ea5711fda03f80</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1190" label="cut_out | INTEGER"
            created="1272405378378" x="34.0" y="104.0" width="113.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/414f1a437f00010135ea57113c08fcf5</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1187" label="cut_duration | INTEGER"
            created="1272405350634" x="34.0" y="124.25" width="141.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/414f1a427f00010135ea57111b10b68c</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4083" label="image_format | ONE | IMAGEFORMAT"
            created="1365064238300" x="34.0" y="144.5" width="216.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/d42b5a304cc66935691a928b2c7f7d9d</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3828" label="elements | MANY | FILE"
            created="1336475842369" x="34.0" y="164.75" width="138.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/2c2b7b2c7f00010118154bb12694d02a</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2009" label="online | LINK" created="1289702971599"
            x="34.0" y="185.0" width="81.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/4849e7047f0001015a3184d87c51e040</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1960" label="inter_axial_distance | FLOAT"
            created="1289153571666" x="34.0" y="205.25" width="167.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/278b9ab27f000101775978e70bad0084</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1961" label="convergence_distance | FLOAT"
            created="1289153587186" x="34.0" y="225.5" width="181.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/278b9ab37f000101775978e7a0658db2</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2502" created="1295294828167" x="34.0" y="245.75"
            width="29.0" height="23.0" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#FEFB03</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/9597b76d7f0001015ee819a88a8002c7</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <child ID="4051" created="1358118709200" x="34.0" y="266.0"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#30D643</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/362ef8b9c0a8000435fa8379dcc6daaa</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="1205" label="Task" layerID="1" created="1272406042962"
        x="8630.298" y="1858.0773" width="251.25" height="367.99997"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/4156061a7f00010135ea5711dcb8f75c</URIString>
        <child ID="3464" label="StatusMixin" created="1296050015627"
            x="34.0" y="23.0" width="78.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/c2bbcee47f0001013611c755c3131536</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4275" label="DateRangeMixin" created="1471423858087"
            x="34.0" y="43.25" width="103.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/97b7c5abc0a82a4a010817798adf5656</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3560" label="ReferenceMixin" created="1298330420443"
            x="34.0" y="63.5" width="100.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/4a8686e97f000101101873a641c3e7c7</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4200" label="ScheduleMixin" created="1459165867490"
            x="34.0" y="83.75" width="94.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/bd12292ac0a82a4a01152d0efe940b4e</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4246" label="DAGMixin" created="1471423013290"
            x="34.0" y="104.0" width="65.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/97ae2c9ec0a82a4a019efdbd325202e5</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1795" label="priority | INTEGER"
            created="1279319036406" x="34.0" y="124.25" width="109.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/dd5bc86d7f0001010c4cdf7ed67689ed</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1206" label="resources | MANY | USER"
            created="1272406057074" x="34.0" y="144.5" width="151.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/4156061a7f00010135ea57115fceb210</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4086" label="bid | STRING" created="1365379231725"
            x="34.0" y="164.75" width="82.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/e8a41d0c648c6197658d749a0994b723</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1394" label="depends | MANY | TASK"
            created="1277500447230" x="34.0" y="185.0" width="140.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/70f609097f0001010a5304612bb962ad</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1213" label="milestone | BOOL"
            created="1272406111786" x="34.0" y="205.25" width="108.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/4156061c7f00010135ea5711df3cc9d5</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1219" label="time_logs | MANY | TIMELOG"
            created="1272406178764" x="34.0" y="225.5" width="168.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/4156061d7f00010135ea5711f78c3ffb</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1482" label="versions | MANY | VERSION"
            created="1277543180124" x="34.0" y="245.75" width="161.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/738287e27f000101783f472bbfb64909</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4062" label="project | ONE | PROJECT"
            created="1363733633376" x="34.0" y="266.0" width="148.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/8c9f23eceead3b3354aea17a20109083</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1525"
            label="published_versions | version.published == True"
            created="1277850285557" x="34.0" y="286.25" width="277.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/85d09fcc7f0001010cdfa65c9b0d5367</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1526"
            label="last_published_version | published_versions[-1]"
            created="1277850333863" x="34.0" y="306.5" width="282.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/85d0f0437f0001010cdfa65c3ab63ddc</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2501" created="1295294816788" x="34.0" y="326.75"
            width="29.0" height="23.0" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#FEFB03</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/9597b76f7f0001015ee819a8bb925774</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <child ID="4040" created="1358111953850" x="34.0" y="347.0"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35c8aed0c0a8000435fa83790c3a64d8</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="1220" layerID="1" created="1272406209511" x="8757.033"
        y="751.1875" width="18.544922" height="1107.4375"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/4156061d7f00010135ea5711c2172d7e</URIString>
        <point1 x="8775.078" y="751.6875"/>
        <point2 x="8757.533" y="1858.125"/>
        <ID1 xsi:type="node">1887</ID1>
        <ID2 xsi:type="node">1205</ID2>
        <ctrlPoint0 x="8757.643" y="1831.0906" xsi:type="point"/>
        <ctrlPoint1 x="8757.919" y="1814.0806" xsi:type="point"/>
    </child>
    <child ID="1227" label="TimeLog" layerID="1" created="1272406409370"
        x="9203.517" y="1514.2256" width="143.25" height="124.99998"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/4156061e7f00010135ea571114c0ee72</URIString>
        <child ID="3814" label="ScheduleMixin" created="1316463635820"
            x="34.0" y="23.0" width="94.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/835f6ce87f000101123acf342d8e5a89</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1231" label="task | ONE | TASK"
            created="1272406430066" x="34.0" y="43.25" width="108.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/4156061f7f00010135ea57112dad9a44</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1232" label="resource | ONE | USER"
            created="1272406435518" x="34.0" y="63.5" width="138.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/4156061f7f00010135ea57110e5bbdd2</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2493" created="1295294718645" x="34.0" y="83.75"
            width="29.0" height="23.0" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/9597b7707f0001015ee819a8d0c7631c</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <child ID="4050" created="1358118036346" x="34.0" y="104.0"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#30D643</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/3624ae42c0a8000435fa83796660f2f7</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="1235" label="User" layerID="1" created="1272406484302"
        x="8087.4805" y="1348.2256" width="206.25" height="415.2499"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/415b1bc47f00010135ea571102f905e6</URIString>
        <child ID="4072" label="ACLMixin" created="1365050633635"
            x="34.0" y="23.0" width="64.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/d35be5214cc66935691a928b1dbc39ce</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3958" label="StatusMixin" created="1337252633904"
            x="34.0" y="43.25" width="78.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/5f5269207f0001011df0758683afba34</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1237" label="email | UNICODE" created="1272406498598"
            x="34.0" y="63.5" width="107.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/415b1bc57f00010135ea5711bb1452df</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1243" label="login | UNICODE" created="1272406543618"
            x="34.0" y="83.75" width="103.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/415b1bc67f00010135ea5711cc5235aa</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1241" label="departments | MANY | DEPARTMENT"
            created="1272406526891" x="34.0" y="104.0" width="212.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/415b1bc57f00010135ea5711df108926</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1245" label="password | UNICODE"
            created="1272406576770" x="34.0" y="124.25" width="131.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/415b1bc67f00010135ea57118ffe5686</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1246" label="groups | MANY | GROUPS"
            created="1272406583326" x="34.0" y="144.5" width="152.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/415b1bc77f00010135ea57113656d812</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1442" label="tasks | MANY | TASK"
            created="1277507144415" x="34.0" y="164.75" width="121.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/715c39c47f0001010a53046192f39ef8</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2370" label="last_login | DATETIME"
            created="1294054995724" x="34.0" y="185.0" width="132.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/4bb089d87f000101283b96bbba818d63</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4158" label="vacations | MANY | VACATION"
            created="1382708754314" x="34.0" y="205.25" width="171.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/efe00692c8211d50048a3743a2254d81</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1530" label="sequences_lead | MANY | SEQUENCE"
            created="1277850633268" x="34.0" y="225.5" width="222.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#DAA9FF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/85d5b47c7f0001010cdfa65ca49c356e</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2222" label="projects_lead| MANY | PROJECT"
            created="1291125082795" x="34.0" y="245.75" width="189.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#DAA9FF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/9d0d965c7f0001015d73b35d3456f432</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3381" label="messages | MANY | MESSAGE"
            created="1295882058734" x="34.0" y="266.0" width="178.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#DAA9FF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/b8973e797f0001017e6ab533ca951c0f</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1528" label="projects | tasks -> project"
            created="1277850527948" x="34.0" y="286.25" width="156.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/85d3daed7f0001010cdfa65c1ca9c5be</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2220"
            label="assets | tasks -> owner&#xa;query(Asset).filter(User.assets)"
            created="1291042704348" x="34.0" y="306.5" width="190.0"
            height="38.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/982531327f000101247aa859a673b56a</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2485" created="1295294628854" x="34.0" y="338.0"
            width="29.0" height="23.0" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#FEFB03</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/959391d87f0001015ee819a88b8f9436</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <child ID="4034" created="1358111917529" x="34.0" y="358.25"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35c8aed4c0a8000435fa83798d86a388</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <child ID="4223" created="1468653377365" x="34.0" y="376.24997"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#33A8F5</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/f2902e11c0a8000f011bc52f7510551a</URIString>
            <shape xsi:type="chevron"/>
        </child>
        <child ID="4224" created="1468653377365" x="34.0" y="394.24994"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#AF55F4</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/f2902e11c0a8000f011bc52fda3eaaf8</URIString>
            <shape xsi:type="rhombus"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="1258" label="ShootTake" layerID="1"
        created="1272406937150" x="9259.6045" y="1858.0773"
        width="138.0" height="228.5" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#EA2218</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/416412927f00010135ea5711ee89e7ec</URIString>
        <child ID="1262" label="first_frame" created="1272406965702"
            x="34.0" y="23.0" width="75.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/416412937f00010135ea5711e1bf0da0</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1263" label="frames_aspect_ratio"
            created="1272406971602" x="34.0" y="43.25" width="131.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/416412937f00010135ea5711025e4908</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1264" label="frames_have_slate"
            created="1272406979937" x="34.0" y="63.5" width="121.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/416412937f00010135ea571114f2487c</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1265" label="last_frame" created="1272406987082"
            x="34.0" y="83.75" width="73.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/416412947f00010135ea5711ba3432c6</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1266" label="link (entity)" created="1272406995742"
            x="34.0" y="104.0" width="73.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/416412947f00010135ea57119c4a005f</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1267" label="movie_aspect_ratio"
            created="1272407002554" x="34.0" y="124.25" width="124.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/416412947f00010135ea571193baeb10</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1268" label="movie_has_slate" created="1272407012830"
            x="34.0" y="144.5" width="108.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/416412947f00010135ea57111edef472</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1269" label="path_to_frames" created="1272407024178"
            x="34.0" y="164.75" width="102.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/416412947f00010135ea571180cf8f37</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1275" label="uploaded_movie" created="1272407065510"
            x="34.0" y="185.0" width="104.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/416412967f00010135ea5711e7b676e0</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2506" created="1295294864385" x="34.0" y="205.25"
            width="29.0" height="23.0" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/9597b7747f0001015ee819a8fb4d3c74</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="1279" layerID="1" created="1272407119615" x="8774.577"
        y="751.1875" width="550.34766" height="1107.4062"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/416412967f00010135ea57116b7f484f</URIString>
        <point1 x="8775.077" y="751.6875"/>
        <point2 x="9324.425" y="1858.0938"/>
        <ID1 xsi:type="node">1887</ID1>
        <ID2 xsi:type="node">1258</ID2>
        <ctrlPoint0 x="8757.623" y="1831.3555" xsi:type="point"/>
        <ctrlPoint1 x="9321.81" y="1786.6119" xsi:type="point"/>
    </child>
    <child ID="1295" label="Tag" layerID="1" created="1272407402727"
        x="8619.469" y="609.8772" width="62.5" height="102.2019"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/41652daa7f00010135ea57110fc3ae1b</URIString>
        <child ID="4150" created="1369384396735" x="34.0" y="23.0"
            width="30.333334" height="25.602621" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#FEFB03</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/d5acabf37b6a9fda5a40fe82fa141e70</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <child ID="4025" created="1358111854271" x="34.0" y="45.201965"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35c8aed6c0a8000435fa8379a22ba050</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <child ID="4215" created="1468653297748" x="34.0" y="63.201942"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#33A8F5</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/f28ffbadc0a8000f011bc52f90de7713</URIString>
            <shape xsi:type="chevron"/>
        </child>
        <child ID="4216" created="1468653301243" x="34.0" y="81.20192"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#AF55F4</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/f28ffbadc0a8000f011bc52f19c47bcb</URIString>
            <shape xsi:type="rhombus"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="1299" label="Event" layerID="1" created="1272407465751"
        x="10222.512" y="828.225" width="158.25" height="167.75"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#EA2218</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/41652dab7f00010135ea57113765e4bc</URIString>
        <child ID="1300" label="attribute_name | UNICODE"
            created="1272407733084" x="34.0" y="23.0" width="158.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/417945b37f00010135ea5711f7eb2636</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1302" label="event_type | INTEGER"
            created="1272407752888" x="34.0" y="43.25" width="130.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/417945b47f00010135ea5711afaf9d5a</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1304" label="entity | ONE | ENTITY"
            created="1272407763884" x="34.0" y="63.5" width="123.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/417945b47f00010135ea57110de734c6</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1308" label="user | ONE | USER"
            created="1272407788218" x="34.0" y="83.75" width="110.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/417945b57f00010135ea5711d9b9080c</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1309" label="date_created | DATETIME"
            created="1272407797912" x="34.0" y="104.0" width="153.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/417945b57f00010135ea5711545ebf12</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1305" label="meta_data | UNICODE"
            created="1272407772990" x="34.0" y="124.25" width="132.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/417945b47f00010135ea57118bfc42a4</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2510" created="1295294901044" x="34.0" y="144.5"
            width="29.0" height="23.0" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/9597b7767f0001015ee819a8317b3c9c</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="1329" label="Department" layerID="1"
        created="1273399280163" x="8326.049" y="1348.2256" width="135.0"
        height="124.99998" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/7c845ab97f00010136a8a19945559ad8</URIString>
        <child ID="4071" label="CodeMixin" created="1364825474109"
            x="34.0" y="23.0" width="70.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/c5f034e54cc66935691a928b23df921d</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1331" label="users | MANY | USER"
            created="1273399329020" x="34.0" y="43.25" width="127.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/7c845ab97f00010136a8a19938a35367</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2223" label="lead | ONE | USER"
            created="1291842810434" x="34.0" y="63.5" width="113.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/c7d533fd7f00010136536500f161e613</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2489" created="1295294666491" x="34.0" y="83.75"
            width="29.0" height="23.0" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#FEFB03</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/959428897f0001015ee819a8be414bd8</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <child ID="4033" created="1358111904397" x="34.0" y="104.0"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35c8aed8c0a8000435fa83795d4655ff</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="1355" label="StatusList" layerID="1"
        created="1274041570267" x="8298.414" y="828.2019" width="156.0"
        height="106.70194" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/a33b57e77f0001016609e5f078560790</URIString>
        <child ID="3825" label="TargetEntityTypeMixin"
            created="1317078064232" x="34.0" y="23.0" width="136.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/a800f7887f00010130ce553671097af3</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1395" label="statuses | MANY | STATUS"
            created="1277500691762" x="34.0" y="43.25" width="155.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/70f9fdd47f0001010a5304617b221190</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4153" created="1369384420632" x="34.0" y="63.5"
            width="30.333334" height="25.602621" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#FEFB03</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/d5acac0d7b6a9fda5a40fe82045cd555</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <child ID="4027" created="1358111867437" x="34.0" y="85.701965"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35c8aed9c0a8000435fa837963fcd72d</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="1362" label="Version" layerID="1" created="1275345214941"
        x="8911.798" y="1858.0773" width="234.0" height="228.49997"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/f0822ba57f000101666558bbfaa1423c</URIString>
        <child ID="4247" label="DAGMixin" created="1471423035442"
            x="34.0" y="23.0" width="65.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/97ae2cb3c0a82a4a019efdbdc1bcd62a</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1365"
            label="take_name | STRING, DEFAULT=&quot;Main&quot;"
            created="1275345214943" x="34.0" y="43.25" width="228.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/f0822ba97f000101666558bbc49201a2</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1452" label="version_number | INTEGER"
            created="1277507797831" x="34.0" y="63.5" width="163.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/716648fb7f0001010a530461d3a9c89a</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3729" label="is_published | BOOL"
            created="1309373289745" x="34.0" y="83.75" width="125.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/dcc1257d7f0001011f758c30f2b26443</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3993" label="inputs | MANY | LINK"
            created="1352652805978" x="34.0" y="104.0" width="122.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/f07316197f00010143bf3d77fa79a157</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1952" label="outputs | MANY | LINK"
            created="1288424286766" x="34.0" y="124.25" width="129.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/fc128cad7f00010120d48785781bf5c0</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2106"
            label="template |&#xa;version_of.project.structure.templates&#xa;.target_entity_type[task.part_of.entity_type]"
            created="1290418741395" x="34.0" y="144.5" width="259.0"
            height="53.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/72f3a55b7f0001010b45ad3d8fd4b444</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2504" created="1295294852115" x="34.0" y="187.25"
            width="29.0" height="23.0" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/9597b7787f0001015ee819a8802e3a4c</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <child ID="4046" created="1358112036428" x="34.0" y="207.5"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#30D643</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/3624ae50c0a8000435fa8379c88ae3ca</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="1367" layerID="1" created="1275345268184" x="9035.223"
        y="1013.17773" width="596.4971" height="845.4004"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/f0822baa7f000101666558bbc8e3f079</URIString>
        <point1 x="9631.22" y="1013.67773"/>
        <point2 x="9035.723" y="1858.0781"/>
        <ID1 xsi:type="node">1145</ID1>
        <ID2 xsi:type="node">1362</ID2>
        <ctrlPoint0 x="9546.949" y="1167.8689" xsi:type="point"/>
        <ctrlPoint1 x="9043.442" y="1730.703" xsi:type="point"/>
    </child>
    <child ID="1372" layerID="1" created="1277498507442" x="8193.134"
        y="751.21094" width="575.3457" height="597.53906"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/70dc3fa77f0001010a5304616bf72ea2</URIString>
        <point1 x="8767.9795" y="751.71094"/>
        <point2 x="8193.634" y="1348.25"/>
        <ID1 xsi:type="node">1887</ID1>
        <ID2 xsi:type="node">1235</ID2>
        <ctrlPoint0 x="8702.578" y="1317.4082" xsi:type="point"/>
        <ctrlPoint1 x="8195.002" y="1254.4216" xsi:type="point"/>
    </child>
    <child ID="1373" layerID="1" created="1277498564390" x="8388.514"
        y="751.1875" width="389.1543" height="597.53125"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/70dc3fa97f0001010a530461bed23383</URIString>
        <point1 x="8777.168" y="751.6875"/>
        <point2 x="8389.014" y="1348.2188"/>
        <ID1 xsi:type="node">1887</ID1>
        <ID2 xsi:type="node">1329</ID2>
        <ctrlPoint0 x="8784.788" y="1330.6824" xsi:type="point"/>
        <ctrlPoint1 x="8379.2705" y="1213.946" xsi:type="point"/>
    </child>
    <child ID="1396" label="Status" layerID="1" created="1277500787817"
        x="8165.1694" y="828.2019" width="92.25" height="120.49993"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/70fbee177f0001010a53046131252ebc</URIString>
        <child ID="4053" label="CodeMixin" created="1358233205184"
            x="34.0" y="23.0" width="70.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/3d020999c0a800044f9e91d0cec913ac</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2474" created="1295294299280" x="34.0" y="43.25"
            width="29.0" height="23.0" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#FEFB03</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/958f53697f0001015ee819a87bfe14d0</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <child ID="4028" created="1358111869751" x="34.0" y="63.5"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35c8aedcc0a8000435fa83792e20f08b</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <child ID="4221" created="1468653326129" x="34.0" y="81.49998"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#33A8F5</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/f28ffbb1c0a8000f011bc52fdce9b087</URIString>
            <shape xsi:type="chevron"/>
        </child>
        <child ID="4222" created="1468653326129" x="34.0" y="99.499954"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#AF55F4</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/f28ffbb1c0a8000f011bc52fdc0896fc</URIString>
            <shape xsi:type="rhombus"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="1402" label="Groups" layerID="1" created="1277500921470"
        x="7935.752" y="1347.4773" width="135.0" height="104.74998"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/70ff4c457f0001010a5304618b714b63</URIString>
        <child ID="4073" label="ACLMixin" created="1365050640218"
            x="34.0" y="23.0" width="64.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/d35be5474cc66935691a928b9ebf2073</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1503" label="users | MANY | USER"
            created="1277544525089" x="34.0" y="43.25" width="127.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/7396a2bf7f000101783f472b0c288dca</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2488" created="1295294645319" x="34.0" y="63.5"
            width="29.0" height="23.0" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/9594288d7f0001015ee819a808d4eae0</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <child ID="4035" created="1358111921466" x="34.0" y="83.75"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35c8aeddc0a8000435fa83793c037b2a</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="1444" label="Playblast / Flipbook" layerID="1"
        created="1277507592356" x="9377.27" y="1514.2256" width="147.0"
        height="86.75" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#EA2218</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/716433b77f0001010a530461ea2689a9</URIString>
        <child ID="1450" label="of | ONE | ASSETBASE"
            created="1277507592357" x="34.0" y="23.0" width="133.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/716433b87f0001010a530461ae077f28</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1938" label="file | ONE | FILE"
            created="1288269754490" x="34.0" y="43.25" width="93.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/f2dcabc17f0001010d9438de492b2f36</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2494" created="1295294732353" x="34.0" y="63.5"
            width="29.0" height="23.0" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/9597b77c7f0001015ee819a88d3af5f8</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="1465" label="ImageFormat" layerID="1"
        created="1277508730708" x="8500.845" y="829.0019" width="147.0"
        height="167.45193" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/7175e3647f0001010a5304618afb3c6d</URIString>
        <child ID="1468" label="width | INTEGER" created="1277508730709"
            x="34.0" y="23.0" width="102.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/7175e3647f0001010a530461c2444b8a</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1469" label="height | INTEGER"
            created="1277508787050" x="34.0" y="43.25" width="107.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/7175e3647f0001010a530461a525aa4a</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1471" label="device_aspect | FLOAT"
            created="1277508818841" x="34.0" y="63.5" width="136.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/7175e3657f0001010a530461d423e0c0</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1470" label="pixel_aspect | FLOAT"
            created="1277508794355" x="34.0" y="83.75" width="126.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/7175e3647f0001010a530461ed5f93c3</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1797" label="print_resolution | FLOAT"
            created="1279319466446" x="34.0" y="104.0" width="143.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/dd623f737f0001010c4cdf7e1538b03a</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4152" created="1369384415644" x="34.0" y="124.25"
            width="30.333334" height="25.602621" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#FEFB03</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/d5acac407b6a9fda5a40fe8240c9c0bf</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <child ID="4026" created="1358111863766" x="34.0" y="146.45197"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35c8aedec0a8000435fa8379c28ae15c</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="1475" layerID="1" created="1277542415873" x="8778.163"
        y="751.1875" width="671.7285" height="763.53125"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/7376e9ec7f000101783f472b76efe287</URIString>
        <point1 x="8778.663" y="751.6875"/>
        <point2 x="9449.392" y="1514.2188"/>
        <ID1 xsi:type="node">1887</ID1>
        <ID2 xsi:type="node">1444</ID2>
        <ctrlPoint0 x="8800.1" y="1379.9562" xsi:type="point"/>
        <ctrlPoint1 x="9446.507" y="1423.3969" xsi:type="point"/>
    </child>
    <child ID="1555"
        label="STALKER OBJECT MODEL&#xa;INHERITANCE DIAGRAM v1.5.0"
        layerID="1" created="1279119389330" x="8543.104" y="-107.323044"
        width="501.333" height="102.66669" strokeWidth="1.0"
        autoSized="false" xsi:type="node">
        <fillColor>#B5B995</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-18</font>
        <URIString>http://vue.tufts.edu/rdf/resource/d175941d7f0001014ed1a34a819236b1</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="1817" label="Implemented classes" layerID="1"
        created="1282128100173" x="9222.637" y="-57.51773" width="208.0"
        height="17.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
        <strokeColor>#404040</strokeColor>
        <textColor>#000000</textColor>
        <font>SansSerif-plain-14</font>
        <URIString>http://vue.tufts.edu/rdf/resource/84ca7c997f00010138aea9817270f61e</URIString>
        <richText>&lt;html&gt;
  &lt;head&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Implemented classes
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
        <label>Implemented classes</label>
    </child>
    <child ID="1879" label="Implemented attributes" layerID="1"
        created="1288167062202" x="9222.637" y="98.84404" width="301.0"
        height="17.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
        <strokeColor>#404040</strokeColor>
        <textColor>#000000</textColor>
        <font>SansSerif-plain-14</font>
        <URIString>http://vue.tufts.edu/rdf/resource/ecbe81757f0001012cc821ef070ba546</URIString>
        <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Implemented attributes
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
        <label>Implemented attributes</label>
    </child>
    <child ID="1887" label="Entity" layerID="1" created="1288257108731"
        x="8709.4795" y="609.0019" width="133.5" height="142.7019"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/f21c2a7a7f0001010d9438de5c6ad321</URIString>
        <child ID="1906" label="tags | MANY | TAG"
            created="1288257253087" x="34.0" y="23.0" width="108.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/f220e0137f0001010d9438de0e9e63e0</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2439" label="notes | MANY | NOTE"
            created="1294846706622" x="34.0" y="43.25" width="125.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/7ae32d587f0001017115e16621f44fe5</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4149" created="1369384391227" x="34.0" y="63.5"
            width="30.333334" height="25.602621" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#FEFB03</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/d5acac4f7b6a9fda5a40fe82e49025e5</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <child ID="4042" created="1358111975811" x="34.0" y="85.701965"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#30D643</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35c8aee0c0a8000435fa837959e2d05e</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <child ID="4217" created="1468653304659" x="34.0" y="103.70194"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#33A8F5</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/f28ffbb4c0a8000f011bc52ff34ef0a9</URIString>
            <shape xsi:type="chevron"/>
        </child>
        <child ID="4218" created="1468653307926" x="34.0" y="121.70192"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#AF55F4</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/f28ffbb4c0a8000f011bc52f65bedc44</URIString>
            <shape xsi:type="rhombus"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="1901" label="SimpleEntity" layerID="1"
        created="1288257253086" x="8702.232" y="141.87709" width="202.5"
        height="345.20187" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/f220e0127f0001010d9438de61473dd2</URIString>
        <child ID="2263" label="name | UNICODE" created="1292744446733"
            x="34.0" y="23.0" width="108.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/ff73db697f0001015481be8a5451779a</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2264" label="description | UNICODE"
            created="1292744446733" x="34.0" y="43.25" width="137.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/ff73db697f0001015481be8af558ee50</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1888" label="created_by | ONE | USER"
            created="1288257108731" x="34.0" y="63.5" width="149.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/f21c2a7a7f0001010d9438de2db62acf</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1889" label="updated_by | ONE | USER"
            created="1288257108731" x="34.0" y="83.75" width="153.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/f21c2a7a7f0001010d9438de885e42f6</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1890" label="date_created | DATETIME"
            created="1288257108732" x="34.0" y="104.0" width="150.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/f21c2a7a7f0001010d9438de584fbe77</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1891" label="date_updated | DATETIME"
            created="1288257108732" x="34.0" y="124.25" width="154.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/f21c2a7a7f0001010d9438dead15905c</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2458" label="nice_name | UNICODE"
            created="1294925510351" x="34.0" y="144.5" width="138.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/7f93720e7f0001017115e1664dc590db</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="1128" label="thumbnail | ONE | LINK"
            created="1272404328842" x="34.0" y="164.75" width="137.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/4137afd27f00010135ea5711ec68f281</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3632" label="type | ONE | TYPE"
            created="1306191718656" x="34.0" y="185.0" width="107.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/1f190a3a7f000101578cef1800e852f8</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3674" label="__stalker_version__ | STRING"
            created="1307851983782" x="34.0" y="205.25" width="177.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/820dc20f7f0001015d3258773963db05</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3478" label="generic_data | MANY | SIMPLEENTITY"
            created="1296077941615" x="34.0" y="225.5" width="217.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/c44466c67f0001013611c755f271acd4</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4272" label="generic_text | STRING"
            created="1471423657470" x="34.0" y="245.75" width="131.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/97afe790c0a82a4a0108177921807cf8</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4151" created="1369384405691" x="34.0" y="266.0"
            width="30.333334" height="25.602621" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#FEFB03</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/d5acac627b6a9fda5a40fe822a456e6a</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <child ID="4021" created="1358111834001" x="34.0" y="288.20197"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#30D643</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35c8aee2c0a8000435fa8379eb933d7b</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <child ID="4211" created="1468653281990" x="34.0" y="306.20193"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#33A8F5</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/f28ffbb5c0a8000f011bc52f94cd3e52</URIString>
            <shape xsi:type="chevron"/>
        </child>
        <child ID="4212" created="1468653285082" x="34.0" y="324.2019"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#AF55F4</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/f28ffbb5c0a8000f011bc52f9eb42d35</URIString>
            <shape xsi:type="rhombus"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="1912" layerID="1" created="1288257599993" x="8214.804"
        y="751.25" width="562.0869" height="77.44531" strokeWidth="1.0"
        autoSized="false" controlCount="2" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/f22534227f0001010d9438dee0d42673</URIString>
        <point1 x="8776.391" y="751.75"/>
        <point2 x="8215.304" y="828.1953"/>
        <ID1 xsi:type="node">1887</ID1>
        <ID2 xsi:type="node">1396</ID2>
        <ctrlPoint0 x="8776.48" y="791.8681" xsi:type="point"/>
        <ctrlPoint1 x="8219.904" y="759.04425" xsi:type="point"/>
    </child>
    <child ID="1913" layerID="1" created="1288257601892" x="8382.308"
        y="750.5" width="394.51367" height="78.203125" strokeWidth="1.0"
        autoSized="false" controlCount="2" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/f22534227f0001010d9438dea8c2b1fc</URIString>
        <point1 x="8776.321" y="751.0"/>
        <point2 x="8382.808" y="828.2031"/>
        <ID1 xsi:type="node">1887</ID1>
        <ID2 xsi:type="node">1355</ID2>
        <ctrlPoint0 x="8776.375" y="792.0636" xsi:type="point"/>
        <ctrlPoint1 x="8391.725" y="753.8007" xsi:type="point"/>
    </child>
    <child ID="1915" layerID="1" created="1288257624543" x="8573.61"
        y="751.5" width="203.1582" height="78.0" strokeWidth="1.0"
        autoSized="false" controlCount="2" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/f22534227f0001010d9438de64236ba0</URIString>
        <point1 x="8776.269" y="752.0"/>
        <point2 x="8574.11" y="829.0"/>
        <ID1 xsi:type="node">1887</ID1>
        <ID2 xsi:type="node">1465</ID2>
        <ctrlPoint0 x="8776.29" y="792.0413" xsi:type="point"/>
        <ctrlPoint1 x="8573.89" y="750.2044" xsi:type="point"/>
    </child>
    <child ID="1918" layerID="1" created="1288257690703" x="8775.942"
        y="751.25" width="1525.5938" height="77.5" strokeWidth="1.0"
        autoSized="false" controlCount="2" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/f22534237f0001010d9438dede87b33c</URIString>
        <point1 x="8776.442" y="751.75"/>
        <point2 x="10301.036" y="828.25"/>
        <ID1 xsi:type="node">1887</ID1>
        <ID2 xsi:type="node">1299</ID2>
        <ctrlPoint0 x="8776.561" y="791.71063" xsi:type="point"/>
        <ctrlPoint1 x="10300.243" y="717.47723" xsi:type="point"/>
    </child>
    <child ID="1919" layerID="1" created="1288257693210" x="8776.023"
        y="751.25" width="906.33496" height="77.25" strokeWidth="1.0"
        autoSized="false" controlCount="2" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/f22534237f0001010d9438de69c8feac</URIString>
        <point1 x="8776.523" y="751.75"/>
        <point2 x="9681.858" y="828.0"/>
        <ID1 xsi:type="node">1887</ID1>
        <ID2 xsi:type="node">1145</ID2>
        <ctrlPoint0 x="8776.689" y="792.14606" xsi:type="point"/>
        <ctrlPoint1 x="9681.819" y="734.10425" xsi:type="point"/>
    </child>
    <child ID="1933" layerID="1" created="1288268055042" x="8776.892"
        y="751.1875" width="496.18262" height="763.53125"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/f2c6c0257f0001010d9438de6add753c</URIString>
        <point1 x="8777.392" y="751.6875"/>
        <point2 x="9272.574" y="1514.2188"/>
        <ID1 xsi:type="node">1887</ID1>
        <ID2 xsi:type="node">1227</ID2>
        <ctrlPoint0 x="8786.786" y="1327.7544" xsi:type="point"/>
        <ctrlPoint1 x="9268.255" y="1409.1014" xsi:type="point"/>
    </child>
    <child ID="1935" layerID="1" created="1288268059310" x="8776.339"
        y="751.1875" width="336.83008" height="763.3125"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/f2c6c0257f0001010d9438de4ef74d59</URIString>
        <point1 x="8776.839" y="751.6875"/>
        <point2 x="9112.669" y="1514.0"/>
        <ID1 xsi:type="node">1887</ID1>
        <ID2 xsi:type="node">1157</ID2>
        <ctrlPoint0 x="8781.626" y="1311.9343" xsi:type="point"/>
        <ctrlPoint1 x="9112.833" y="1388.2299" xsi:type="point"/>
    </child>
    <child ID="1940" label="Package" layerID="1" created="1288424104620"
        x="9427.72" y="1858.0773" width="162.0" height="66.5"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#EA2218</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/fc1178ae7f00010120d48785675aa223</URIString>
        <child ID="1945" label="asset | MANY | ASSETBASE"
            created="1288424104621" x="34.0" y="23.0" width="163.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/fc1178af7f00010120d48785088a5d16</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2507" created="1295294868654" x="34.0" y="43.25"
            width="29.0" height="23.0" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/9597b7857f0001015ee819a81ccbfc97</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="1950" layerID="1" created="1288424120074" x="8774.397"
        y="751.21875" width="732.05273" height="1107.3594"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/fc1178b07f00010120d4878508d5a5da</URIString>
        <point1 x="8774.897" y="751.71875"/>
        <point2 x="9505.95" y="1858.0781"/>
        <ID1 xsi:type="node">1887</ID1>
        <ID2 xsi:type="node">1940</ID2>
        <ctrlPoint0 x="8754.7" y="1834.1465" xsi:type="point"/>
        <ctrlPoint1 x="9500.001" y="1786.6385" xsi:type="point"/>
    </child>
    <child ID="1957" layerID="1" created="1288799345809" x="8775.977"
        y="751.25" width="524.1719" height="77.46875" strokeWidth="1.0"
        autoSized="false" controlCount="2" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/126dc1f67f0001014196c169c59591bf</URIString>
        <point1 x="8776.477" y="751.75"/>
        <point2 x="9299.648" y="828.21875"/>
        <ID1 xsi:type="node">1887</ID1>
        <ID2 xsi:type="node">2180</ID2>
        <ctrlPoint0 x="8776.615" y="792.01715" xsi:type="point"/>
        <ctrlPoint1 x="9303.951" y="766.1328" xsi:type="point"/>
    </child>
    <child ID="1995" label="Delivery" layerID="1"
        created="1289688586892" x="9632.252" y="1508.0773" width="123.0"
        height="86.75" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#ECFFD4</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/476ff9f37f0001015a3184d866b3d9f0</URIString>
        <child ID="3466" label="StatusMixin" created="1296050026720"
            x="34.0" y="23.0" width="78.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/c2bbcf617f0001013611c75512d46e2b</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2001" label="files | MANY | LINK"
            created="1289688586898" x="34.0" y="43.25" width="111.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/476ff9f47f0001015a3184d89f755940</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2508" created="1295294872623" x="34.0" y="63.5"
            width="29.0" height="23.0" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/9597b7867f0001015ee819a85ff2e993</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2005" layerID="1" created="1289688647878" x="8775.092"
        y="751.1875" width="921.4961" height="757.3906"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/476ff9f57f0001015a3184d86becdbe1</URIString>
        <point1 x="8775.592" y="751.6875"/>
        <point2 x="9696.088" y="1508.0781"/>
        <ID1 xsi:type="node">1887</ID1>
        <ID2 xsi:type="node">1995</ID2>
        <ctrlPoint0 x="8769.07" y="1480.4794" xsi:type="point"/>
        <ctrlPoint1 x="9701.445" y="1408.6132" xsi:type="point"/>
    </child>
    <child ID="2012" label="Structure" layerID="1"
        created="1290373811327" x="8976.72" y="828.225" width="216.0"
        height="124.70192" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/704adc107f0001012ed4f13aa93265ac</URIString>
        <child ID="2017" label="templates | MANY | FILENAMETEMPLATE"
            created="1290373958896" x="34.0" y="23.0" width="235.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/704adc117f0001012ed4f13ae0ef0e20</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3630" label="custom_template | STRING"
            created="1306190987002" x="34.0" y="43.25" width="161.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/1f0d0cc37f000101578cef18dc05580c</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4147" created="1369384371710" x="34.0" y="63.5"
            width="30.333334" height="25.602621" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#FEFB03</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/d5acac8a7b6a9fda5a40fe827c2c1502</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <child ID="3790" created="1310511196916" x="34.0" y="85.701965"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#AF55F4</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/208df7367f000101264d107e7f8218a8</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <child ID="4030" created="1358111879155" x="34.0" y="103.70194"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35c8aee6c0a8000435fa8379abbe7bb8</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="2016" layerID="1" created="1290373841415" x="8775.822"
        y="751.5" width="309.38184" height="77.0" strokeWidth="1.0"
        autoSized="false" controlCount="2" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/704adc147f0001012ed4f13a8b835d41</URIString>
        <point1 x="8776.322" y="752.0"/>
        <point2 x="9084.704" y="828.0"/>
        <ID1 xsi:type="node">1887</ID1>
        <ID2 xsi:type="node">2012</ID2>
        <ctrlPoint0 x="8776.375" y="791.803" xsi:type="point"/>
        <ctrlPoint1 x="9084.689" y="766.74225" xsi:type="point"/>
    </child>
    <child ID="2026" label="PROBLEM" layerID="1" created="1290415153137"
        x="12753.83" y="580.21045" width="215.0" height="92.0"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FCDBD9</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72c7681c7f0001010b45ad3d65da5015</URIString>
        <child ID="2025"
            label="Where to save the templates, or more clearly, where to save the data &#xa;      that holds which file of which pipeline_step should saved where..."
            created="1290379453440" x="5.0" y="23.0" width="205.0"
            height="63.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
            <strokeColor>#404040</strokeColor>
            <textColor>#000000</textColor>
            <font>SansSerif-plain-14</font>
            <URIString>http://vue.tufts.edu/rdf/resource/709d95a57f0001012ed4f13a4667f382</URIString>
            <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Where to save the templates, or more clearly, where to save the data 
      that holds which file of which pipeline_step should saved where...
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
            <label>Where to save the templates, or more clearly, where to save the data 
      that holds which file of which pipeline_step should saved where...</label>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2027" label="Hold in AssetType" layerID="1"
        created="1290415471563" x="12585.413" y="825.87695"
        width="160.0" height="107.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72c7681e7f0001010b45ad3dbd6e27e5</URIString>
        <child ID="2029"
            label="Asset Type holds a lot of things, so may be the asset type should hold the data that shows where to save a specific type"
            created="1290415523885" x="5.0" y="23.0" width="150.0"
            height="78.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
            <strokeColor>#404040</strokeColor>
            <textColor>#000000</textColor>
            <font>SansSerif-plain-14</font>
            <URIString>http://vue.tufts.edu/rdf/resource/72c7681f7f0001010b45ad3d77bc2d11</URIString>
            <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Asset Type holds a lot of things, so may be the asset type should hold 
      the
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      data that shows where to save a specific type
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
            <label>Asset Type holds a lot of things, so may be the asset type should hold the data that shows where to save a specific type</label>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2028" label="SOLUTION 1" layerID="1"
        created="1290415471574" x="12706.315" y="671.71094"
        width="119.916016" height="154.66602" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72c768217f0001010b45ad3d27ead190</URIString>
        <point1 x="12825.731" y="672.21094"/>
        <point2 x="12706.815" y="825.87695"/>
        <ID1 xsi:type="node">2026</ID1>
        <ID2 xsi:type="node">2027</ID2>
    </child>
    <child ID="2031" label="limits flexibility" layerID="1"
        created="1290415567637" x="12585.413" y="995.37695"
        width="160.0" height="107.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72c768227f0001010b45ad3de419a2bb</URIString>
        <child ID="2033"
            label="This leads a weird setup where an object has to be saved to the same &#xa;      place in every project, this limits the flexibility we aim"
            created="1290415575363" x="5.0" y="23.0" width="150.0"
            height="78.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
            <strokeColor>#404040</strokeColor>
            <textColor>#000000</textColor>
            <font>SansSerif-plain-14</font>
            <URIString>http://vue.tufts.edu/rdf/resource/72c768237f0001010b45ad3ddd5305d2</URIString>
            <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      This leads a weird setup where an object has to be saved to the same 
      place in every project, this limits the flexibility we aim
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
            <label>This leads a weird setup where an object has to be saved to the same 
      place in every project, this limits the flexibility we aim</label>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2032" layerID="1" created="1290415567645" x="12664.913"
        y="932.37695" width="1.0" height="63.5" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72c768247f0001010b45ad3dedb39d1c</URIString>
        <point1 x="12665.413" y="932.87695"/>
        <point2 x="12665.413" y="995.37695"/>
        <ID1 xsi:type="node">2027</ID1>
        <ID2 xsi:type="node">2031</ID2>
    </child>
    <child ID="2035" label="Hold in Project" layerID="1"
        created="1290415693380" x="12901.163" y="835.37695"
        width="153.0" height="77.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72c768257f0001010b45ad3d2cd5572c</URIString>
        <child ID="2037"
            label="hold the data in project node, so any project can have different setups"
            created="1290415702905" x="5.0" y="23.0" width="143.0"
            height="48.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
            <strokeColor>#404040</strokeColor>
            <textColor>#000000</textColor>
            <font>SansSerif-plain-14</font>
            <URIString>http://vue.tufts.edu/rdf/resource/72c768267f0001010b45ad3d7b74ae69</URIString>
            <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      hold the data in project node, so any project can have different setups
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
            <label>hold the data in project node, so any project can have different setups</label>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2036" label="SOLUTION 2" layerID="1"
        created="1290415693397" x="12882.4375" y="671.71094"
        width="77.6416" height="164.16602" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72c768287f0001010b45ad3ddc9a3886</URIString>
        <point1 x="12882.9375" y="672.21094"/>
        <point2 x="12959.579" y="835.37695"/>
        <ID1 xsi:type="node">2026</ID1>
        <ID2 xsi:type="node">2035</ID2>
    </child>
    <child ID="2038" label="special settings for every node" layerID="1"
        created="1290415727634" x="12884.163" y="971.37695"
        width="187.0" height="152.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72c768297f0001010b45ad3d2ac3c4b6</URIString>
        <child ID="2040"
            label="this needs to have an entry for every asset type, so if I'm going to &#xa;      hold a template for assets I need to specify that this template is for assets, and if I want to have a template for shots, I need to specify that this template is for shots"
            created="1290415748194" x="5.0" y="23.0" width="177.0"
            height="123.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
            <strokeColor>#404040</strokeColor>
            <textColor>#000000</textColor>
            <font>SansSerif-plain-14</font>
            <URIString>http://vue.tufts.edu/rdf/resource/72c768297f0001010b45ad3d14b50bfc</URIString>
            <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      this needs to have an entry for every asset type, so if I'm going to 
      hold a template for assets I need to specify that this template is for 
      assets, and if I want to have a template for shots, I need to specify 
      that this template is for shots
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
            <label>this needs to have an entry for every asset type, so if I'm going to 
      hold a template for assets I need to specify that this template is for assets, and if I want to have a template for shots, I need to specify that this template is for shots</label>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2039" layerID="1" created="1290415727642" x="12977.163"
        y="911.87695" width="1.0" height="60.0" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72c7682b7f0001010b45ad3d805a8614</URIString>
        <point1 x="12977.663" y="912.37695"/>
        <point2 x="12977.663" y="971.37695"/>
        <ID1 xsi:type="node">2035</ID1>
        <ID2 xsi:type="node">2038</ID2>
    </child>
    <child ID="2043" label="REASONABLE" layerID="1"
        created="1290415848293" x="12933.163" y="1184.377" width="89.0"
        height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#30D643</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72c7682c7f0001010b45ad3d801617fc</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2044" layerID="1" created="1290415848302" x="12977.163"
        y="1122.877" width="1.0" height="62.0" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72c7682d7f0001010b45ad3dc2a36449</URIString>
        <point1 x="12977.663" y="1123.377"/>
        <point2 x="12977.663" y="1184.377"/>
        <ID1 xsi:type="node">2038</ID1>
        <ID2 xsi:type="node">2043</ID2>
    </child>
    <child ID="2045" label="BAD" layerID="1" created="1290415882271"
        x="12627.413" y="1189.377" width="76.0" height="27.0"
        strokeWidth="1.0" autoSized="false" xsi:type="node">
        <fillColor>#EA2218</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72c8b39c7f0001010b45ad3d3de72c20</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2046" layerID="1" created="1290415882275" x="12664.913"
        y="1101.877" width="1.0" height="88.0" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72c8b39d7f0001010b45ad3dbf4d4a87</URIString>
        <point1 x="12665.413" y="1102.377"/>
        <point2 x="12665.413" y="1189.377"/>
        <ID1 xsi:type="node">2031</ID1>
        <ID2 xsi:type="node">2045</ID2>
    </child>
    <child ID="2048" label="how to connect" layerID="1"
        created="1290415915729" x="12897.663" y="1259.377" width="160.0"
        height="60.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72c8b39e7f0001010b45ad3dce9e9452</URIString>
        <child ID="2050" label="how to connect the data, in this case"
            created="1290415921232" x="5.0" y="23.0" width="150.0"
            height="31.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
            <strokeColor>#404040</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/72c8b39e7f0001010b45ad3d443056e8</URIString>
            <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 11; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 11 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 11 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      &lt;font style="font-size:12;"&gt;how to connect the data, in this case&lt;/font&gt;
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
            <label>how to connect the data, in this case</label>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2049" layerID="1" created="1290415915733" x="12977.163"
        y="1206.877" width="1.0" height="53.0" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72c8b3a07f0001010b45ad3d861cccb5</URIString>
        <point1 x="12977.663" y="1207.377"/>
        <point2 x="12977.663" y="1259.377"/>
        <ID1 xsi:type="node">2043</ID1>
        <ID2 xsi:type="node">2048</ID2>
    </child>
    <child ID="2051" label="AssetType" layerID="1"
        created="1290415957287" x="12786.163" y="1403.377" width="169.0"
        height="92.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72ccceac7f0001010b45ad3deca8e4e1</URIString>
        <child ID="2054"
            label="so for every assetType given to the system, the user can specify the &#xa;      default location to save"
            created="1290415993540" x="5.0" y="23.0" width="159.0"
            height="63.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
            <strokeColor>#404040</strokeColor>
            <textColor>#000000</textColor>
            <font>SansSerif-plain-14</font>
            <URIString>http://vue.tufts.edu/rdf/resource/72cccead7f0001010b45ad3d050b14ca</URIString>
            <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      so for every assetType given to the system, the user can specify the 
      default location to save
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
            <label>so for every assetType given to the system, the user can specify the 
      default location to save</label>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2052" layerID="1" created="1290415957292" x="12900.926"
        y="1318.877" width="57.174805" height="85.0" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72ccceaf7f0001010b45ad3d72cbe1eb</URIString>
        <point1 x="12957.601" y="1319.377"/>
        <point2 x="12901.426" y="1403.377"/>
        <ID1 xsi:type="node">2048</ID1>
        <ID2 xsi:type="node">2051</ID2>
    </child>
    <child ID="2057" label="PipelineStep" layerID="1"
        created="1290416080796" x="13016.663" y="1396.377" width="160.0"
        height="158.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72ccceb07f0001010b45ad3d6a47dcac</URIString>
        <child ID="2059"
            label="the files are produced in pipeline steps, for a &quot;Character&quot; assetType, &#xa;      the &quot;Modeling&quot; pipelne step produces the file, so we need three data to hold, one the assetType, second the pipeline step, and third the template string"
            created="1290416089913" x="5.0" y="23.0" width="150.0"
            height="129.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
            <strokeColor>#404040</strokeColor>
            <textColor>#000000</textColor>
            <font>SansSerif-plain-14</font>
            <URIString>http://vue.tufts.edu/rdf/resource/72ccceb07f0001010b45ad3d80acd543</URIString>
            <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 11; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 11 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 11 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      the files are produced in pipeline steps, for a &amp;quot;Character&amp;quot; assetType, 
      the &amp;quot;Modeling&amp;quot; pipelne step produces the file, so we need three data to 
      hold, one the assetType, second the pipeline step, and third the 
      template string
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
            <label>the files are produced in pipeline steps, for a "Character" assetType, 
      the "Modeling" pipelne step produces the file, so we need three data to hold, one the assetType, second the pipeline step, and third the template string</label>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2058" layerID="1" created="1290416080803" x="12996.357"
        y="1318.877" width="50.262695" height="78.0" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72ccceb27f0001010b45ad3d93aaab40</URIString>
        <point1 x="12996.856" y="1319.377"/>
        <point2 x="13046.119" y="1396.377"/>
        <ID1 xsi:type="node">2048</ID1>
        <ID2 xsi:type="node">2057</ID2>
    </child>
    <child ID="2060" label="Idea 2" layerID="1" created="1290416289541"
        x="12923.163" y="1621.377" width="160.0" height="107.0"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FC938D</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72d4e8727f0001010b45ad3d6534da51</URIString>
        <child ID="2062"
            label="Create a new composite foreign key, that holds a key to one asset type &#xa;      object and a pipeline step, and has a string template"
            created="1290416301932" x="5.0" y="23.0" width="150.0"
            height="78.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
            <strokeColor>#404040</strokeColor>
            <textColor>#000000</textColor>
            <font>SansSerif-plain-14</font>
            <URIString>http://vue.tufts.edu/rdf/resource/72d4e8727f0001010b45ad3d7e0392c9</URIString>
            <richText>&lt;html&gt;
  &lt;head&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Create a new composite foreign key, that holds a key to one asset type 
      object and a pipeline step, and has a string template
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
            <label>Create a new composite foreign key, that holds a key to one asset type 
      object and a pipeline step, and has a string template</label>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2061" layerID="1" created="1290416289545" x="13027.737"
        y="1553.877" width="32.40039" height="68.0" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72d4e8737f0001010b45ad3d19b9fc7e</URIString>
        <point1 x="13059.638" y="1554.377"/>
        <point2 x="13028.237" y="1621.377"/>
        <ID1 xsi:type="node">2057</ID1>
        <ID2 xsi:type="node">2060</ID2>
    </child>
    <child ID="2063" label="Structure" layerID="1"
        created="1290416676242" x="12942.163" y="1783.377" width="124.0"
        height="62.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#30D643</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72d4e8737f0001010b45ad3d683e5230</URIString>
        <child ID="2065" label="this can work with the structure system"
            created="1290416682757" x="5.0" y="23.0" width="114.0"
            height="33.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
            <strokeColor>#404040</strokeColor>
            <textColor>#000000</textColor>
            <font>SansSerif-plain-14</font>
            <URIString>http://vue.tufts.edu/rdf/resource/72d4e8737f0001010b45ad3db5912744</URIString>
            <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      this can work with the structure system
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
            <label>this can work with the structure system</label>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2064" layerID="1" created="1290416676246" x="13003.047"
        y="1727.875" width="1.3945312" height="56.0" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72d4e8747f0001010b45ad3d2421dadd</URIString>
        <point1 x="13003.547" y="1728.375"/>
        <point2 x="13003.941" y="1783.375"/>
        <ID1 xsi:type="node">2060</ID1>
        <ID2 xsi:type="node">2063</ID2>
    </child>
    <child ID="2066" layerID="1" created="1290416717375" x="12897.192"
        y="1494.877" width="75.03516" height="127.0" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72d4e8747f0001010b45ad3dfa561263</URIString>
        <point1 x="12897.692" y="1495.377"/>
        <point2 x="12971.728" y="1621.377"/>
        <ID1 xsi:type="node">2051</ID1>
        <ID2 xsi:type="node">2060</ID2>
    </child>
    <child ID="2067" label="idea 1" layerID="1" created="1290416821416"
        x="12698.252" y="1588.377" width="160.0" height="62.0"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FC938D</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72d6ca677f0001010b45ad3d78d6eb10</URIString>
        <child ID="2069"
            label="use assetType side by side with a template"
            created="1290416826833" x="5.0" y="23.0" width="150.0"
            height="33.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
            <strokeColor>#404040</strokeColor>
            <textColor>#000000</textColor>
            <font>SansSerif-plain-14</font>
            <URIString>http://vue.tufts.edu/rdf/resource/72d6ca687f0001010b45ad3d900a2d2c</URIString>
            <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 11; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 11 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 11 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      &lt;font style="font-size:12;"&gt;use assetType side by side with a template&lt;/font&gt;
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
            <label>use assetType side by side with a template</label>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2068" layerID="1" created="1290416821420" x="12794.604"
        y="1494.877" width="51.554688" height="94.0" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72d6ca687f0001010b45ad3dde441d00</URIString>
        <point1 x="12845.658" y="1495.377"/>
        <point2 x="12795.104" y="1588.377"/>
        <ID1 xsi:type="node">2051</ID1>
        <ID2 xsi:type="node">2067</ID2>
    </child>
    <child ID="2075" label="BIGGER QUESTION" layerID="1"
        created="1290417771427" x="13785.086" y="569.37695"
        width="160.0" height="92.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FCDBD9</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72e63ba77f0001010b45ad3d2724344a</URIString>
        <child ID="2077"
            label="Should I have a template for any entity those have a connection with the &#xa;      file system"
            created="1290417822261" x="5.0" y="23.0" width="150.0"
            height="63.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
            <strokeColor>#404040</strokeColor>
            <textColor>#000000</textColor>
            <font>SansSerif-plain-14</font>
            <URIString>http://vue.tufts.edu/rdf/resource/72e63ba87f0001010b45ad3dc2870b40</URIString>
            <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Should I have a template for any entity those have a connection with the 
      file system
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
            <label>Should I have a template for any entity those have a connection with the 
      file system</label>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2080" label="For Example?" layerID="1"
        created="1290417984688" x="13820.586" y="716.37695" width="88.0"
        height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72f113a57f0001010b45ad3d8a1153d0</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2081" layerID="1" created="1290417984692" x="13864.135"
        y="661.0" width="1.2451172" height="56.0" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72f113a57f0001010b45ad3d490f63de</URIString>
        <point1 x="13864.881" y="661.5"/>
        <point2 x="13864.636" y="716.5"/>
        <ID1 xsi:type="node">2075</ID1>
        <ID2 xsi:type="node">2080</ID2>
    </child>
    <child ID="2082" label="where to save the a version of a task ?"
        layerID="1" created="1290418001363" x="13641.586" y="794.37695"
        width="247.0" height="51.5" strokeWidth="1.0" autoSized="false" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72f113a67f0001010b45ad3d49b4e257</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2083" layerID="1" created="1290418001367" x="13792.359"
        y="738.87695" width="60.322266" height="56.0" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72f113a67f0001010b45ad3ddebb1a70</URIString>
        <point1 x="13852.182" y="739.37695"/>
        <point2 x="13792.859" y="794.37695"/>
        <ID1 xsi:type="node">2080</ID1>
        <ID2 xsi:type="node">2082</ID2>
    </child>
    <child ID="2084" label="does it need to have a template ?"
        layerID="1" created="1290418016558" x="13653.086" y="905.0198"
        width="220.0" height="55.5" strokeWidth="1.0" autoSized="false" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72f113a67f0001010b45ad3d95cb489f</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2085" layerID="1" created="1290418016563" x="13763.078"
        y="845.375" width="2.0498047" height="60.125" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72f113a67f0001010b45ad3dc1ae5313</URIString>
        <point1 x="13764.629" y="845.875"/>
        <point2 x="13763.579" y="905.0"/>
        <ID1 xsi:type="node">2082</ID1>
        <ID2 xsi:type="node">2084</ID2>
    </child>
    <child ID="2086" label="where&#xa;to create&#xa;a sequence folder"
        layerID="1" created="1290418067315" x="13985.086" y="798.37695"
        width="110.0" height="53.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72f113a77f0001010b45ad3d3f31f2fc</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2087" layerID="1" created="1290418067319" x="13884.893"
        y="738.87695" width="108.53711" height="60.436523"
        strokeWidth="1.0" autoSized="false" controlCount="0"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72f113a77f0001010b45ad3d06e536cd</URIString>
        <point1 x="13885.393" y="739.37695"/>
        <point2 x="13992.93" y="798.8135"/>
        <ID1 xsi:type="node">2080</ID1>
        <ID2 xsi:type="node">2086</ID2>
    </child>
    <child ID="2088"
        label="sequences are&#xa;dummy folders like projects"
        layerID="1" created="1290418090858" x="13952.586" y="892.62695"
        width="190.0" height="58.5" strokeWidth="1.0" autoSized="false" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72f113a77f0001010b45ad3d2df9b014</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2089" layerID="1" created="1290418090865" x="14041.635"
        y="850.875" width="4.1904297" height="42.25" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72f113a87f0001010b45ad3d2466129e</URIString>
        <point1 x="14042.135" y="851.375"/>
        <point2 x="14045.325" y="892.625"/>
        <ID1 xsi:type="node">2086</ID1>
        <ID2 xsi:type="node">2088</ID2>
    </child>
    <child ID="2091"
        label="thus they are hidden in the&#xa;template code"
        layerID="1" created="1290418155937" x="13958.836" y="992.37695"
        width="183.0" height="65.5" strokeWidth="1.0" autoSized="false" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72f113a87f0001010b45ad3d3c38709f</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2092" layerID="1" created="1290418155942" x="14047.865"
        y="950.625" width="2.0986328" height="42.25" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72f113a87f0001010b45ad3d908c11e0</URIString>
        <point1 x="14048.364" y="951.125"/>
        <point2 x="14049.463" y="992.375"/>
        <ID1 xsi:type="node">2088</ID1>
        <ID2 xsi:type="node">2091</ID2>
    </child>
    <child ID="2095"
        label="in fact it has, the whole idea&#xa;behind project.template is to&#xa;define the place to save a&#xa;version of a pipeline step"
        layerID="1" created="1290418232979" x="13675.086" y="1019.6626"
        width="166.0" height="68.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72f113a97f0001010b45ad3d6fed6405</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2096" layerID="1" created="1290418232984" x="13758.993"
        y="960.03125" width="3.4453125" height="60.125"
        strokeWidth="1.0" autoSized="false" controlCount="0"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72f113a97f0001010b45ad3d34de9e5c</URIString>
        <point1 x="13761.938" y="960.53125"/>
        <point2 x="13759.493" y="1019.65625"/>
        <ID1 xsi:type="node">2084</ID1>
        <ID2 xsi:type="node">2095</ID2>
    </child>
    <child ID="2100"
        label="how to pass the data to be used&#xa;inside templates"
        layerID="1" created="1290418291978" x="13665.086" y="1146.8054"
        width="186.0" height="38.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72f113a97f0001010b45ad3d6728aea3</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2101" layerID="1" created="1290418291983" x="13757.586"
        y="1087.1626" width="1.0" height="60.142822" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72f113a97f0001010b45ad3d3911b327</URIString>
        <point1 x="13758.086" y="1087.6626"/>
        <point2 x="13758.086" y="1146.8054"/>
        <ID1 xsi:type="node">2095</ID1>
        <ID2 xsi:type="node">2100</ID2>
    </child>
    <child ID="2102"
        label="templates are jinja2 template, they&#xa;need variables, so how can I pass the&#xa;neccesary data to the version"
        layerID="1" created="1290418304858" x="13649.086" y="1243.9482"
        width="218.0" height="53.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72f113aa7f0001010b45ad3d9af06b7f</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2103" layerID="1" created="1290418304863" x="13757.586"
        y="1184.3054" width="1.0" height="60.142822" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72f113aa7f0001010b45ad3dd1e49c4e</URIString>
        <point1 x="13758.086" y="1184.8054"/>
        <point2 x="13758.086" y="1243.9482"/>
        <ID1 xsi:type="node">2100</ID1>
        <ID2 xsi:type="node">2102</ID2>
    </child>
    <child ID="2104"
        label="the templates should&#xa;be using the version itself,&#xa;so a template for a version&#xa;should be like:&#xa;&#xa;{{version.project.name}}/ASSETS/{{version.asset.code}}/&#xa;{{version.asset_type.pipeline_step.code}}/&#xa;{{version.asset.code}}_{{version.asset.sub_name}}&#xa;&#xa;where the desired template should look like&#xa;&#xa;{{project.name}}/ASSET/{{asset.code}}/{{pipeline_step.code}}...&#xa;&#xa;which is more clear then the previous one"
        layerID="1" created="1290418353149" x="13581.586" y="1356.0911"
        width="353.0" height="218.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72f113aa7f0001010b45ad3d3e1507b7</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2105" layerID="1" created="1290418353153" x="13757.586"
        y="1296.4482" width="1.0" height="60.142822" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72f113ab7f0001010b45ad3d1cbe64b2</URIString>
        <point1 x="13758.086" y="1296.9482"/>
        <point2 x="13758.086" y="1356.0911"/>
        <ID1 xsi:type="node">2102</ID1>
        <ID2 xsi:type="node">2104</ID2>
    </child>
    <child ID="2109"
        label="the template can be given these variables:&#xa;&#xa;project&#xa;sequence&#xa;asset&#xa;assetType&#xa;task&#xa;version&#xa;&#xa;so a user can by default use this template&#xa;variables"
        layerID="1" created="1290419014194" x="13637.586" y="1633.2339"
        width="241.0" height="173.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72fbe81f7f0001010b45ad3d3a6d4e41</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2110" layerID="1" created="1290419014198" x="13757.586"
        y="1573.5911" width="1.0" height="60.142822" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72fbe8207f0001010b45ad3d2b6e222b</URIString>
        <point1 x="13758.086" y="1574.0911"/>
        <point2 x="13758.086" y="1633.2339"/>
        <ID1 xsi:type="node">2104</ID1>
        <ID2 xsi:type="node">2109</ID2>
    </child>
    <child ID="2111"
        label="so the answer to the question is&#xa;&#xa;NO&#xa;&#xa;we don't have to have a template&#xa;for any entity"
        layerID="1" created="1290419270418" x="13664.086" y="1865.3767"
        width="188.0" height="98.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72fbe8207f0001010b45ad3df22cfe6c</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2112" layerID="1" created="1290419270423" x="13757.586"
        y="1805.7339" width="1.0" height="60.142822" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/72fbe8207f0001010b45ad3d28033eea</URIString>
        <point1 x="13758.086" y="1806.2339"/>
        <point2 x="13758.086" y="1865.3767"/>
        <ID1 xsi:type="node">2109</ID1>
        <ID2 xsi:type="node">2111</ID2>
    </child>
    <child ID="2142" label="external references" layerID="1"
        created="1290428386234" x="15312.611" y="561.544" width="188.0"
        height="79.0" strokeWidth="1.0" autoSized="false" xsi:type="node">
        <fillColor>#FCDBD9</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/7389b2a27f0001010b45ad3d774a4589</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2143"
        label="the template of a reference&#xa;can be saved inside project"
        layerID="1" created="1290428397581" x="15206.822" y="938.24414"
        width="161.0" height="38.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#C1F780</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/7389b2a27f0001010b45ad3d4fa061ad</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2144" layerID="1" created="1290428397586" x="15300.024"
        y="880.34375" width="40.884766" height="58.40039"
        strokeWidth="1.0" autoSized="false" controlCount="0"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/7389b2a27f0001010b45ad3d045938fe</URIString>
        <point1 x="15340.409" y="880.84375"/>
        <point2 x="15300.524" y="938.24414"/>
        <ID1 xsi:type="node">2167</ID1>
        <ID2 xsi:type="node">2143</ID2>
    </child>
    <child ID="2145"
        label="need to have another data column&#xa;then the templates column,&#xa;because a template is like:"
        layerID="1" created="1290428439578" x="15174.822" y="1001.24365"
        width="225.0" height="171.875" strokeWidth="1.0"
        autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/7389b2a37f0001010b45ad3d2eb1871e</URIString>
        <child ID="2149" label="Template" created="1290428510175"
            x="34.0" y="53.0" width="236.0" height="150.5"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#ECFFD4</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/7389b2a37f0001010b45ad3d2ffed3ed</URIString>
            <child ID="2150" label="asset_type | ONE | ASSETTYPE"
                created="1290428510175" x="34.0" y="23.0" width="183.0"
                height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
                <fillColor>#FDE888</fillColor>
                <strokeColor>#776D6D</strokeColor>
                <textColor>#000000</textColor>
                <font>Arial-plain-12</font>
                <URIString>http://vue.tufts.edu/rdf/resource/7389b2a37f0001010b45ad3da34391da</URIString>
                <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
            </child>
            <child ID="2151" label="pipeline_step | ONE | PIPELENSTEP"
                created="1290428510176" x="34.0" y="42.5" width="212.0"
                height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
                <fillColor>#FDE888</fillColor>
                <strokeColor>#776D6D</strokeColor>
                <textColor>#000000</textColor>
                <font>Arial-plain-12</font>
                <URIString>http://vue.tufts.edu/rdf/resource/7389b2a37f0001010b45ad3d5d01bd82</URIString>
                <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
            </child>
            <child ID="2152" label="template_code | UNICODE"
                created="1290428510176" x="34.0" y="62.0" width="158.0"
                height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
                <fillColor>#FDE888</fillColor>
                <strokeColor>#776D6D</strokeColor>
                <textColor>#000000</textColor>
                <font>Arial-plain-12</font>
                <URIString>http://vue.tufts.edu/rdf/resource/7389b2a47f0001010b45ad3dcbb17974</URIString>
                <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
            </child>
            <child ID="2153"
                label="Examples: {{projects.root}}/{{project.name}} /SEQUENCES/{{sequence.name}} /SHOTS/{{shot.name}}"
                created="1290428510176" x="34.0" y="81.5" width="197.0"
                height="63.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
                <strokeColor>#404040</strokeColor>
                <textColor>#000000</textColor>
                <font>SansSerif-plain-14</font>
                <URIString>http://vue.tufts.edu/rdf/resource/7389b2a47f0001010b45ad3d91817f7f</URIString>
                <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Examples:
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      {{projects.root}}/{{project.name}}
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      /SEQUENCES/{{sequence.name}}
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      /SHOTS/{{shot.name}}
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
                <label>Examples: {{projects.root}}/{{project.name}} /SEQUENCES/{{sequence.name}} /SHOTS/{{shot.name}}</label>
            </child>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2146" layerID="1" created="1290428439582" x="15286.822"
        y="975.74414" width="1.0" height="25.999512" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/7389b2a57f0001010b45ad3de520dce3</URIString>
        <point1 x="15287.322" y="976.24414"/>
        <point2 x="15287.322" y="1001.24365"/>
        <ID1 xsi:type="node">2143</ID1>
        <ID2 xsi:type="node">2145</ID2>
    </child>
    <child ID="2147" label="too much specialization" layerID="1"
        created="1290428484654" x="15218.6" y="1259.4553" width="145.0"
        height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#EA2218</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/7389b2a57f0001010b45ad3dda5f0f57</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2154" label="Repository" layerID="1"
        created="1290429229074" x="7954.5513" y="828.2019"
        width="156.75" height="178.99997" strokeWidth="1.0"
        autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/7394cf4e7f0001010b45ad3d4505f265</URIString>
        <child ID="2192" label="linux_path | UNICODE"
            created="1290433909826" x="34.0" y="23.0" width="132.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/73db9f4c7f0001010b45ad3de656431d</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2193" label="osx_path | UNICODE"
            created="1290433929378" x="34.0" y="43.25" width="126.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/73db9f4c7f0001010b45ad3dbced640d</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2194" label="windows_path | UNICODE"
            created="1290433937695" x="34.0" y="63.5" width="156.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/73db9f4c7f0001010b45ad3d456aaae7</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2197"
            label="path -> one of the path&#xa;above, defined&#xa;according to the&#xa;current os"
            created="1290435168262" x="34.0" y="83.75" width="137.0"
            height="68.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/73ee6ed37f0001010b45ad3d6434d46f</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2475" created="1295294303904" x="34.0" y="137.75"
            width="29.0" height="23.0" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#FEFB03</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/958f53917f0001015ee819a8ab2b371f</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <child ID="4029" created="1358111873136" x="34.0" y="158.0"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35c8aef7c0a8000435fa837957f37360</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="2157" layerID="1" created="1290429260559" x="8044.364"
        y="751.0" width="732.4839" height="77.703125" strokeWidth="1.0"
        autoSized="false" controlCount="2" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/7394cf4e7f0001010b45ad3d9f393b4e</URIString>
        <point1 x="8776.348" y="751.5"/>
        <point2 x="8044.864" y="828.2031"/>
        <ID1 xsi:type="node">1887</ID1>
        <ID2 xsi:type="node">2154</ID2>
        <ctrlPoint0 x="8776.415" y="791.9108" xsi:type="point"/>
        <ctrlPoint1 x="8054.59" y="755.27936" xsi:type="point"/>
    </child>
    <child ID="2160"
        label="I want to be able to move the&#xa;localy selected files in to the&#xa;project"
        layerID="1" created="1290429671872" x="15270.611" y="684.44336"
        width="166.0" height="53.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/739c551d7f0001010b45ad3d8cc96286</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2161" layerID="1" created="1290429671877" x="15365.891"
        y="640.0449" width="22.170898" height="44.898438"
        strokeWidth="1.0" autoSized="false" controlCount="0"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/739c551d7f0001010b45ad3d7e48d57e</URIString>
        <point1 x="15387.5625" y="640.5449"/>
        <point2 x="15366.392" y="684.44336"/>
        <ID1 xsi:type="node">2142</ID1>
        <ID2 xsi:type="node">2160</ID2>
    </child>
    <child ID="2162"
        label="so I need templates&#xa;to tell me where to&#xa;put the reference files"
        layerID="1" created="1290429695965" x="15289.111" y="762.84326"
        width="129.0" height="53.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/739c551d7f0001010b45ad3d5a0a2c93</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2163" layerID="1" created="1290429695971" x="15353.111"
        y="736.94336" width="1.0" height="26.399902" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/739c551e7f0001010b45ad3d485244a7</URIString>
        <point1 x="15353.611" y="737.44336"/>
        <point2 x="15353.611" y="762.84326"/>
        <ID1 xsi:type="node">2160</ID1>
        <ID2 xsi:type="node">2162</ID2>
    </child>
    <child ID="2167" label="where to save&#xa;the template information"
        layerID="1" created="1290429724283" x="15281.611" y="842.84326"
        width="144.0" height="38.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/739c551e7f0001010b45ad3d4388adf9</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2168" layerID="1" created="1290429724287" x="15353.111"
        y="815.34326" width="1.0" height="28.0" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/739c551e7f0001010b45ad3d829e5aec</URIString>
        <point1 x="15353.611" y="815.84326"/>
        <point2 x="15353.611" y="842.84326"/>
        <ID1 xsi:type="node">2162</ID1>
        <ID2 xsi:type="node">2167</ID2>
    </child>
    <child ID="2169" label="add another reference_templates column"
        layerID="1" created="1290429902656" x="15172.3" y="1192.8125"
        width="238.0" height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/739e1da67f0001010b45ad3dba5f91ba</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2175" layerID="1" created="1290429913998" x="15289.74"
        y="1172.625" width="1.6689453" height="20.6875"
        strokeWidth="1.0" autoSized="false" controlCount="0"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/739e1da67f0001010b45ad3d81c3b80d</URIString>
        <point1 x="15290.24" y="1173.125"/>
        <point2 x="15290.909" y="1192.8125"/>
        <ID1 xsi:type="node">2145</ID1>
        <ID2 xsi:type="node">2169</ID2>
    </child>
    <child ID="2176" layerID="1" created="1290429916033" x="15290.635"
        y="1215.5" width="1.1298828" height="44.5" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/739e1da77f0001010b45ad3ddeae5d00</URIString>
        <point1 x="15291.265" y="1216.0"/>
        <point2 x="15291.135" y="1259.5"/>
        <ID1 xsi:type="node">2169</ID1>
        <ID2 xsi:type="node">2147</ID2>
    </child>
    <child ID="2177"
        label="template should be like this:&#xa;{{repository.path}}/{{project.code}}/REFS/{{entity.code}}/{{file.id}}_{{file.name}}"
        layerID="1" created="1290430092371" x="15522.985" y="702.04297"
        width="428.0" height="38.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/73a89ec47f0001010b45ad3d3340d8ba</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2178" layerID="1" created="1290430092376" x="15499.074"
        y="634.31006" width="186.10254" height="68.23291"
        strokeWidth="1.0" autoSized="false" controlCount="0"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/73a89ec57f0001010b45ad3d5be80605</URIString>
        <point1 x="15499.573" y="634.81006"/>
        <point2 x="15684.676" y="702.04297"/>
        <ID1 xsi:type="node">2142</ID1>
        <ID2 xsi:type="node">2177</ID2>
    </child>
    <child ID="2180" label="FilenameTemplate" layerID="1"
        created="1290430525012" x="9223.047" y="828.225" width="143.0"
        height="147.20193" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/73a89ec67f0001010b45ad3d49fb4e30</URIString>
        <child ID="3826" label="TargetEntityTypeMixin"
            created="1317078076240" x="34.0" y="23.0" width="136.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/a800f9087f00010130ce553694f16644</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2332" label="path | STRING" created="1293710778089"
            x="34.0" y="43.25" width="89.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/372d244c7f00010150882100fb860d9b</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2331" label="filename | STRING"
            created="1293710767892" x="34.0" y="63.5" width="113.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/372d244c7f000101508821007f113662</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3628" label="output_path | STRING"
            created="1306190609523" x="34.0" y="83.75" width="130.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/1f0762537f000101578cef187c1d8dbe</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4146" created="1369384364857" x="34.0" y="104.0"
            width="30.333334" height="25.602621" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#FEFB03</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/d5acad2e7b6a9fda5a40fe8247f8b756</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <child ID="4031" created="1358111882603" x="34.0" y="126.201965"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35c8aefac0a8000435fa837971185c26</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="2198" label="BUT!!!&#xa;REASONABLE ENOUGH" layerID="1"
        created="1290435474198" x="15221.573" y="1332.5925"
        width="140.0" height="38.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#30D643</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/73f32c657f0001010b45ad3de4631d2a</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2199" layerID="1" created="1290435474203" x="15290.668"
        y="1282.0" width="1.2939453" height="51.0" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/73f32c657f0001010b45ad3d5690194d</URIString>
        <point1 x="15291.168" y="1282.5"/>
        <point2 x="15291.462" y="1332.5"/>
        <ID1 xsi:type="node">2147</ID1>
        <ID2 xsi:type="node">2198</ID2>
    </child>
    <child ID="2203" label="Back -references" layerID="1"
        created="1290436660061" x="9222.637" y="130.2057" width="301.0"
        height="17.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
        <strokeColor>#404040</strokeColor>
        <textColor>#000000</textColor>
        <font>SansSerif-plain-14</font>
        <URIString>http://vue.tufts.edu/rdf/resource/740838427f0001010b45ad3d5e0a73f3</URIString>
        <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Back -references
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
        <label>Back -references</label>
    </child>
    <child ID="2205"
        label="Secondary attributes which are derived from current attributes and are not persistet in the database"
        layerID="1" created="1290436660061" x="9222.637" y="196.23404"
        width="300.0" height="32.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
        <strokeColor>#404040</strokeColor>
        <textColor>#000000</textColor>
        <font>SansSerif-plain-14</font>
        <URIString>http://vue.tufts.edu/rdf/resource/740838437f0001010b45ad3d8a55f990</URIString>
        <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Secondary attributes which are derived from current attributes and are 
      not persistet in the database
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
        <label>Secondary attributes which are derived from current attributes and are not persistet in the database</label>
    </child>
    <child ID="2208" label="Nodes those are not going to be implemented"
        layerID="1" created="1290436767093" x="9222.637" y="230.26205"
        width="301.0" height="24.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
        <strokeColor>#404040</strokeColor>
        <textColor>#000000</textColor>
        <font>SansSerif-plain-14</font>
        <URIString>http://vue.tufts.edu/rdf/resource/740838437f0001010b45ad3d5ec22072</URIString>
        <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Nodes those are not going to be implemented
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
        <label>Nodes those are not going to be implemented</label>
    </child>
    <child ID="1813" layerID="1" created="1282128089956" x="9187.644"
        y="-57.51773" width="29.0" height="23.0" strokeWidth="1.0"
        autoSized="false" xsi:type="node">
        <fillColor>#9DDB53</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/84ca7c987f00010138aea98153c19261</URIString>
        <shape xsi:type="ellipse"/>
    </child>
    <child ID="1818" layerID="1" created="1282128771399" x="9187.644"
        y="98.84404" width="29.0" height="23.0" strokeWidth="1.0"
        autoSized="false" xsi:type="node">
        <fillColor>#F2AE45</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/84d4ca357f00010138aea981c2362867</URIString>
        <shape xsi:type="ellipse"/>
    </child>
    <child ID="2202" layerID="1" created="1290436660061" x="9187.644"
        y="130.2057" width="29.0" height="23.0" strokeWidth="1.0"
        autoSized="false" xsi:type="node">
        <fillColor>#DAA9FF</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/740838427f0001010b45ad3d613481d3</URIString>
        <shape xsi:type="ellipse"/>
    </child>
    <child ID="2204" layerID="1" created="1290436660061" x="9187.644"
        y="196.23404" width="29.0" height="23.0" strokeWidth="1.0"
        autoSized="false" xsi:type="node">
        <fillColor>#83CEFF</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/740838437f0001010b45ad3d6fcb50ca</URIString>
        <shape xsi:type="ellipse"/>
    </child>
    <child ID="2209" layerID="1" created="1290436833398" x="9187.644"
        y="230.26205" width="29.0" height="23.0" strokeWidth="1.0"
        autoSized="false" xsi:type="node">
        <fillColor>#EA2218</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/740838447f0001010b45ad3df4e42c98</URIString>
        <shape xsi:type="ellipse"/>
    </child>
    <child ID="2214" label="STILL COOKING IDEAS" layerID="1"
        created="1290437542596" x="10976.716" y="-34.489746"
        width="305.667" height="102.66669" strokeWidth="1.0"
        autoSized="false" xsi:type="node">
        <fillColor>#B5B995</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-18</font>
        <URIString>http://vue.tufts.edu/rdf/resource/7412d2a47f0001010b45ad3d0fe8838d</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2224" label="Mapper" layerID="1" created="1292274937082"
        x="14691.515" y="-86.656494" width="80.0" height="44.0"
        strokeWidth="1.0" autoSized="false" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/e19ce5597f00010177026e406428bc52</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2225" label="Tables" layerID="1" created="1292274943148"
        x="14707.515" y="-178.6565" width="77.0" height="38.0"
        strokeWidth="1.0" autoSized="false" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/e19ce55a7f00010177026e405b6f7397</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2227" label="Python&#xa;Object Model" layerID="1"
        created="1292274951125" x="14586.515" y="-173.6565" width="82.0"
        height="38.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/e19ce55b7f00010177026e40538d6e0a</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2228" layerID="1" created="1292274951919" x="14648.971"
        y="-136.15625" width="57.62207" height="50.0" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/e19ce55c7f00010177026e404ac18180</URIString>
        <point1 x="14649.471" y="-135.65625"/>
        <point2 x="14706.093" y="-86.65625"/>
        <ID1 xsi:type="node">2227</ID1>
        <ID2 xsi:type="node">2224</ID2>
    </child>
    <child ID="2229" label="SessionMaker" layerID="1"
        created="1292274975500" x="14420.515" y="-221.6565"
        width="118.0" height="47.0" strokeWidth="1.0" autoSized="false" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/e19ce55d7f00010177026e40f5873189</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2230" label="engine" layerID="1" created="1292274983370"
        x="14580.715" y="-380.65637" width="77.0" height="39.0"
        strokeWidth="1.0" autoSized="false" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/e19ce55e7f00010177026e4085954d8d</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2231" layerID="1" created="1292275116284" x="14499.155"
        y="-342.15625" width="103.8457" height="121.0" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/e19ce55f7f00010177026e40305f270b</URIString>
        <point1 x="14602.501" y="-341.65625"/>
        <point2 x="14499.655" y="-221.65625"/>
        <ID1 xsi:type="node">2230</ID1>
        <ID2 xsi:type="node">2229</ID2>
    </child>
    <child ID="2232" label="session" layerID="1" created="1292275126025"
        x="14545.015" y="-20.156494" width="84.0" height="50.5"
        strokeWidth="1.0" autoSized="false" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/e19ce5617f00010177026e40f6ef7cde</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2233" layerID="1" created="1292275126033" x="14491.443"
        y="-175.15625" width="82.7168" height="155.5" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/e19ce5627f00010177026e400796cc3f</URIString>
        <point1 x="14491.943" y="-174.65625"/>
        <point2 x="14573.66" y="-20.15625"/>
        <ID1 xsi:type="node">2229</ID1>
        <ID2 xsi:type="node">2232</ID2>
    </child>
    <child ID="2234" label="MetaData" layerID="1"
        created="1292275149153" x="14711.515" y="-275.6565" width="91.0"
        height="43.0" strokeWidth="1.0" autoSized="false" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/e19ce5637f00010177026e4015495ed2</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2236" label="Database" layerID="1"
        created="1292275223829" x="14578.715" y="-481.65637"
        width="93.0" height="45.0" strokeWidth="1.0" autoSized="false" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/e19ce5647f00010177026e406e92ee08</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2237" label="engine(database)" layerID="1"
        created="1292275229366" x="14574.123" y="-437.15625"
        width="96.0" height="57.0" strokeWidth="1.0" autoSized="false"
        controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/e19ce5657f00010177026e40a5e1ab26</URIString>
        <point1 x="14623.837" y="-436.65625"/>
        <point2 x="14620.408" y="-380.65625"/>
        <ID1 xsi:type="node">2236</ID1>
        <ID2 xsi:type="node">2230</ID2>
    </child>
    <child ID="2238" layerID="1" created="1292275242605" x="14747.727"
        y="-233.16406" width="7.286133" height="55.007812"
        strokeWidth="1.0" autoSized="false" controlCount="0"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/e19ce5667f00010177026e40a2f1c46f</URIString>
        <point1 x="14754.513" y="-232.66406"/>
        <point2 x="14748.227" y="-178.65625"/>
        <ID1 xsi:type="node">2234</ID1>
        <ID2 xsi:type="node">2225</ID2>
    </child>
    <child ID="2226" layerID="1" created="1292274944072" x="14734.373"
        y="-141.15625" width="9.2421875" height="55.0" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/e19ce5677f00010177026e40fab7b7de</URIString>
        <point1 x="14743.115" y="-140.65625"/>
        <point2 x="14734.873" y="-86.65625"/>
        <ID1 xsi:type="node">2225</ID1>
        <ID2 xsi:type="node">2224</ID2>
    </child>
    <child ID="2242" label="metadata.create_all(engine)" layerID="1"
        created="1292275291662" x="14610.327" y="-342.15625"
        width="153.0" height="67.0" strokeWidth="1.0" autoSized="false"
        controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/e19ce5687f00010177026e40d871ebc7</URIString>
        <point1 x="14644.328" y="-341.65625"/>
        <point2 x="14729.326" y="-275.65625"/>
        <ID1 xsi:type="node">2230</ID1>
        <ID2 xsi:type="node">2234</ID2>
    </child>
    <child ID="2249" label="add" layerID="1" created="1292275365435"
        x="14592.916" y="-136.15625" width="30.28125" height="116.5"
        strokeWidth="1.0" autoSized="false" controlCount="0"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/e1a292207f00010177026e40616be0a9</URIString>
        <point1 x="14622.697" y="-135.65625"/>
        <point2 x="14593.416" y="-20.15625"/>
        <ID1 xsi:type="node">2227</ID1>
        <ID2 xsi:type="node">2232</ID2>
    </child>
    <child ID="2250" label="Persisting" layerID="1"
        created="1292275372356" x="14305.607" y="-457.34113"
        width="273.60742" height="480.51657" strokeWidth="1.0"
        autoSized="false" controlCount="2" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/e1a292217f00010177026e4079dd75c1</URIString>
        <point1 x="14545.015" y="16.992676"/>
        <point2 x="14578.715" y="-456.84113"/>
        <ID1 xsi:type="node">2232</ID1>
        <ID2 xsi:type="node">2236</ID2>
        <ctrlPoint0 x="14313.736" y="82.51904" xsi:type="point"/>
        <ctrlPoint1 x="14202.64" y="-438.11633" xsi:type="point"/>
    </child>
    <child ID="2254" label="config&#xa;(environment&#xa;variables)"
        layerID="1" created="1292275586156" x="14574.215" y="-642.6564"
        width="83.0" height="53.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/e1a292227f00010177026e40f20f40e8</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2255" label="database_adress&#xa;&amp;&#xa;user_name"
        layerID="1" created="1292275586704" x="14573.086" y="-590.15625"
        width="95.0" height="109.0" strokeWidth="1.0" autoSized="false"
        controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/e1a292227f00010177026e404f21e562</URIString>
        <point1 x="14617.318" y="-589.65625"/>
        <point2 x="14623.853" y="-481.65625"/>
        <ID1 xsi:type="node">2254</ID1>
        <ID2 xsi:type="node">2236</ID2>
    </child>
    <child ID="2257" label="SQLALCHEMY DB SETUP" layerID="1"
        created="1292277139230" x="14451.715" y="-773.65643"
        width="328.0" height="87.0" strokeWidth="1.0" autoSized="false" xsi:type="node">
        <fillColor>#B5B995</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-18</font>
        <URIString>http://vue.tufts.edu/rdf/resource/e1b958057f00010177026e4065f7dcfe</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2267" layerID="1" created="1292744499011" x="8649.359"
        y="486.5625" width="157.43555" height="123.78125"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/ff73db6b7f0001015481be8a2537d4ab</URIString>
        <point1 x="8806.295" y="487.0625"/>
        <point2 x="8649.859" y="609.84375"/>
        <ID1 xsi:type="node">1901</ID1>
        <ID2 xsi:type="node">1295</ID2>
        <ctrlPoint0 x="8807.627" y="568.82007" xsi:type="point"/>
        <ctrlPoint1 x="8648.785" y="545.93036" xsi:type="point"/>
    </child>
    <child ID="1900" layerID="1" created="1288257142624" x="8779.806"
        y="486.5625" width="27.305664" height="122.9375"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/f21c2a7b7f0001010d9438ded95282fa</URIString>
        <point1 x="8806.611" y="487.0625"/>
        <point2 x="8780.306" y="609.0"/>
        <ID1 xsi:type="node">1901</ID1>
        <ID2 xsi:type="node">1887</ID2>
        <ctrlPoint0 x="8807.837" y="554.70557" xsi:type="point"/>
        <ctrlPoint1 x="8783.59" y="551.4998" xsi:type="point"/>
    </child>
    <child ID="2287" label="Hold in Structure" layerID="1"
        created="1293638447932" x="13135.107" y="844.37695"
        width="241.0" height="173.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#30D643</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/32deba3f7f0001015088210050d1939f</URIString>
        <child ID="2288"
            label="Generalize the structure system and add: folder_templates asset_templates reference_templates to hold templates for every one of them. This leads us a complete structure object where one can see the whole picture in one object..."
            created="1293638447932" x="5.0" y="23.0" width="231.0"
            height="144.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
            <strokeColor>#404040</strokeColor>
            <textColor>#000000</textColor>
            <font>SansSerif-plain-14</font>
            <URIString>http://vue.tufts.edu/rdf/resource/32deba3f7f000101508821006c2d59f4</URIString>
            <richText>&lt;html&gt;
  &lt;head&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Generalize the structure system and add:
    &lt;/p&gt;
    &lt;ul color="#000000"&gt;
      &lt;li style="color: #000000" color="#000000"&gt;
        folder_templates
      &lt;/li&gt;
      &lt;li style="color: #000000" color="#000000"&gt;
        asset_templates
      &lt;/li&gt;
      &lt;li style="color: #000000" color="#000000"&gt;
        reference_templates
      &lt;/li&gt;
    &lt;/ul&gt;
    &lt;p color="#000000"&gt;
      
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      to hold templates for every one of them. This leads us a complete 
      structure object where one can see the whole picture in one object...
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
            <label>Generalize the structure system and add: folder_templates asset_templates reference_templates to hold templates for every one of them. This leads us a complete structure object where one can see the whole picture in one object...</label>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2289" label="SOLUTION 3&#xa;THE BEST SOLUTION"
        layerID="1" created="1293638451191" x="12919.361" y="670.71094"
        width="226.10352" height="175.39746" strokeWidth="3.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/32deba407f00010150882100a056b737</URIString>
        <point1 x="12920.861" y="672.21094"/>
        <point2 x="13143.965" y="844.6084"/>
        <ID1 xsi:type="node">2026</ID1>
        <ID2 xsi:type="node">2287</ID2>
    </child>
    <child ID="2319" label="version" layerID="1" created="1293706572638"
        x="12419.274" y="1489.2103" width="93.0" height="29.0"
        strokeWidth="1.0" autoSized="false" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/370c82157f00010150882100318ba225</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2321" label="version.task" layerID="1"
        created="1293706577144" x="12430.139" y="1517.6875" width="69.0"
        height="82.1875" strokeWidth="1.0" autoSized="false"
        controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/370c82167f00010150882100fbc03f47</URIString>
        <point1 x="12465.476" y="1518.1875"/>
        <point2 x="12463.803" y="1599.375"/>
        <ID1 xsi:type="node">2319</ID1>
        <ID2 xsi:type="node">2322</ID2>
    </child>
    <child ID="2322" label="task" layerID="1" created="1293706587561"
        x="12419.774" y="1599.377" width="87.5" height="27.0"
        strokeWidth="1.0" autoSized="false" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/370c82167f00010150882100c967ecd4</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2324" label="pipelineStep" layerID="1"
        created="1293706642673" x="12315.108" y="1716.377" width="80.0"
        height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/370c82167f00010150882100be8598dc</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2325" label="task.pipeline_step" layerID="1"
        created="1293706642678" x="12357.374" y="1625.877" width="102.0"
        height="91.0" strokeWidth="1.0" autoSized="false"
        controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/370c82167f000101508821004c57c340</URIString>
        <point1 x="12450.798" y="1626.377"/>
        <point2 x="12365.95" y="1716.377"/>
        <ID1 xsi:type="node">2322</ID1>
        <ID2 xsi:type="node">2324</ID2>
    </child>
    <child ID="2328" label="assetBase" layerID="1"
        created="1293706765117" x="12559.774" y="1733.7102" width="71.0"
        height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/370c82167f000101508821000ce84590</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2329" label="parent_asset" layerID="1"
        created="1293706765122" x="12476.465" y="1625.877"
        width="107.86035" height="108.33301" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/370c82167f000101508821004cd5726c</URIString>
        <point1 x="12476.965" y="1626.377"/>
        <point2 x="12583.825" y="1733.71"/>
        <ID1 xsi:type="node">2322</ID1>
        <ID2 xsi:type="node">2328</ID2>
    </child>
    <child ID="2333" label="AssetTemplate" layerID="1"
        created="1293717259136" x="12360.107" y="-412.16458"
        width="177.0" height="46.25" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#9DDB53</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/378f410b7f00010150882100d07ef254</URIString>
        <child ID="2369" label="asset_type | ONE | ASSETTYPE"
            created="1293721259644" x="34.0" y="23.0" width="183.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/37cc22ee7f00010150882100864e6ffb</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2358" label="ReferenceTemplate" layerID="1"
        created="1293717927562" x="12338.357" y="-487.16458"
        width="220.5" height="46.25" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#9DDB53</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/379966d07f000101508821000b2bd7fd</URIString>
        <child ID="2359" label="reference_type | ONE | REFERENCETYPE"
            created="1293717927562" x="34.0" y="23.0" width="241.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/379966d07f0001015088210028a2ff79</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2374"
        label="Implemented classes with persistancy (Classic SOM)"
        layerID="1" created="1294147396477" x="9222.637" y="-21.742615"
        width="332.0" height="21.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
        <strokeColor>#404040</strokeColor>
        <textColor>#000000</textColor>
        <font>SansSerif-plain-14</font>
        <URIString>http://vue.tufts.edu/rdf/resource/51327de87f0001017517e53b4e0b6faa</URIString>
        <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Implemented classes with persistancy (Classic SOM)
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
        <label>Implemented classes with persistancy (Classic SOM)</label>
    </child>
    <child ID="2375" layerID="1" created="1294147396477" x="9187.644"
        y="-26.542633" width="11.0" height="22.0" strokeWidth="1.0"
        autoSized="true" xsi:type="node">
        <fillColor>#9DDB53</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/51327de87f0001017517e53b06dcb343</URIString>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="2379" label="FileSequence" layerID="1"
        created="1294150177342" x="12312.344" y="-219.42288"
        width="115.0" height="66.5" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#ECFFD4</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/5161bffb7f0001017517e53b94128c41</URIString>
        <child ID="2380" label="start | INTEGER" created="1294150177342"
            x="34.0" y="23.0" width="97.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/5161bffb7f0001017517e53b3481e15b</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2381" label="end | INTEGER" created="1294150177342"
            x="34.0" y="43.25" width="94.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/5161bffc7f0001017517e53b06eb0375</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2382" label="File" layerID="1" created="1294150177342"
        x="12154.38" y="-219.04764" width="128.25" height="46.25"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#ECFFD4</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/5161bffc7f0001017517e53b0a74a25b</URIString>
        <child ID="2383" label="file_size | INTEGER"
            created="1294150177342" x="34.0" y="23.0" width="118.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/5161bffc7f0001017517e53bfcda26b7</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2384" layerID="1" created="1294150177342" x="12309.803"
        y="-399.32327" width="61.476562" height="180.38577"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/5161bffc7f0001017517e53b1d3dd4c2</URIString>
        <point1 x="12310.303" y="-398.82327"/>
        <point2 x="12370.779" y="-219.4375"/>
        <ID2 xsi:type="node">2379</ID2>
        <ctrlPoint0 x="12315.923" y="-233.81949" xsi:type="point"/>
        <ctrlPoint1 x="12372.488" y="-280.19986" xsi:type="point"/>
    </child>
    <child ID="2385" layerID="1" created="1294150177342" x="12218.27"
        y="-399.32327" width="92.475586" height="180.82327"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/5161bffc7f0001017517e53b0c1d6699</URIString>
        <point1 x="12310.245" y="-398.82327"/>
        <point2 x="12218.77" y="-219.0"/>
        <ID2 xsi:type="node">2382</ID2>
        <ctrlPoint0 x="12315.6" y="-233.2283" xsi:type="point"/>
        <ctrlPoint1 x="12219.3955" y="-273.93698" xsi:type="point"/>
    </child>
    <child ID="2386" label="Folder" layerID="1" created="1294150177342"
        x="12078.92" y="-219.04764" width="46.0" height="23.0"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#ECFFD4</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/5161bffd7f0001017517e53b80768083</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2387" layerID="1" created="1294150177342" x="12101.456"
        y="-399.32327" width="209.29492" height="180.82327"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/5161bffd7f0001017517e53b94d526a0</URIString>
        <point1 x="12310.251" y="-398.82327"/>
        <point2 x="12101.956" y="-219.0"/>
        <ID2 xsi:type="node">2386</ID2>
        <ctrlPoint0 x="12315.551" y="-235.86832" xsi:type="point"/>
        <ctrlPoint1 x="12102.15" y="-280.1417" xsi:type="point"/>
    </child>
    <child ID="2388" label="Web" layerID="1" created="1294150177342"
        x="12457.045" y="-219.42288" width="107.25" height="46.25"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#ECFFD4</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/5161bffd7f0001017517e53b87414322</URIString>
        <child ID="2389" label="url | UNICODE" created="1294150177343"
            x="34.0" y="23.0" width="90.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/5161bffd7f0001017517e53bf8dd37fb</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2390" layerID="1" created="1294150177342" x="12309.773"
        y="-399.32327" width="202.05957" height="180.38577"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/5161bffd7f0001017517e53b5ed7b8f6</URIString>
        <point1 x="12310.273" y="-398.82327"/>
        <point2 x="12511.333" y="-219.4375"/>
        <ID2 xsi:type="node">2388</ID2>
        <ctrlPoint0 x="12315.568" y="-239.22952" xsi:type="point"/>
        <ctrlPoint1 x="12513.098" y="-280.98056" xsi:type="point"/>
    </child>
    <child ID="2397" label="Type" layerID="1" created="1294154217183"
        x="9415.732" y="828.225" width="141.75" height="106.70194"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/519d2c637f0001017517e53b1a4782a4</URIString>
        <child ID="3827" label="TargetEntityTypeMixin"
            created="1317078622396" x="34.0" y="23.0" width="136.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/a800f9bb7f00010130ce55362b9cdc24</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4054" label="CodeMixin" created="1358233231637"
            x="34.0" y="43.25" width="70.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/3d029f0cc0a800044f9e91d03a4720d1</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4145" created="1369384356043" x="34.0" y="63.5"
            width="30.333334" height="25.602621" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#FEFB03</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/d5acad957b6a9fda5a40fe820dca0155</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <child ID="4032" created="1358111888146" x="34.0" y="85.701965"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35c8af06c0a8000435fa83795f0c1c20</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="2363"
        label="Reference Template Example 1: ReferenceType = Image {{repository.path}}/{{project.code}}/REFS/{{reference.type.name}}/{{reference.link.name}}"
        layerID="1" created="1293717968929" x="12136.389" y="2687.0854"
        width="518.0" height="48.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#404040</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/379ae6b47f00010150882100c2142256</URIString>
        <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Reference Template Example 1:
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      ReferenceType = Image
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      {{repository.path}}/{{project.code}}/REFS/{{reference.type.name}}/{{reference.link.name}}
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
        <label>Reference Template Example 1: ReferenceType = Image {{repository.path}}/{{project.code}}/REFS/{{reference.type.name}}/{{reference.link.name}}</label>
    </child>
    <child ID="2338"
        label="Asset Version Template Example 1: AssetType.name = Shot {{repository.path}}/{{project.code}}/SEQUENCES/{{sequence.name}}/SHOTS/{{shot.code}}/{{pipeline_step.code}}/{{shot.code}}_{{take.name}}_{{version.version_number}}"
        layerID="1" created="1293717259137" x="12136.389" y="2575.0854"
        width="984.0" height="48.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#404040</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/378f410b7f0001015088210053e34b34</URIString>
        <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Asset Version Template Example 1:
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      AssetType.name = Shot
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      {{repository.path}}/{{project.code}}/SEQUENCES/{{sequence.name}}/SHOTS/{{shot.code}}/{{pipeline_step.code}}/{{shot.code}}_{{take.name}}_{{version.version_number}}
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
        <label>Asset Version Template Example 1: AssetType.name = Shot {{repository.path}}/{{project.code}}/SEQUENCES/{{sequence.name}}/SHOTS/{{shot.code}}/{{pipeline_step.code}}/{{shot.code}}_{{take.name}}_{{version.version_number}}</label>
    </child>
    <child ID="2408" label="TRASHED IDEAS" layerID="1"
        created="1294155278342" x="12147.773" y="-633.4564"
        width="305.667" height="102.66669" strokeWidth="1.0"
        autoSized="false" xsi:type="node">
        <fillColor>#B5B995</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-18</font>
        <URIString>http://vue.tufts.edu/rdf/resource/51aaed357f0001017517e53b20cfc539</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2409"
        label="Asset Version Template Example 2: AssetType.name = Character {{repository.path}}/{{project.code}}/ASSETS/{{asset_type.name}}/{{pipeline_step.code}}/{{asset.name}}_{{take.name}}_{{asset_type.name}}_{{version.version_number}}"
        layerID="1" created="1294155909846" x="12136.389" y="2631.0435"
        width="977.0" height="48.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#404040</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/51b7d1837f0001017517e53b679d9674</URIString>
        <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Asset Version Template Example 2:
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      AssetType.name = Character
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      {{repository.path}}/{{project.code}}/ASSETS/{{asset_type.name}}/{{pipeline_step.code}}/{{asset.name}}_{{take.name}}_{{asset_type.name}}_{{version.version_number}}
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
        <label>Asset Version Template Example 2: AssetType.name = Character {{repository.path}}/{{project.code}}/ASSETS/{{asset_type.name}}/{{pipeline_step.code}}/{{asset.name}}_{{take.name}}_{{asset_type.name}}_{{version.version_number}}</label>
    </child>
    <child ID="2413"
        label="Reference Template Example 2: ReferenceType = Web (the same with the Image example) {{repository.path}}/{{project.code}}/REFS/{{reference.type.name}}/{{reference.link.name}}"
        layerID="1" created="1294156830278" x="12136.389" y="2741.0435"
        width="518.0" height="48.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#404040</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/51c2b1ad7f0001017517e53b94d0c271</URIString>
        <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Reference Template Example 2:
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      ReferenceType = Web (the same with the Image example)
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      {{repository.path}}/{{project.code}}/REFS/{{reference.type.name}}/{{reference.link.name}}
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
        <label>Reference Template Example 2: ReferenceType = Web (the same with the Image example) {{repository.path}}/{{project.code}}/REFS/{{reference.type.name}}/{{reference.link.name}}</label>
    </child>
    <child ID="2440" label="Note" layerID="1" created="1294846767149"
        x="8891.292" y="607.6561" width="126.75" height="122.4519"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/7ae32d837f0001017115e1660c06a911</URIString>
        <child ID="2441" label="content | UNICODE"
            created="1294846767149" x="34.0" y="23.0" width="116.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/7ae32d837f0001017115e1666b17d87e</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4148" created="1369384385601" x="34.0" y="43.25"
            width="30.333334" height="25.602621" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#FEFB03</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/d5acada37b6a9fda5a40fe82eb524c4b</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <child ID="4043" created="1358111978077" x="34.0" y="65.451965"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#30D643</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35c8af09c0a8000435fa8379afc3b443</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <child ID="4219" created="1468653311663" x="34.0" y="83.45194"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#33A8F5</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/f28ffbcfc0a8000f011bc52f32888ef6</URIString>
            <shape xsi:type="chevron"/>
        </child>
        <child ID="4220" created="1468653314809" x="34.0" y="101.45192"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#AF55F4</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/f28ffbcfc0a8000f011bc52f462ab2eb</URIString>
            <shape xsi:type="rhombus"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="2442" layerID="1" created="1294846767149" x="8805.921"
        y="486.59375" width="149.67773" height="121.40625"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/7ae32d837f0001017115e16676d33287</URIString>
        <point1 x="8806.421" y="487.09375"/>
        <point2 x="8955.099" y="607.5"/>
        <ID1 xsi:type="node">1901</ID1>
        <ID2 xsi:type="node">2440</ID2>
        <ctrlPoint0 x="8807.702" y="562.3818" xsi:type="point"/>
        <ctrlPoint1 x="8955.573" y="540.1338" xsi:type="point"/>
    </child>
    <child ID="1507"
        label="PipelineStep Examples: design-DESIGN model-MODEL rig-RIG fur-FUR shading-SHADE previs-PREVIS match move-MM animation-ANIM fx-FX cloth sim-CLOTHSIM layout-LAYOUT lighting-LIGHT compositing-COMP"
        layerID="1" created="1277557969560" x="11740.921" y="2513.225"
        width="114.0" height="243.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#404040</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/7464cb477f0001014a6b2ab74e33abce</URIString>
        <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      PipelineStep
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Examples:
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      design-DESIGN
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      model-MODEL
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      rig-RIG
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      fur-FUR
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      shading-SHADE
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      previs-PREVIS
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      match move-MM
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      animation-ANIM
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      fx-FX
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      cloth sim-CLOTHSIM
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      layout-LAYOUT
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      lighting-LIGHT
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      compositing-COMP
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
        <label>PipelineStep Examples: design-DESIGN model-MODEL rig-RIG fur-FUR shading-SHADE previs-PREVIS match move-MM animation-ANIM fx-FX cloth sim-CLOTHSIM layout-LAYOUT lighting-LIGHT compositing-COMP</label>
    </child>
    <child ID="1508"
        label="AssetType Examples: Character FuryCharacter Vehicle Prop Environment Shot"
        layerID="1" created="1277558023775" x="11934.202" y="2578.475"
        width="115.0" height="123.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#404040</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/7464cb547f0001014a6b2ab78c34774b</URIString>
        <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      AssetType
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Examples:
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Character
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      FuryCharacter
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Vehicle
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Prop
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Environment
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Shot
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
        <label>AssetType Examples: Character FuryCharacter Vehicle Prop Environment Shot</label>
    </child>
    <child ID="2014"
        label="Examples: ASSETS SEQUENCES SEQUENCES\EDIT_MOVIE ..."
        layerID="1" created="1290373811339" x="11725.721" y="2419.475"
        width="156.0" height="78.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#404040</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/704adc127f0001012ed4f13a3db45dfe</URIString>
        <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Examples:
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      ASSETS
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      SEQUENCES
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      SEQUENCES\EDIT_MOVIE
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      ...
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
        <label>Examples: ASSETS SEQUENCES SEQUENCES\EDIT_MOVIE ...</label>
    </child>
    <child ID="2305"
        label="LinkType Examples: Image ImageSequence Video Text Web ..."
        layerID="1" created="1293639905702" x="12177.091" y="2275.8252"
        width="111.0" height="108.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#404040</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/32f3ce3d7f000101508821003b9a78db</URIString>
        <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      LinkType Examples:
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Image
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      ImageSequence
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Video
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Text
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Web
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      ...
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
        <label>LinkType Examples: Image ImageSequence Video Text Web ...</label>
    </child>
    <child ID="2403"
        label="ProjectType Examples: Commercial Movie Still ..."
        layerID="1" created="1294154324356" x="12322.508" y="2271.425"
        width="128.0" height="78.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#404040</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/519d2c647f0001017517e53b19274f95</URIString>
        <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      ProjectType Examples:
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Commercial
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Movie
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Still
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      ...
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
        <label>ProjectType Examples: Commercial Movie Still ...</label>
    </child>
    <child ID="2406"
        label="UserType Examples: SuperUser Admin Normal ..."
        layerID="1" created="1294154384750" x="12621.506" y="2361.425"
        width="120.0" height="78.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
        <fillColor>#FCDBD9</fillColor>
        <strokeColor>#404040</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/519d2c647f0001017517e53b912c4954</URIString>
        <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      UserType Examples:
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      SuperUser
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Admin
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Normal
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      ...
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
        <label>UserType Examples: SuperUser Admin Normal ...</label>
    </child>
    <child ID="2462" layerID="1" created="1295291873435" x="9187.644"
        y="255.62396" width="29.0" height="23.0" strokeWidth="1.0"
        autoSized="false" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/9589d5c77f0001015ee819a888178704</URIString>
        <shape xsi:type="triangle"/>
    </child>
    <child ID="2463" label="has __eq__, __ne__" layerID="1"
        created="1295291882607" x="9222.637" y="257.62396" width="302.0"
        height="18.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
        <strokeColor>#404040</strokeColor>
        <textColor>#000000</textColor>
        <font>SansSerif-plain-14</font>
        <URIString>http://vue.tufts.edu/rdf/resource/9589d5c87f0001015ee819a8969060b6</URIString>
        <richText>&lt;html&gt;
  &lt;head&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      has __eq__, __ne__
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
        <label>has __eq__, __ne__</label>
    </child>
    <child ID="2468" layerID="1" created="1295294154121" x="9187.644"
        y="287.62393" width="29.0" height="23.0" strokeWidth="1.0"
        autoSized="false" xsi:type="node">
        <fillColor>#EA2218</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/958e29dc7f0001015ee819a82f59b16a</URIString>
        <shape xsi:type="triangle"/>
    </child>
    <child ID="2469" label="doesn't have __eq__, __ne__" layerID="1"
        created="1295294165800" x="9222.637" y="290.62393" width="302.0"
        height="18.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
        <strokeColor>#404040</strokeColor>
        <textColor>#000000</textColor>
        <font>SansSerif-plain-14</font>
        <URIString>http://vue.tufts.edu/rdf/resource/958e29dc7f0001015ee819a836cff506</URIString>
        <richText>&lt;html&gt;
  &lt;head&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      doesn't have __eq__, __ne__
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
        <label>doesn't have __eq__, __ne__</label>
    </child>
    <child ID="2516" layerID="1" created="1295345430716" x="9820.951"
        y="1733.8281" width="30.24414" height="96.88281"
        strokeWidth="1.0" autoSized="false" controlCount="0"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/989c41f37f0001015ee819a8aa4c7a01</URIString>
        <point1 x="9850.695" y="1734.3281"/>
        <point2 x="9821.451" y="1830.2109"/>
        <ID1 xsi:type="node">3382</ID1>
        <ID2 xsi:type="node">2517</ID2>
    </child>
    <child ID="2517" label="Statuses" layerID="1"
        created="1295345447286" x="9741.384" y="1830.21" width="127.5"
        height="107.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#83CEFF</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/989c41f37f0001015ee819a84c18ca80</URIString>
        <child ID="2519" label="NotRead - NR" created="1295345464908"
            x="34.0" y="23.0" width="85.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/989c41f47f0001015ee819a869e77b53</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2520" label="Read - R" created="1295345495895"
            x="34.0" y="43.25" width="58.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/989c41f47f0001015ee819a82998ee58</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="2521" label="Replied - REP" created="1295345503438"
            x="34.0" y="63.5" width="86.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/989c41f47f0001015ee819a8dac3e9ea</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3901" label="Forwarded - FORW"
            created="1336640474414" x="34.0" y="83.75" width="117.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fce5c77f000101663ffd147c8b9e26</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2536" label="PROBLEM" layerID="1" created="1295363437120"
        x="14496.209" y="565.01025" width="215.0" height="76.0"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FCDBD9</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/99b647627f00010103d66c396d43c345</URIString>
        <child ID="2537"
            label="the Status and StatusList objects doesn't carry any usage information &#xa;      other than the name"
            created="1295363437121" x="5.0" y="23.0" width="205.0"
            height="47.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
            <strokeColor>#404040</strokeColor>
            <textColor>#000000</textColor>
            <font>SansSerif-plain-14</font>
            <URIString>http://vue.tufts.edu/rdf/resource/99b647627f00010103d66c39e7b3a569</URIString>
            <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      the Status and StatusList objects doesn't carry any usage information 
      other than the name
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
            <label>the Status and StatusList objects doesn't carry any usage information 
      other than the name</label>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2540" label="Ex1: Project Status" layerID="1"
        created="1295363507887" x="14517.209" y="751.5659" width="173.0"
        height="151.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/99b647637f00010103d66c3913f91d93</URIString>
        <child ID="2535"
            label="A project StatusList may have these status objects * WaitingToStart * In Progress * Stopped * Completed"
            created="1295363362392" x="5.0" y="23.0" width="163.0"
            height="122.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
            <strokeColor>#404040</strokeColor>
            <textColor>#000000</textColor>
            <font>SansSerif-plain-14</font>
            <URIString>http://vue.tufts.edu/rdf/resource/99b647637f00010103d66c39798b7d69</URIString>
            <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      A project StatusList may have these status objects
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      * WaitingToStart
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      * In Progress
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      * Stopped
    &lt;/p&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      * Completed
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
            <label>A project StatusList may have these status objects * WaitingToStart * In Progress * Stopped * Completed</label>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2541" layerID="1" created="1295363507901" x="14603.209"
        y="707.2881" width="1.0" height="44.777832" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/99b647647f00010103d66c39554a3237</URIString>
        <point1 x="14603.709" y="707.7881"/>
        <point2 x="14603.709" y="751.5659"/>
        <ID1 xsi:type="node">2542</ID1>
        <ID2 xsi:type="node">2540</ID2>
    </child>
    <child ID="2542" label="For Example?" layerID="1"
        created="1295363518289" x="14559.709" y="684.7881" width="88.0"
        height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/99b647647f00010103d66c3971f65221</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2544" layerID="1" created="1295363523767" x="14603.209"
        y="640.51025" width="1.0" height="44.777832" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/99b647647f00010103d66c39b6f57f91</URIString>
        <point1 x="14603.709" y="641.01025"/>
        <point2 x="14603.709" y="684.7881"/>
        <ID1 xsi:type="node">2536</ID1>
        <ID2 xsi:type="node">2542</ID2>
    </child>
    <child ID="2545"
        label="When somebody creates a new project&#xa;he/she also needs to connect the&#xa;suitable StatusList to the created&#xa;project&#xa;&#xa;which is awkward"
        layerID="1" created="1295363602206" x="14482.0625" y="946.34375"
        width="252.0" height="114.166504" strokeWidth="1.0"
        autoSized="false" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/99b647657f00010103d66c391ab244f1</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2546" layerID="1" created="1295363602216" x="14605.073"
        y="902.0625" width="2.0800781" height="44.75" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/99b647657f00010103d66c39a31ed307</URIString>
        <point1 x="14605.573" y="902.5625"/>
        <point2 x="14606.653" y="946.3125"/>
        <ID1 xsi:type="node">2540</ID1>
        <ID2 xsi:type="node">2545</ID2>
    </child>
    <child ID="2547"
        label="There could be a configuration&#xa;(StatusListConfig)&#xa;object storing the config for&#xa;an entity_type and a corresponding StatusLists"
        layerID="1" created="1295363665880" x="14325.361" y="1155.4437"
        width="295.0" height="95.06653" strokeWidth="1.0"
        autoSized="false" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/99b647657f00010103d66c3962c652da</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2548" label="SOLUTION 1" layerID="1"
        created="1295363665890" x="14504.567" y="1060.0098"
        width="65.32031" height="95.93359" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/99b647657f00010103d66c39f018504b</URIString>
        <point1 x="14569.388" y="1060.5098"/>
        <point2 x="14505.067" y="1155.4434"/>
        <ID1 xsi:type="node">2545</ID1>
        <ID2 xsi:type="node">2547</ID2>
    </child>
    <child ID="2550"
        label="entity_type is not stored in objects,&#xa;it is introduced with SQLAlchemy&#xa;inheritance mapping"
        layerID="1" created="1295363894917" x="14355.109" y="1290.302"
        width="227.83008" height="72.916626" strokeWidth="1.0"
        autoSized="false" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/99b647667f00010103d66c3963aa548b</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2551" layerID="1" created="1295363894931" x="14469.653"
        y="1250.0" width="2.234375" height="40.8125" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/99b647667f00010103d66c39cc52cbaf</URIString>
        <point1 x="14471.388" y="1250.5"/>
        <point2 x="14470.153" y="1290.3125"/>
        <ID1 xsi:type="node">2547</ID1>
        <ID2 xsi:type="node">2550</ID2>
    </child>
    <child ID="2552"
        label="adding the entity_type as a&#xa;class attribute and trying to&#xa;store it makes things complex"
        layerID="1" created="1295363983600" x="14364.109" y="1403.0103"
        width="209.83008" height="74.166626" strokeWidth="1.0"
        autoSized="false" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/99b647667f00010103d66c39bd63898b</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="2553" layerID="1" created="1295363983606" x="14468.524"
        y="1362.7188" width="1.0" height="40.791626" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/99b647677f00010103d66c39516f6584</URIString>
        <point1 x="14469.024" y="1363.2186"/>
        <point2 x="14469.024" y="1403.0103"/>
        <ID1 xsi:type="node">2550</ID1>
        <ID2 xsi:type="node">2552</ID2>
    </child>
    <child ID="3268" label="statusList_statuses" layerID="1"
        created="1295449567282" x="8027.6895" y="1093.0936"
        width="191.4" height="96.9" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#F4E5FF</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/9ed04b217f000101584edf4a532914e6</URIString>
        <child ID="3269" label="status_id | INTEGER"
            created="1295449567283" x="49.199997" y="39.2" width="124.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F4E5FF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/9ed04b227f000101584edf4adc4f16e1</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3270" label="status_list_id | INTEGER"
            created="1295449567284" x="49.199997" y="59.45"
            width="147.0" height="23.0" strokeWidth="1.0"
            autoSized="true" xsi:type="node">
            <fillColor>#F4E5FF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/9ed04b227f000101584edf4ac287ddd0</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape xsi:type="hexagon"/>
    </child>
    <child ID="3271" layerID="1" created="1295449631420" x="8149.727"
        y="925.28906" width="28.620605" height="168.30469"
        strokeWidth="1.0" strokeStyle="4" autoSized="false"
        controlCount="2" arrowState="3" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/9ed0d73a7f000101584edf4aec9c5bc9</URIString>
        <point1 x="8165.1694" y="925.78906"/>
        <point2 x="8156.662" y="1093.0938"/>
        <ID1 xsi:type="node">1396</ID1>
        <ID2 xsi:type="node">3268</ID2>
        <ctrlPoint0 x="8112.1006" y="968.7471" xsi:type="point"/>
        <ctrlPoint1 x="8221.418" y="998.7998" xsi:type="point"/>
    </child>
    <child ID="3272" layerID="1" created="1295449633485" x="8184.4805"
        y="917.54395" width="114.43359" height="186.24512"
        strokeWidth="1.0" strokeStyle="4" autoSized="false"
        controlCount="2" arrowState="3" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/9ed0d73a7f000101584edf4a0d1e47f0</URIString>
        <point1 x="8298.414" y="918.04395"/>
        <point2 x="8184.9805" y="1103.2891"/>
        <ID1 xsi:type="node">1355</ID1>
        <ID2 xsi:type="node">3268</ID2>
        <ctrlPoint0 x="8195.113" y="966.3718" xsi:type="point"/>
        <ctrlPoint1 x="8284.184" y="1041.6741" xsi:type="point"/>
    </child>
    <child ID="3277" label="SimpleInfoMixin" layerID="1"
        created="1295454217799" x="6918.8574" y="886.5633"
        width="143.25" height="107.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#ECFFD4</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/a090ad487f000101584edf4a94fcff70</URIString>
        <child ID="3278" label="name | UNICODE" created="1295454217799"
            x="34.0" y="23.0" width="108.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/a090ad487f000101584edf4ae72ab318</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3279" label="description | UNICODE"
            created="1295454217799" x="34.0" y="43.25" width="137.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/a090ad497f000101584edf4ad849b8ad</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3280" label="nice_name | UNICODE"
            created="1295454217800" x="34.0" y="63.5" width="138.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/a090ad497f000101584edf4a3bff7252</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3281" label="code | UNICODE" created="1295454217800"
            x="34.0" y="83.75" width="103.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/a090ad497f000101584edf4ab7894019</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3282" label="AuditInfoMixin" layerID="1"
        created="1295454217799" x="7092.4014" y="886.5633"
        width="155.25" height="107.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#ECFFD4</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/a090ad497f000101584edf4acff2ae90</URIString>
        <child ID="3283" label="created_by | ONE | USER"
            created="1295454217800" x="34.0" y="23.0" width="149.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/a090ad497f000101584edf4a1105aa98</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3284" label="updated_by | ONE | USER"
            created="1295454217800" x="34.0" y="43.25" width="153.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/a090ad4a7f000101584edf4aeaffa9cd</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3285" label="date_created | DATETIME"
            created="1295454217801" x="34.0" y="63.5" width="150.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/a090ad4a7f000101584edf4a3be63e9c</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3286" label="date_updated | DATETIME"
            created="1295454217801" x="34.0" y="83.75" width="154.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/a090ad4a7f000101584edf4a6202ee85</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3287" label="NoteMixin" layerID="1"
        created="1295454217799" x="7428.9893" y="886.5633" width="135.0"
        height="46.25" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#ECFFD4</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/a090ad4a7f000101584edf4a8c4e4827</URIString>
        <child ID="3288" label="Notes | MANY | NOTE"
            created="1295454217801" x="34.0" y="23.0" width="127.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/a090ad4a7f000101584edf4a026ab232</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3289" label="StatusMixin" layerID="1"
        created="1295454217799" x="6804.215" y="1381.2302" width="177.0"
        height="66.5" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/a090ad4a7f000101584edf4a8be2ca88</URIString>
        <child ID="3290" label="status | INTEGER"
            created="1295454217802" x="34.0" y="23.0" width="107.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/a090ad4b7f000101584edf4a57f6e798</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3291" label="status_list | ONE | STATUSLIST"
            created="1295454217802" x="34.0" y="43.25" width="183.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/a090ad4b7f000101584edf4abdcb2014</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="3294" label="LinkMixin" layerID="1"
        created="1295454217799" x="6932.709" y="1066.5635"
        width="125.25" height="46.25" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#ECFFD4</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/a090ad4b7f000101584edf4a9c160278</URIString>
        <child ID="3295" label="links | MANY | LINK"
            created="1295454217803" x="34.0" y="23.0" width="114.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/a090ad4b7f000101584edf4a8f7bf7ed</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3296" label="TagMixin" layerID="1"
        created="1295454217799" x="7277.9453" y="886.5633"
        width="120.75" height="46.25" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#ECFFD4</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/a090ad4c7f000101584edf4aa59d18f5</URIString>
        <child ID="3297" label="tags | MANY | TAG"
            created="1295454217803" x="34.0" y="23.0" width="108.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/a090ad4c7f000101584edf4a9920537a</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3298" label="SimpleInfoMixin" layerID="1"
        created="1295454217799" x="6939.9824" y="858.62384"
        width="101.0" height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FF66CC</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/a090ad4c7f000101584edf4a2e38037c</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3299" label="AuditInfoMixin" layerID="1"
        created="1295454217799" x="7124.5264" y="858.62384" width="91.0"
        height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FF66CC</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/a090ad4c7f000101584edf4a31e3a548</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3300" label="TagMixin" layerID="1"
        created="1295454217799" x="7307.3203" y="858.62384" width="62.0"
        height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FF66CC</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/a090ad4c7f000101584edf4a5dc9cb23</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3301" label="NoteMixin" layerID="1"
        created="1295454217799" x="7462.9893" y="858.62384" width="67.0"
        height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FF66CC</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/a090ad4c7f000101584edf4a6a5f9529</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3302" label="StatusMixin" layerID="1"
        created="1295454217799" x="6853.715" y="1353.2908" width="78.0"
        height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FF66CC</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/a090ad4d7f000101584edf4a28f3414f</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3304" label="LinkMixin" layerID="1"
        created="1295454217799" x="6962.834" y="1038.624" width="65.0"
        height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FF66CC</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/a090ad4d7f000101584edf4a684a1372</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3305" label="ReferenceMixin" layerID="1"
        created="1295454231670" x="7011.083" y="1381.2302"
        width="192.75" height="46.25" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/a090ad4d7f000101584edf4a0f42ffd2</URIString>
        <child ID="3306" label="references | MANY | SIMPLEENTITY"
            created="1295454231670" x="34.0" y="23.0" width="204.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/a090ad4d7f000101584edf4ae3d8db49</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="3307" label="ReferenceMixin" layerID="1"
        created="1295454231670" x="7030.833" y="1353.2908" width="100.0"
        height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FF66CC</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/a090ad4d7f000101584edf4a6e4c0543</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3308" label="entity_tags" layerID="1"
        created="1295513081494" x="8329.315" y="629.09357" width="129.0"
        height="85.5" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#F4E5FF</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/a3e3fcb37f000101584edf4a6c183c36</URIString>
        <child ID="3309" label="entity_id" created="1295513081494"
            x="43.5" y="33.5" width="56.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F4E5FF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/a3e3fcb37f000101584edf4ab9417f26</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3310" label="tag_id" created="1295513081494" x="43.5"
            y="53.75" width="45.0" height="23.0" strokeWidth="1.0"
            autoSized="true" xsi:type="node">
            <fillColor>#F4E5FF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/a3e3fcb37f000101584edf4af1431523</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape xsi:type="hexagon"/>
    </child>
    <child ID="3311" layerID="1" created="1295513085191" x="8453.757"
        y="639.91956" width="166.21191" height="38.878357"
        strokeWidth="1.0" strokeStyle="4" autoSized="false"
        controlCount="2" arrowState="3" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/a3e3fcb47f000101584edf4a6af13a2b</URIString>
        <point1 x="8619.469" y="650.3679"/>
        <point2 x="8454.257" y="677.8034"/>
        <ID1 xsi:type="node">1295</ID1>
        <ID2 xsi:type="node">3308</ID2>
        <ctrlPoint0 x="8515.571" y="615.0924" xsi:type="point"/>
        <ctrlPoint1 x="8527.499" y="685.0255" xsi:type="point"/>
    </child>
    <child ID="3312" layerID="1" created="1295513087786" x="8444.074"
        y="691.51855" width="265.90527" height="28.437012"
        strokeWidth="1.0" strokeStyle="4" autoSized="false"
        controlCount="2" arrowState="3" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/a3e3fcb47f000101584edf4aefd4d6f5</URIString>
        <point1 x="8709.4795" y="699.2102"/>
        <point2 x="8444.574" y="692.01855"/>
        <ID1 xsi:type="node">1887</ID1>
        <ID2 xsi:type="node">3308</ID2>
        <ctrlPoint0 x="8567.119" y="739.42786" xsi:type="point"/>
        <ctrlPoint1 x="8490.824" y="710.40155" xsi:type="point"/>
    </child>
    <child ID="3327" label="Secondary tables and columns" layerID="1"
        created="1295615853370" x="9222.637" y="162.42393" width="268.0"
        height="24.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
        <strokeColor>#404040</strokeColor>
        <textColor>#000000</textColor>
        <font>SansSerif-plain-14</font>
        <URIString>http://vue.tufts.edu/rdf/resource/a8ba8fd57f000101584edf4a096af17f</URIString>
        <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Secondary tables and columns
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
        <label>Secondary tables and columns</label>
    </child>
    <child ID="3328" layerID="1" created="1295615853370" x="9187.644"
        y="162.42393" width="29.0" height="23.0" strokeWidth="1.0"
        autoSized="false" xsi:type="node">
        <fillColor>#F4E5FF</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/a8ba8fd57f000101584edf4a3546060b</URIString>
        <shape xsi:type="hexagon"/>
    </child>
    <child ID="3345" label="Ticket" layerID="1" created="1295880821526"
        x="9781.032" y="1113.4767" width="177.0" height="185.74997"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/b88461ff7f0001017e6ab533d4503f36</URIString>
        <child ID="3471" label="StatusMixin" created="1296054703214"
            x="34.0" y="23.0" width="78.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/c2e1996b7f0001013611c755eb7ed3ec</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3929" label="summary | STRING"
            created="1336649323755" x="34.0" y="43.25" width="117.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/36a21ec47f000101663ffd1411756b19</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3872" label="links| MANY | SIMPLEENTITY"
            created="1336638928135" x="34.0" y="63.5" width="168.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fa8c717f000101663ffd14fa7d5a3c</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3427" label="related_tickets | MANY | TICKET"
            created="1295885927221" x="34.0" y="83.75" width="183.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/b8d52ad37f0001017e6ab533649e6f1a</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3429" label="priority | ENUM" created="1295885972789"
            x="34.0" y="104.0" width="91.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/b8d52ad37f0001017e6ab5334b256314</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3930" label="comments | MANY | NOTE"
            created="1336649467413" x="34.0" y="124.25" width="153.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/36a21ec57f000101663ffd14e6bf6992</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3891" label="reported_by --> created_by"
            created="1336639168963" x="34.0" y="144.5" width="165.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fa8c767f000101663ffd141afe5d40</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4047" created="1358112045645" x="34.0" y="164.75"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#30D643</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/3624ae91c0a8000435fa8379e0b02f97</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="3372" label="message_attachments" layerID="1"
        created="1295881713679" x="9989.352" y="1781.927" width="213.6"
        height="101.1" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#F4E5FF</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/b892c8087f0001017e6ab53319adbe2b</URIString>
        <child ID="3373" label="message_id | INTEGER"
            created="1295881713679" x="51.300003" y="41.3" width="143.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F4E5FF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/b892c8087f0001017e6ab53369965690</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3374" label="simpleEntity_id | INTEGER"
            created="1295881713680" x="51.300003" y="61.55"
            width="157.0" height="23.0" strokeWidth="1.0"
            autoSized="true" xsi:type="node">
            <fillColor>#F4E5FF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/b892c8097f0001017e6ab5339330d0c8</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape xsi:type="hexagon"/>
    </child>
    <child ID="3377" label="entity_types" layerID="1"
        created="1295881843057" x="8390.583" y="348.39355"
        width="163.75" height="99.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#EA2218</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/b89461017f0001017e6ab533c77dd91b</URIString>
        <child ID="3378" label="id | INTEGER" created="1295881843057"
            x="50.25" y="40.25" width="83.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/b89461027f0001017e6ab533d3d5b3b9</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3379" label="entity_type | STRING"
            created="1295881843057" x="50.25" y="60.5" width="122.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/b89461027f0001017e6ab533421c6a11</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape xsi:type="hexagon"/>
    </child>
    <child ID="3393" label="Dailies" layerID="1" created="1295883108630"
        x="10014.451" y="1508.0773" width="150.0" height="86.75"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#ECFFD4</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/b8a7d59e7f0001017e6ab5334de98edf</URIString>
        <child ID="3917" label="ScheduleMixin" created="1336641056893"
            x="34.0" y="23.0" width="94.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/360402fc7f000101663ffd14d8ec2158</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3468" label="StatusMixin" created="1296050036295"
            x="34.0" y="43.25" width="78.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/c2bbd0437f0001013611c755eeb1c974</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3394" label="playlist | ONE | PLAYLIST"
            created="1295883108630" x="34.0" y="63.5" width="147.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/b8a7d59e7f0001017e6ab5339706b727</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3396" layerID="1" created="1295883153950" x="9876.9375"
        y="1733.8281" width="126.6875" height="117.22119"
        strokeWidth="1.0" strokeStyle="4" autoSized="false"
        controlCount="2" arrowState="3" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/b8a7d59e7f0001017e6ab5334b036143</URIString>
        <point1 x="9877.964" y="1734.3281"/>
        <point2 x="10003.125" y="1846.9181"/>
        <ID1 xsi:type="node">3382</ID1>
        <ID2 xsi:type="node">3372</ID2>
        <ctrlPoint0 x="9871.733" y="1831.7656" xsi:type="point"/>
        <ctrlPoint1 x="9902.772" y="1862.4963" xsi:type="point"/>
    </child>
    <child ID="3397" layerID="1" created="1295883197344" x="8775.108"
        y="751.1875" width="1314.5088" height="757.4375"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/b8a8a4607f0001017e6ab533617f8ad0</URIString>
        <point1 x="8775.608" y="751.6875"/>
        <point2 x="10089.117" y="1508.125"/>
        <ID1 xsi:type="node">1887</ID1>
        <ID2 xsi:type="node">3393</ID2>
        <ctrlPoint0 x="8769.253" y="1481.3043" xsi:type="point"/>
        <ctrlPoint1 x="10088.245" y="1394.849" xsi:type="point"/>
    </child>
    <child ID="3453" layerID="1" created="1295909615611" x="8807.853"
        y="486.59375" width="2027.4814" height="1297.0859"
        strokeWidth="1.0" strokeStyle="4" autoSized="false"
        controlCount="2" arrowState="3" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/ba3bf5b37f0001017e6ab5338b9ff6a2</URIString>
        <point1 x="10155.936" y="1783.1797"/>
        <point2 x="8808.353" y="487.09375"/>
        <ID1 xsi:type="node">3372</ID1>
        <ID2 xsi:type="node">1901</ID2>
        <ctrlPoint0 x="12256.546" y="51.03253" xsi:type="point"/>
        <ctrlPoint1 x="8812.2295" y="624.50507" xsi:type="point"/>
    </child>
    <child ID="3476" label="GENERAL DESIGN QUESTIONS" layerID="1"
        created="1296055192405" x="13538.116" y="311.34357"
        width="328.0" height="87.0" strokeWidth="1.0" autoSized="false" xsi:type="node">
        <fillColor>#B5B995</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-18</font>
        <URIString>http://vue.tufts.edu/rdf/resource/c2e990f77f0001013611c75558b3d989</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3480" label="ScheduleMixin" layerID="1"
        created="1297409383496" x="7228.116" y="1381.2302" width="255.0"
        height="188.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/13a3d6fc7f0001017490649e5a37d165</URIString>
        <child ID="4203" label="schedule_timing | FLOAT"
            created="1459165978619" x="34.0" y="23.0" width="149.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/bd122965c0a82a4a01152d0eff7dbcbe</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4202"
            label="schedule_unit | STRING:{min, h, d, w, m, y}"
            created="1459165946055" x="34.0" y="43.25" width="244.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/bd122965c0a82a4a01152d0e8ff34d55</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4201"
            label="schedule_model | STRING:{effort, duration, length}"
            created="1459165895160" x="34.0" y="63.5" width="287.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/bd122965c0a82a4a01152d0ee270ee7f</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4265"
            label="schedule_constraint | STRING: {onstart, onend}"
            created="1471423327881" x="34.0" y="83.75" width="270.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/97ae2cf5c0a82a4a019efdbd112abd0a</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4090" label="timing_resolution | TIMEDELTA"
            created="1365407705378" x="34.0" y="104.0" width="181.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/e8a539b8648c6197658d749ac2c528ef</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4266" label="least_meaningful_time_unit"
            created="1471423396444" x="34.0" y="124.25" width="169.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/97ae2cf5c0a82a4a019efdbdb69fc72f</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4091" label="to_seconds" created="1365407725309"
            x="34.0" y="144.5" width="78.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/e8a539b8648c6197658d749a4719b74d</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4267" label="schedule_seconds"
            created="1471423416618" x="34.0" y="164.75" width="119.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/97ae2cf6c0a82a4a019efdbd9323b8c2</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="3482" label="ScheduleMixin" layerID="1"
        created="1297409383496" x="7250.866" y="1353.2908" width="94.0"
        height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FF66CC</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/13a3d6fc7f0001017490649eaefc5577</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3493" label="Hold it in StatusList.target_entity_type"
        layerID="1" created="1297950196089" x="14708.116" y="1155.4437"
        width="313.0" height="92.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#30D643</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/33e78f547f0001016203171e285ee458</URIString>
        <child ID="3495"
            label="Just add another parameter called &quot;taget_entity_type&quot; to the &#xa;      StatusLits object, and use it when assigning to other objects to check if it is compatible with the StatusList"
            created="1297950490161" x="5.0" y="23.0" width="303.0"
            height="63.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
            <strokeColor>#404040</strokeColor>
            <textColor>#000000</textColor>
            <font>SansSerif-plain-14</font>
            <URIString>http://vue.tufts.edu/rdf/resource/33e78f547f0001016203171e3badff02</URIString>
            <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="text-align: center; color: #000000" color="#000000"&gt;
      Just add another parameter called &amp;quot;taget_entity_type&amp;quot; to the StatusLits 
      object,
    &lt;/p&gt;
    &lt;p style="text-align: center; color: #000000" color="#000000"&gt;
      and use it when assigning to other objects to check if it is compatible 
      with the StatusList
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
            <label>Just add another parameter called "taget_entity_type" to the 
      StatusLits object, and use it when assigning to other objects to check if it is compatible with the StatusList</label>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3494" label="SOLUTION 2&#xa;BEST SOLUTION" layerID="1"
        created="1297950196100" x="14681.521" y="1060.0098"
        width="123.99707" height="95.93359" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/33e78f557f0001016203171e4e51f849</URIString>
        <point1 x="14682.0205" y="1060.5098"/>
        <point2 x="14805.018" y="1155.4434"/>
        <ID1 xsi:type="node">2545</ID1>
        <ID2 xsi:type="node">3493</ID2>
    </child>
    <child ID="3501" label="Tasks And TaskTypes" layerID="1"
        created="1297950697096" x="16212.05" y="575.9103"
        width="257.33398" height="84.0" strokeWidth="1.0"
        autoSized="false" xsi:type="node">
        <fillColor>#FCDBD9</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/33e78f557f0001016203171ee3b835b9</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3523"
        label="An Asset could have an AssetType&#xa;but it shouldn't limit the availale&#xa;PipelineSteps it should be just&#xa;a template for initialization&#xa;or reuse"
        layerID="1" created="1297951962490" x="16241.717" y="751.34357"
        width="198.0" height="83.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/33fbcb187f0001016203171e914499e4</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3524" layerID="1" created="1297951962497" x="16340.217"
        y="659.4103" width="1.0" height="92.43329" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/33fbcb187f0001016203171eb92d4790</URIString>
        <point1 x="16340.717" y="659.9103"/>
        <point2 x="16340.717" y="751.34357"/>
        <ID1 xsi:type="node">3501</ID1>
        <ID2 xsi:type="node">3523</ID2>
    </child>
    <child ID="3525"
        label="it should be possible&#xa;to add a new step thus a new&#xa;task to only one Asset without&#xa;creating a new AssetType&#xa;or new step"
        layerID="1" created="1297952019051" x="16254.717" y="1044.3435"
        width="172.0" height="83.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/33fbcb257f0001016203171eda0f1c6e</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3526" layerID="1" created="1297952019059" x="16340.217"
        y="991.8435" width="1.0" height="52.99994" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/33fbcb257f0001016203171e7277c051</URIString>
        <point1 x="16340.717" y="992.34357"/>
        <point2 x="16340.717" y="1044.3435"/>
        <ID1 xsi:type="node">3527</ID1>
        <ID2 xsi:type="node">3525</ID2>
    </child>
    <child ID="3527"
        label="whenever the&#xa;asset is created with the&#xa;type is set, the list of tasks&#xa;also should be initialized,&#xa;with the predefined list&#xa;of tasks"
        layerID="1" created="1297952051105" x="16263.717" y="894.34357"
        width="154.0" height="98.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/33fbcb257f0001016203171ef4f135a1</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3528" layerID="1" created="1297952051113" x="16340.217"
        y="833.84357" width="1.0" height="61.0" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/33fbcb257f0001016203171ec0f120f3</URIString>
        <point1 x="16340.717" y="834.34357"/>
        <point2 x="16340.717" y="894.34357"/>
        <ID1 xsi:type="node">3523</ID1>
        <ID2 xsi:type="node">3527</ID2>
    </child>
    <child ID="3529"
        label="TaskTemplates --> list of TaskTypes&#xa;Character --> [Model, Shading etc.]"
        layerID="1" created="1297952087374" x="16499.46" y="924.34357"
        width="210.0" height="38.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/33fbcb267f0001016203171e0dce531a</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3530" layerID="1" created="1297952087388" x="16417.219"
        y="942.84357" width="82.74414" height="1.0" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/33fbcb267f0001016203171e019e7b69</URIString>
        <point1 x="16417.717" y="943.34357"/>
        <point2 x="16499.46" y="943.34357"/>
        <ID1 xsi:type="node">3527</ID1>
        <ID2 xsi:type="node">3529</ID2>
    </child>
    <child ID="3531" label="AssetBase.tasks --> list of Tasks"
        layerID="1" created="1297952165157" x="16500.71" y="1073.5935"
        width="191.0" height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/33fbcb267f0001016203171ed809bcbd</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3532" layerID="1" created="1297952165165" x="16426.219"
        y="1084.8738" width="74.99414" height="1.2172852"
        strokeWidth="1.0" autoSized="false" controlCount="0"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/33fbcb267f0001016203171ed634beb3</URIString>
        <point1 x="16426.717" y="1085.5911"/>
        <point2 x="16500.71" y="1085.3738"/>
        <ID1 xsi:type="node">3525</ID1>
        <ID2 xsi:type="node">3531</ID2>
    </child>
    <child ID="3533"
        label="Tasks then can have different types like:&#xa;Model&#xa;Animation&#xa;Lighting&#xa;Shading&#xa;etc."
        layerID="1" created="1297952179634" x="16226.717" y="1182.3435"
        width="228.0" height="98.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/33fbcb277f0001016203171e4fc43b50</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3534" layerID="1" created="1297952179642" x="16340.217"
        y="1126.8435" width="1.0" height="56.0" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/33fbcb277f0001016203171ed315cddc</URIString>
        <point1 x="16340.717" y="1127.3435"/>
        <point2 x="16340.717" y="1182.3435"/>
        <ID1 xsi:type="node">3525</ID1>
        <ID2 xsi:type="node">3533</ID2>
    </child>
    <child ID="3536" label="Task.type --> TaskTypes" layerID="1"
        created="1297952222946" x="16527.21" y="1217.0935" width="172.0"
        height="28.5" strokeWidth="1.0" autoSized="false" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/33fbcb277f0001016203171e807ae3dd</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3537" layerID="1" created="1297952222954" x="16454.219"
        y="1230.8435" width="73.49414" height="1.0" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/33fbcb277f0001016203171e88a8737b</URIString>
        <point1 x="16454.717" y="1231.3435"/>
        <point2 x="16527.21" y="1231.3435"/>
        <ID1 xsi:type="node">3533</ID1>
        <ID2 xsi:type="node">3536</ID2>
    </child>
    <child ID="3538" label="AssetTypes --> TaskTemplates" layerID="1"
        created="1297952773356" x="16513.46" y="986.34357" width="182.0"
        height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/340fb3a07f0001016203171e0b84f385</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3539" layerID="1" created="1297952773364" x="16603.96"
        y="961.84357" width="1.0" height="25.0" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/340fb3a17f0001016203171ee96598cc</URIString>
        <point1 x="16604.46" y="962.34357"/>
        <point2 x="16604.46" y="986.34357"/>
        <ID1 xsi:type="node">3529</ID1>
        <ID2 xsi:type="node">3538</ID2>
    </child>
    <child ID="3540" label="AssetType" layerID="1"
        created="1297952795403" x="16783.367" y="994.84357" width="68.0"
        height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/340fb3a17f0001016203171efaeeb79b</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3541" label="TaskTemplate" layerID="1"
        created="1297952801933" x="16772.367" y="1079.3435" width="90.0"
        height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/340fb3a17f0001016203171e5fdad64e</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3542" layerID="1" created="1297952801941" x="16816.867"
        y="1017.3435" width="1.0" height="62.49994" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/340fb3a17f0001016203171e87530ff3</URIString>
        <point1 x="16817.367" y="1017.84357"/>
        <point2 x="16817.367" y="1079.3435"/>
        <ID1 xsi:type="node">3540</ID1>
        <ID2 xsi:type="node">3541</ID2>
    </child>
    <child ID="3543" label="TaskTypes" layerID="1"
        created="1297952806342" x="16781.867" y="1149.3435" width="71.0"
        height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/340fb3a17f0001016203171e05062db5</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3544" layerID="1" created="1297952806349" x="16816.867"
        y="1101.8435" width="1.0" height="48.0" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/340fb3a27f0001016203171ee4b4bb51</URIString>
        <point1 x="16817.367" y="1102.3435"/>
        <point2 x="16817.367" y="1149.3435"/>
        <ID1 xsi:type="node">3541</ID1>
        <ID2 xsi:type="node">3543</ID2>
    </child>
    <child ID="3562" label="Non-implemented attributes" layerID="1"
        created="1298540076103" x="9222.637" y="68.42398" width="300.0"
        height="33.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
        <strokeColor>#404040</strokeColor>
        <textColor>#000000</textColor>
        <font>SansSerif-plain-14</font>
        <URIString>http://vue.tufts.edu/rdf/resource/570552607f0001016f0e090494a33c8f</URIString>
        <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Non-implemented attributes
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
        <label>Non-implemented attributes</label>
    </child>
    <child ID="3563" layerID="1" created="1298540076103" x="9187.644"
        y="68.42398" width="29.0" height="23.0" strokeWidth="1.0"
        autoSized="false" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/570552617f0001016f0e09045f703f7f</URIString>
        <shape xsi:type="ellipse"/>
    </child>
    <child ID="3565" label="EnvironmentVariables" layerID="1"
        created="1299605590562" x="10929.242" y="110.968506"
        width="164.0" height="46.25" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#ECFFD4</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/9a259fb57f0001010772fbca6a52863b</URIString>
        <child ID="3569" label="stalker repository_path"
            created="1299605618563" x="34.0" y="23.0" width="136.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/9a259fb67f0001010772fbcafae74713</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3570" label="POSSIBLE MIXINS" layerID="1"
        created="1305333600438" x="6978.449" y="684.5101"
        width="501.333" height="102.66669" strokeWidth="1.0"
        autoSized="false" xsi:type="node">
        <fillColor>#B5B995</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-18</font>
        <URIString>http://vue.tufts.edu/rdf/resource/ebf7261b7f0001012e9d478bae99d2a2</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3571" label="AVAILABLE MIXINS" layerID="1"
        created="1305333615676" x="6940.4497" y="1220.51"
        width="501.333" height="102.66669" strokeWidth="1.0"
        autoSized="false" xsi:type="node">
        <fillColor>#B5B995</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-18</font>
        <URIString>http://vue.tufts.edu/rdf/resource/ebf7261b7f0001012e9d478b95c14742</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3605" layerID="1" created="1305784780169" x="8776.104"
        y="751.25" width="714.76465" height="77.46875" strokeWidth="1.0"
        autoSized="false" controlCount="2" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/06dc9a487f000101049ada8600a11492</URIString>
        <point1 x="8776.604" y="751.75"/>
        <point2 x="9490.368" y="828.21875"/>
        <ID1 xsi:type="node">1887</ID1>
        <ID2 xsi:type="node">2397</ID2>
        <ctrlPoint0 x="8776.836" y="795.92163" xsi:type="point"/>
        <ctrlPoint1 x="9495.639" y="753.4334" xsi:type="point"/>
    </child>
    <child ID="3615"
        label="The duty of TypeEntities and the derived classes&#xa;are not clear enough?"
        layerID="1" created="1306189397240" x="17124.117" y="572.84357"
        width="349.0" height="96.0" strokeWidth="1.0" autoSized="false" xsi:type="node">
        <fillColor>#FCDBD9</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/1ef854967f000101578cef183082fb96</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3620"
        label="Enhance the design&#xa;by changign the TypeEntity&#xa;to Type and adding two attributes&#xa;like the name of the type (like Character)&#xa;and the target entity_type (like Asset)"
        layerID="1" created="1306189451069" x="17114.617" y="753.34357"
        width="230.0" height="83.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/1ef854967f000101578cef1825e67a00</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3621" layerID="1" created="1306189451076" x="17245.574"
        y="668.34375" width="34.507812" height="85.5" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/1ef854967f000101578cef184d47f17e</URIString>
        <point1 x="17279.582" y="668.84375"/>
        <point2 x="17246.074" y="753.34375"/>
        <ID1 xsi:type="node">3615</ID1>
        <ID2 xsi:type="node">3620</ID2>
    </child>
    <child ID="3622"
        label="Liked it very much,&#xa;what could be the&#xa;implications?"
        layerID="1" created="1306189507844" x="17174.117" y="878.34357"
        width="111.0" height="53.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/1ef854967f000101578cef183f94113a</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3623" layerID="1" created="1306189507862" x="17229.117"
        y="835.84357" width="1.0" height="43.0" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/1ef854977f000101578cef18f1f525b9</URIString>
        <point1 x="17229.617" y="836.34357"/>
        <point2 x="17229.617" y="878.34357"/>
        <ID1 xsi:type="node">3620</ID1>
        <ID2 xsi:type="node">3622</ID2>
    </child>
    <child ID="3624"
        label="how would you relate&#xa;them with StructureTemplates"
        layerID="1" created="1306189536026" x="17142.117" y="961.3435"
        width="175.0" height="38.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/1ef854977f000101578cef1889e1b257</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3625" layerID="1" created="1306189536033" x="17229.117"
        y="930.8435" width="1.0" height="30.999939" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/1ef854977f000101578cef18b1a0a07f</URIString>
        <point1 x="17229.617" y="931.34357"/>
        <point2 x="17229.617" y="961.3435"/>
        <ID1 xsi:type="node">3622</ID1>
        <ID2 xsi:type="node">3624</ID2>
    </child>
    <child ID="3626"
        label="StructureTemplates can also&#xa;be defined with entity_types&#xa;so one can create a&#xa;StructureTemplate with a specific&#xa;template code for Asset entity_types"
        layerID="1" created="1306189558421" x="17126.117" y="1040.3435"
        width="207.0" height="83.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/1ef854977f000101578cef18dd82dd5a</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3627" layerID="1" created="1306189558428" x="17229.117"
        y="998.8435" width="1.0" height="42.0" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/1ef854977f000101578cef18a1f46511</URIString>
        <point1 x="17229.617" y="999.3435"/>
        <point2 x="17229.617" y="1040.3435"/>
        <ID1 xsi:type="node">3624</ID1>
        <ID2 xsi:type="node">3626</ID2>
    </child>
    <child ID="3634" label="POSSIBLE DATABASE SCENARIOS" layerID="1"
        created="1306708027747" x="5165.7827" y="-24.82308"
        width="512.0" height="138.66667" strokeWidth="1.0"
        autoSized="false" xsi:type="node">
        <fillColor>#B5B995</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-18</font>
        <URIString>http://vue.tufts.edu/rdf/resource/3de32a157f0001010892949f5a269b82</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3637" label="Single Database" layerID="1"
        created="1306708073182" x="5057.783" y="219.17693"
        width="213.3335" height="85.33333" strokeWidth="1.0"
        autoSized="false" xsi:type="node">
        <fillColor>#FCDBD9</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/3de32a167f0001010892949fd04bdf89</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3638" label="Per Project Database" layerID="1"
        created="1306708082193" x="5574.449" y="220.51028"
        width="206.33398" height="85.33334" strokeWidth="1.0"
        autoSized="false" xsi:type="node">
        <fillColor>#FCDBD9</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/3de32a167f0001010892949f8dd61baf</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3639"
        label="single SQLite3 database&#xa;file on project root"
        layerID="1" created="1306708117853" x="5604.7827" y="370.01025"
        width="149.0" height="38.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFD8C</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/3de32a227f0001010892949f64b52421</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3640" layerID="1" created="1306708117868" x="5677.681"
        y="305.34375" width="1.8496094" height="65.15625"
        strokeWidth="1.0" autoSized="false" controlCount="0"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/3de32a227f0001010892949ff8a4b051</URIString>
        <point1 x="5678.181" y="305.84375"/>
        <point2 x="5679.031" y="370.0"/>
        <ID1 xsi:type="node">3638</ID1>
        <ID2 xsi:type="node">3639</ID2>
    </child>
    <child ID="3641" label="easy to backup/archive" layerID="1"
        created="1306708144463" x="5498.449" y="474.34357" width="170.0"
        height="44.5" strokeWidth="1.0" autoSized="false" xsi:type="node">
        <fillColor>#9DDB53</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/3de32a237f0001010892949f22be8b46</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3642" layerID="1" created="1306708144493" x="5602.769"
        y="407.51025" width="60.088867" height="67.333496"
        strokeWidth="1.0" autoSized="false" controlCount="0"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/3de32a237f0001010892949fe0834314</URIString>
        <point1 x="5662.358" y="408.01025"/>
        <point2 x="5603.269" y="474.34375"/>
        <ID1 xsi:type="node">3639</ID1>
        <ID2 xsi:type="node">3641</ID2>
    </child>
    <child ID="3643"
        label="single databaes on a&#xa;dedicated database&#xa;server"
        layerID="1" created="1306708158266" x="5094.449" y="351.34357"
        width="128.0" height="53.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFD8C</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/3de32a247f0001010892949f1625e7bd</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3644" layerID="1" created="1306708158290" x="5159.3203"
        y="304.0078" width="3.4228516" height="47.835938"
        strokeWidth="1.0" autoSized="false" controlCount="0"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/3de32a247f0001010892949fd1aa4890</URIString>
        <point1 x="5162.243" y="304.5078"/>
        <point2 x="5159.8203" y="351.34375"/>
        <ID1 xsi:type="node">3637</ID1>
        <ID2 xsi:type="node">3643</ID2>
    </child>
    <child ID="3645" label="fast" layerID="1" created="1306708173021"
        x="4943.449" y="447.34357" width="68.0" height="32.5"
        strokeWidth="1.0" autoSized="false" xsi:type="node">
        <fillColor>#9DDB53</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/3de32a257f0001010892949f7cb2b9f1</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3646" layerID="1" created="1306708173028" x="5006.801"
        y="403.56958" width="96.79053" height="46.381348"
        strokeWidth="1.0" autoSized="false" controlCount="0"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/3de32a257f0001010892949f67f3b25a</URIString>
        <point1 x="5103.092" y="404.06958"/>
        <point2 x="5007.3013" y="449.45093"/>
        <ID1 xsi:type="node">3643</ID1>
        <ID2 xsi:type="node">3645</ID2>
    </child>
    <child ID="3647" label="error prone" layerID="1"
        created="1306708179566" x="5019.449" y="522.34357" width="76.0"
        height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#9DDB53</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/3de32a267f0001010892949fcba370c0</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3648" layerID="1" created="1306708179577" x="5064.3945"
        y="403.84424" width="77.39746" height="118.99951"
        strokeWidth="1.0" autoSized="false" controlCount="0"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/3de32a267f0001010892949f5c46788f</URIString>
        <point1 x="5141.292" y="404.34424"/>
        <point2 x="5064.8945" y="522.34375"/>
        <ID1 xsi:type="node">3643</ID1>
        <ID2 xsi:type="node">3647</ID2>
    </child>
    <child ID="3649" label="database lives with&#xa;the studio itself"
        layerID="1" created="1306708184176" x="5121.449" y="517.34357"
        width="120.0" height="38.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#9DDB53</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/3de32a277f0001010892949f8996f92c</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3650" layerID="1" created="1306708184185" x="5161.795"
        y="403.84375" width="17.396973" height="114.0" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/3de32a277f0001010892949f2fb8309b</URIString>
        <point1 x="5162.295" y="404.34375"/>
        <point2 x="5178.692" y="517.34375"/>
        <ID1 xsi:type="node">3643</ID1>
        <ID2 xsi:type="node">3649</ID2>
    </child>
    <child ID="3651" label="hard to&#xa;backup/archive" layerID="1"
        created="1306708203854" x="5253.449" y="472.34357" width="98.0"
        height="38.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#9DDB53</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/3de32a287f0001010892949f44fa5d31</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3652" layerID="1" created="1306708203862" x="5191.5703"
        y="403.8435" width="87.27344" height="69.000244"
        strokeWidth="1.0" autoSized="false" controlCount="0"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/3de32a287f0001010892949fa01e4444</URIString>
        <point1 x="5192.0703" y="404.3435"/>
        <point2 x="5278.3438" y="472.34375"/>
        <ID1 xsi:type="node">3643</ID1>
        <ID2 xsi:type="node">3651</ID2>
    </child>
    <child ID="3653" label="easy to manage" layerID="1"
        created="1306708256008" x="5711.449" y="512.34357" width="101.0"
        height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#9DDB53</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/3de32a297f0001010892949ff4d93d34</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3654" layerID="1" created="1306708256016" x="5690.4316"
        y="407.50977" width="64.967285" height="105.333984"
        strokeWidth="1.0" autoSized="false" controlCount="0"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/3de32a297f0001010892949fa023f701</URIString>
        <point1 x="5690.9316" y="408.00977"/>
        <point2 x="5754.899" y="512.34375"/>
        <ID1 xsi:type="node">3639</ID1>
        <ID2 xsi:type="node">3653</ID2>
    </child>
    <child ID="3655" label="more open to&#xa;new features" layerID="1"
        created="1306708263231" x="5832.449" y="446.34357" width="86.0"
        height="38.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#9DDB53</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/3de32a2a7f0001010892949f44b54c39</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3656" layerID="1" created="1306708263250" x="5727.6104"
        y="407.51025" width="108.01221" height="42.641113"
        strokeWidth="1.0" autoSized="false" controlCount="0"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/3de32a2a7f0001010892949fb8a36c74</URIString>
        <point1 x="5728.11" y="408.01025"/>
        <point2 x="5835.122" y="449.65137"/>
        <ID1 xsi:type="node">3639</ID1>
        <ID2 xsi:type="node">3655</ID2>
    </child>
    <child ID="3657"
        label="needs a second&#xa;database for the&#xa;persistent data&#xa;like users,&#xa;departments&#xa;etc."
        layerID="1" created="1306708290786" x="5846.449" y="301.34357"
        width="104.0" height="98.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#9DDB53</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/3de32a2a7f0001010892949febaf0c39</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3658" layerID="1" created="1306708290794" x="5752.4297"
        y="359.0177" width="94.52002" height="17.49945"
        strokeWidth="1.0" autoSized="false" controlCount="0"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/3de32a2b7f0001010892949f2e76a112</URIString>
        <point1 x="5752.929" y="376.01715"/>
        <point2 x="5846.449" y="359.5177"/>
        <ID1 xsi:type="node">3639</ID1>
        <ID2 xsi:type="node">3657</ID2>
    </child>
    <child ID="3659" layerID="1" created="1307711459168" x="8001.266"
        y="751.2031" width="763.2661" height="596.7656"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/79af23367f0001017997f53852f91300</URIString>
        <point1 x="8764.032" y="751.7031"/>
        <point2 x="8001.766" y="1347.4688"/>
        <ID1 xsi:type="node">1887</ID1>
        <ID2 xsi:type="node">1402</ID2>
        <ctrlPoint0 x="8666.721" y="1320.9504" xsi:type="point"/>
        <ctrlPoint1 x="7998.3296" y="1226.3097" xsi:type="point"/>
    </child>
    <child ID="3669" label="group_acls" layerID="1"
        created="1307720099855" x="7704.449" y="1276.6936"
        width="148.95" height="96.2" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#F4E5FF</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/7a3132fe7f0001017997f538a8ecadd7</URIString>
        <child ID="3670" label="gid | INTEGER" created="1307720099856"
            x="48.85" y="38.85" width="90.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F4E5FF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/7a3132ff7f0001017997f538ca751ab1</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3982" label="acl_id | INTEGER"
            created="1337334133016" x="48.85" y="59.1" width="106.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F4E5FF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/5f58e5f87f0001011df07586553401a4</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape xsi:type="hexagon"/>
    </child>
    <child ID="3672" layerID="1" created="1307720106243" x="7847.4253"
        y="1316.4622" width="88.82666" height="68.62268"
        strokeWidth="1.0" strokeStyle="4" autoSized="false"
        controlCount="2" arrowState="3" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/7a31df207f0001017997f538762e76d7</URIString>
        <point1 x="7847.9253" y="1316.9622"/>
        <point2 x="7935.752" y="1384.5848"/>
        <ID1 xsi:type="node">3669</ID1>
        <ID2 xsi:type="node">1402</ID2>
        <ctrlPoint0 x="7887.2637" y="1312.4973" xsi:type="point"/>
        <ctrlPoint1 x="7869.869" y="1369.6831" xsi:type="point"/>
    </child>
    <child ID="3673" layerID="1" created="1307720113696" x="7833.7964"
        y="1351.625" width="102.45557" height="63.256104"
        strokeWidth="1.0" strokeStyle="4" autoSized="false"
        controlCount="2" arrowState="3" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/7a31df207f0001017997f538fcd5851b</URIString>
        <point1 x="7834.2964" y="1352.125"/>
        <point2 x="7935.752" y="1411.9473"/>
        <ID1 xsi:type="node">3669</ID1>
        <ID2 xsi:type="node">1402</ID2>
        <ctrlPoint0 x="7869.869" y="1369.6831" xsi:type="point"/>
        <ctrlPoint1 x="7852.475" y="1426.8691" xsi:type="point"/>
    </child>
    <child ID="3689" label="TaskDependRelation" layerID="1"
        created="1307887603557" x="11112.949" y="110.968506"
        width="180.0" height="139.25" strokeWidth="1.0"
        autoSized="false" xsi:type="node">
        <fillColor>#ECFFD4</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/842e31877f0001015d3258770684f65b</URIString>
        <child ID="3690" label="task | ONE | TASK"
            created="1307887603557" x="34.0" y="23.0" width="108.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/842e31877f0001015d32587797aa8b4f</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3691" label="depends | MANY | TASK"
            created="1307887603558" x="34.0" y="43.25" width="140.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/842e31877f0001015d32587701245b1b</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3692" label="type | ENUM" created="1307887603558"
            x="34.0" y="63.5" width="77.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/842e31887f0001015d325877a276acf6</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3693" label="lag | TIMEDELTA" created="1307887603558"
            x="34.0" y="83.75" width="102.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/842e31887f0001015d3258776ba432ce</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3694" created="1307887603558" x="34.0" y="104.0"
            width="29.0" height="23.0" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/842e31887f0001015d325877eaa6dd71</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3695"
        label="type Enums:&#xa;0: start_start&#xa;1: start_end&#xa;2: end_to_end"
        layerID="1" created="1307887603557" x="11157.449" y="262.5935"
        width="91.0" height="68.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#83CEFF</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/842e31887f0001015d325877f78ac9f5</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3696" layerID="1" created="1307887603557" x="11202.449"
        y="249.7185" width="1.0" height="13.375" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/842e31887f0001015d3258774eee4aba</URIString>
        <point1 x="11202.949" y="250.2185"/>
        <point2 x="11202.949" y="262.5935"/>
        <ID1 xsi:type="node">3689</ID1>
        <ID2 xsi:type="node">3695</ID2>
    </child>
    <child ID="3707" label="ResourceData" layerID="1"
        created="1308436257183" x="11313.156" y="110.968506"
        width="158.25" height="86.75" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#ECFFD4</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/a4e147e67f0001015c382de564be1104</URIString>
        <child ID="3708" label="allocate | MANY | USER"
            created="1308436257184" x="34.0" y="23.0" width="138.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/a4e147e77f0001015c382de55503d112</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3709" label="alternatives | MANY | USER"
            created="1308436257184" x="34.0" y="43.25" width="158.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/a4e147e87f0001015c382de59322ab1e</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3710" label="load_condition | ENUM"
            created="1308436257184" x="34.0" y="63.5" width="136.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/a4e147e87f0001015c382de54a61be40</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3718"
        label="Turn the status&#xa;&quot;Archived - ARCH&quot; for&#xa;archived projects."
        layerID="1" created="1309124113512" x="8041.093" y="2868.3438"
        width="134.0" height="53.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#83CEFF</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/cde1332d7f0001011f758c30b7215c18</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3719" layerID="1" created="1309124113529" x="8119.6025"
        y="2295.3281" width="260.4629" height="573.5156"
        strokeWidth="1.0" autoSized="false" controlCount="0"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/cde1332e7f0001011f758c30a99bec06</URIString>
        <point1 x="8379.565" y="2295.8281"/>
        <point2 x="8120.1025" y="2868.3438"/>
        <ID1 xsi:type="node">1166</ID1>
        <ID2 xsi:type="node">3718</ID2>
    </child>
    <child ID="3720" label="maya file&#xa;(ma)" layerID="1"
        created="1309208518501" x="7526.949" y="2236.8435" width="60.0"
        height="38.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/d2e9e3e57f0001011f758c305fb99f9c</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3721" label="can have other&#xa;ma files" layerID="1"
        created="1309208528127" x="7394.949" y="2335.8435" width="91.0"
        height="38.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/d2e9e3e57f0001011f758c30c1da54d4</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3723"
        label="can have photoshop&#xa;illustrasions&#xa;(Asset)"
        layerID="1" created="1309208538064" x="7495.949" y="2351.8435"
        width="122.0" height="53.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/d2e9e3e57f0001011f758c309827a705</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3725"
        label="can be exported&#xa;to other formats&#xa;like&#xa;fbx, obj"
        layerID="1" created="1309208570727" x="7634.449" y="2310.3435"
        width="98.0" height="68.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/d2e9e3e57f0001011f758c30cd7fa2ed</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3726" layerID="1" created="1309208570736" x="7581.4316"
        y="2272.822" width="57.290527" height="40.38086"
        strokeWidth="1.0" autoSized="false" controlCount="0"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/d2e9e3e67f0001011f758c30307e5d6e</URIString>
        <point1 x="7581.932" y="2273.322"/>
        <point2 x="7638.2227" y="2312.703"/>
        <ID1 xsi:type="node">3720</ID1>
        <ID2 xsi:type="node">3725</ID2>
    </child>
    <child ID="3727" layerID="1" created="1309208576692" x="7556.449"
        y="2274.3435" width="1.0" height="78.0" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/d2e9e3e67f0001011f758c30a9bc47e0</URIString>
        <point1 x="7556.949" y="2274.8435"/>
        <point2 x="7556.949" y="2351.8435"/>
        <ID1 xsi:type="node">3720</ID1>
        <ID2 xsi:type="node">3723</ID2>
    </child>
    <child ID="3728" layerID="1" created="1309208578201" x="7462.3076"
        y="2273.9585" width="73.23633" height="62.385254"
        strokeWidth="1.0" autoSized="false" controlCount="0"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/d2e9e3e67f0001011f758c30f0fdbd62</URIString>
        <point1 x="7535.044" y="2274.4585"/>
        <point2 x="7462.8076" y="2335.8438"/>
        <ID1 xsi:type="node">3720</ID1>
        <ID2 xsi:type="node">3721</ID2>
    </child>
    <child ID="3733" label="TaskMixin" layerID="1"
        created="1309652883304" x="6843.949" y="2276.8435" width="93.0"
        height="66.5" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/ed9b76207f0001014f9078ed4e765585</URIString>
        <child ID="3735" label="project" created="1309652905862"
            x="34.0" y="23.0" width="48.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/ed9b76207f0001014f9078ed1d81a9f5</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3734" label="tasks" created="1309652897029" x="34.0"
            y="43.25" width="41.0" height="23.0" strokeWidth="1.0"
            autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/ed9b76217f0001014f9078ed56d6b831</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3736" label="Task" layerID="1" created="1309652914965"
        x="7079.949" y="2282.5935" width="78.0" height="46.25"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/ed9b76217f0001014f9078ede683b13e</URIString>
        <child ID="3737" label="task_of" created="1309652914966"
            x="34.0" y="23.0" width="51.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/ed9b76217f0001014f9078ed28a8095a</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3739" label="update" layerID="1" created="1309652940549"
        x="6908.1416" y="2285.199" width="210.33057" height="46.20288"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/ed9b76217f0001014f9078edc617eb94</URIString>
        <point1 x="7117.972" y="2306.5593"/>
        <point2 x="6908.6416" y="2330.1287"/>
        <ID1 xsi:type="node">3737</ID1>
        <ID2 xsi:type="node">3734</ID2>
        <ctrlPoint0 x="6996.421" y="2244.9138" xsi:type="point"/>
        <ctrlPoint1 x="7021.6943" y="2340.5369" xsi:type="point"/>
    </child>
    <child ID="3740" label="update" layerID="1" created="1309652952811"
        x="6967.7314" y="2322.3438" width="166.38525" height="124.60376"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/ed9b76227f0001014f9078ede397f58b</URIString>
        <point1 x="6968.2314" y="2443.6016"/>
        <point2 x="7133.6167" y="2322.8438"/>
        <ID1 xsi:type="node">3741</ID1>
        <ID2 xsi:type="node">3737</ID2>
        <ctrlPoint0 x="7021.1436" y="2455.0156" xsi:type="point"/>
        <ctrlPoint1 x="7140.914" y="2438.8425" xsi:type="point"/>
    </child>
    <child ID="3741" label="ValidatedList" layerID="1"
        created="1309652986125" x="6888.949" y="2423.8435" width="82.0"
        height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FDE888</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/ed9b76227f0001014f9078ed41e7d3ff</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3742" layerID="1" created="1309652986128" x="6890.7695"
        y="2336.8438" width="35.59912" height="87.5" strokeWidth="1.0"
        autoSized="false" controlCount="2" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/ed9b76227f0001014f9078ed447ef19a</URIString>
        <point1 x="6892.318" y="2337.3438"/>
        <point2 x="6925.8687" y="2423.8438"/>
        <ID1 xsi:type="node">3734</ID1>
        <ID2 xsi:type="node">3741</ID2>
        <ctrlPoint0 x="6886.4326" y="2387.7788" xsi:type="point"/>
        <ctrlPoint1 x="6906.3545" y="2368.8518" xsi:type="point"/>
    </child>
    <child ID="3745" layerID="1" created="1309657617797" x="9187.644"
        y="35.02394" width="11.0" height="22.0" strokeWidth="1.0"
        autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/edafea3d7f0001014f9078ed989a19b5</URIString>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="3746" label="Declarative SOM (Completed with Tests)"
        layerID="1" created="1309657617797" x="9222.637" y="37.52394"
        width="300.0" height="18.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
        <strokeColor>#404040</strokeColor>
        <textColor>#000000</textColor>
        <font>SansSerif-plain-14</font>
        <URIString>http://vue.tufts.edu/rdf/resource/edafea3d7f0001014f9078edca304b53</URIString>
        <richText>&lt;html&gt;
  &lt;head&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Declarative SOM (Completed with Tests)
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
        <label>Declarative SOM (Completed with Tests)</label>
    </child>
    <child ID="3784" layerID="1" created="1310511051896" x="9189.595"
        y="323.87396" width="29.0" height="19.99997" strokeWidth="1.0"
        autoSized="false" xsi:type="node">
        <fillColor>#AF55F4</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/208db6a77f000101264d107e132cc774</URIString>
        <shape xsi:type="diamond"/>
    </child>
    <child ID="3785" label="strictly_typed" layerID="1"
        created="1310511081221" x="9222.637" y="325.37393" width="300.0"
        height="17.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
        <strokeColor>#404040</strokeColor>
        <textColor>#000000</textColor>
        <font>SansSerif-plain-14</font>
        <URIString>http://vue.tufts.edu/rdf/resource/208db6a77f000101264d107ea800a149</URIString>
        <richText>&lt;html&gt;
  &lt;head&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      strictly_typed
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
        <label>strictly_typed</label>
    </child>
    <child ID="3793" label="ProjectMixin" layerID="1"
        created="1313059262614" x="7503.245" y="1381.2302"
        width="150.75" height="46.25" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/b8701a037f00010133854518f97ed189</URIString>
        <child ID="3794" label="project | ONE | PROJECT"
            created="1313059262615" x="34.0" y="23.0" width="148.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/b8701a047f000101338545189002cbe7</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="3797" label="ProjectMixin" layerID="1"
        created="1313059299166" x="7529.449" y="1353.2908" width="82.0"
        height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FF66CC</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/b8701a047f00010133854518c82767d8</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3804" layerID="1" created="1315128486050" x="9187.644"
        y="4.4239693" width="11.0" height="22.0" strokeWidth="1.0"
        autoSized="true" xsi:type="node">
        <fillColor>#D4FF00</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/33c552027f0001017e6d32099d199fb8</URIString>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="3805"
        label="Declarative SOM (Started or Partially moved)" layerID="1"
        created="1315128486050" x="9222.637" y="6.9239655" width="300.0"
        height="18.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
        <strokeColor>#404040</strokeColor>
        <textColor>#000000</textColor>
        <font>SansSerif-plain-14</font>
        <URIString>http://vue.tufts.edu/rdf/resource/33c552037f0001017e6d320901980c9c</URIString>
        <richText>&lt;html&gt;
  &lt;head&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Declarative SOM (Started or Partially moved)
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
        <label>Declarative SOM (Started or Partially moved)</label>
    </child>
    <child ID="3807" layerID="1" created="1315218792592" x="8477.472"
        y="751.21094" width="314.4541" height="1107.5391"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/3ed254137f0001017e6d320990e92d47</URIString>
        <point1 x="8781.481" y="751.71094"/>
        <point2 x="8477.972" y="1858.25"/>
        <ID1 xsi:type="node">1887</ID1>
        <ID2 xsi:type="node">1166</ID2>
        <ctrlPoint0 x="8857.064" y="1778.6975" xsi:type="point"/>
        <ctrlPoint1 x="8477.463" y="1716.7916" xsi:type="point"/>
    </child>
    <child ID="3808" layerID="1" created="1315218794336" x="8588.786"
        y="2225.625" width="169.15723" height="250.15625"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/3ed254137f0001017e6d3209f4c83f0b</URIString>
        <point1 x="8757.443" y="2226.125"/>
        <point2 x="8589.286" y="2475.2812"/>
        <ID1 xsi:type="node">1205</ID1>
        <ID2 xsi:type="node">1176</ID2>
        <ctrlPoint0 x="8759.088" y="2424.922" xsi:type="point"/>
        <ctrlPoint1 x="8592.151" y="2380.4355" xsi:type="point"/>
    </child>
    <child ID="3809" layerID="1" created="1315218796042" x="8755.995"
        y="2225.75" width="39.566406" height="250.03125"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/3ed254137f0001017e6d32091356ed17</URIString>
        <point1 x="8756.495" y="2226.25"/>
        <point2 x="8795.062" y="2475.2812"/>
        <ID1 xsi:type="node">1205</ID1>
        <ID2 xsi:type="node">1185</ID2>
        <ctrlPoint0 x="8757.103" y="2421.8352" xsi:type="point"/>
        <ctrlPoint1 x="8789.982" y="2369.506" xsi:type="point"/>
    </child>
    <child ID="3810" layerID="1" created="1315218797063" x="8757.566"
        y="2225.5625" width="251.97852" height="250.21484"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/3ed254147f0001017e6d32092876fa0e</URIString>
        <point1 x="8758.066" y="2226.0625"/>
        <point2 x="9009.045" y="2475.2773"/>
        <ID1 xsi:type="node">1205</ID1>
        <ID2 xsi:type="node">1088</ID2>
        <ctrlPoint0 x="8760.34" y="2421.2244" xsi:type="point"/>
        <ctrlPoint1 x="8998.261" y="2416.4392" xsi:type="point"/>
    </child>
    <child ID="3815" label="Notifications" layerID="1"
        created="1316728436014" x="9564.924" y="1249.1434" width="165.0"
        height="86.75" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#ECFFD4</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/9323191a7f0001010820c09b268bea70</URIString>
        <child ID="3817" label="user | ONE | USER"
            created="1316728436017" x="34.0" y="23.0" width="114.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/9323191c7f0001010820c09b48ba2648</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3818" label="entity | ONE | SIMPLEENTITY"
            created="1316728436018" x="34.0" y="43.25" width="167.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/9323191c7f0001010820c09b3fdbd157</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3821" label="action | ENUM" created="1316728511464"
            x="34.0" y="63.5" width="88.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/9323191d7f0001010820c09b3d11ba97</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3820" layerID="1" created="1316728444997" x="8776.768"
        y="751.1875" width="858.1592" height="498.45508"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/9323191e7f0001010820c09b0d9f2466</URIString>
        <point1 x="8777.268" y="751.6875"/>
        <point2 x="9634.427" y="1249.1426"/>
        <ID1 xsi:type="node">1887</ID1>
        <ID2 xsi:type="node">3815</ID2>
        <ctrlPoint0 x="8785.653" y="1327.8733" xsi:type="point"/>
        <ctrlPoint1 x="9598.088" y="1127.8678" xsi:type="point"/>
    </child>
    <child ID="3822" label="TargetEntityTypeMixin" layerID="1"
        created="1316972313110" x="6829.116" y="1680.6133" width="164.0"
        height="46.25" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/a1ac43f57f0001010820c09b897a0e17</URIString>
        <child ID="3823" label="target_entity_type | STRING"
            created="1316972313111" x="34.0" y="23.0" width="160.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/a1ac43f67f0001010820c09ba9d40681</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="3824" label="TargetEntityTypeMixin" layerID="1"
        created="1316972313110" x="6843.116" y="1652.6738" width="136.0"
        height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FF66CC</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/a1ac43f77f0001010820c09b3959f466</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3834" label="Ticket" layerID="1" created="1336638487865"
        x="11373.065" y="428.9685" width="177.0" height="167.75"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#ECFFD4</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/35fa8d207f000101663ffd1486c76cc8</URIString>
        <child ID="3835" label="StatusMixin" created="1336638487866"
            x="34.0" y="23.0" width="78.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fa8d217f000101663ffd14e3cb886c</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3836" label="project | ONE | PROJECT"
            created="1336638487866" x="34.0" y="43.25" width="148.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fa8d237f000101663ffd14a0656966</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3837" label="assigned_to | ONE | USER"
            created="1336638487867" x="34.0" y="63.5" width="158.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fa8d247f000101663ffd1482254dd5</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3838" label="releases | MANY | RELEASE"
            created="1336638487867" x="34.0" y="83.75" width="165.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fa8d267f000101663ffd1401ed83d8</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3839" label="related_tickets | MANY | TICKET"
            created="1336638487867" x="34.0" y="104.0" width="183.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fa8d267f000101663ffd140a50984d</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3840" label="revisions | MANY | REVISION"
            created="1336638487868" x="34.0" y="124.25" width="167.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fa8d277f000101663ffd14c571ff64</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3841" label="priority | INTEGER"
            created="1336638487868" x="34.0" y="144.5" width="109.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fa8d287f000101663ffd140a12fddf</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3842" label="Release" layerID="1" created="1336638487865"
        x="11142.399" y="428.9685" width="150.75" height="86.75"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#ECFFD4</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/35fa8d297f000101663ffd14556b3698</URIString>
        <child ID="3843" label="StatusMixin" created="1336638487869"
            x="34.0" y="23.0" width="78.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fa8d2a7f000101663ffd14e671d93e</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3844" label="project | ONE | PROJECT"
            created="1336638487869" x="34.0" y="43.25" width="148.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fa8d2b7f000101663ffd14fa90b1e9</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3845" label="tickets | MANY | TICKET"
            created="1336638487870" x="34.0" y="63.5" width="138.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fa8d2d7f000101663ffd144c7d1f8a</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3846" label="Revision" layerID="1"
        created="1336638487865" x="11596.315" y="428.9685"
        width="198.75" height="107.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#ECFFD4</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/35fa8d2e7f000101663ffd1499fddc91</URIString>
        <child ID="3847" label="StatusMixin" created="1336638487870"
            x="34.0" y="23.0" width="78.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fa8d2f7f000101663ffd1440611e1b</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3848" label="project | ONE | PROJECT"
            created="1336638487871" x="34.0" y="43.25" width="148.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fa8d307f000101663ffd14fe9a5dcc</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3849" label="tickets | MANY | TICKET"
            created="1336638487871" x="34.0" y="63.5" width="138.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fa8d317f000101663ffd14fbffc76b</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3850" label="related_revisions | MANY | REVISION"
            created="1336638487872" x="34.0" y="83.75" width="212.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FDE888</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fa8d317f000101663ffd149a6733df</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3851" label="revision_revisions" layerID="1"
        created="1336638487865" x="11854.948" y="537.2517"
        width="214.55" height="107.8" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#F4E5FF</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/35fa8d337f000101663ffd141a200f3e</URIString>
        <child ID="3852" label="source_revision_id | INTEGER"
            created="1336638487872" x="54.65" y="44.65" width="178.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F4E5FF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fa8d337f000101663ffd14d4a75b5e</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3853" label="target_revision_id | INTEGER"
            created="1336638487873" x="54.65" y="64.9" width="171.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F4E5FF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fa8d347f000101663ffd146f3c629b</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape xsi:type="hexagon"/>
    </child>
    <child ID="3854" layerID="1" created="1336638487865" x="11794.565"
        y="483.01587" width="122.3418" height="54.736084"
        strokeWidth="1.0" strokeStyle="4" autoSized="false"
        controlCount="2" arrowState="3" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/35fa8d367f000101663ffd14d3f7c399</URIString>
        <point1 x="11795.065" y="483.51587"/>
        <point2 x="11916.407" y="537.25195"/>
        <ID1 xsi:type="node">3846</ID1>
        <ID2 xsi:type="node">3851</ID2>
        <ctrlPoint0 x="11872.342" y="484.33032" xsi:type="point"/>
        <ctrlPoint1 x="11879.569" y="493.91528" xsi:type="point"/>
    </child>
    <child ID="3855" label="revision_tickets" layerID="1"
        created="1336638487865" x="11662.016" y="668.5852" width="174.3"
        height="101.3" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#F4E5FF</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/35fa8d377f000101663ffd140888ec46</URIString>
        <child ID="3856" label="revision_id | INTEGER"
            created="1336638487874" x="51.4" y="41.4" width="133.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F4E5FF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fa8d397f000101663ffd14395cbf10</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3857" label="ticket_id | INTEGER"
            created="1336638487874" x="51.4" y="61.65" width="118.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F4E5FF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fa8d3a7f000101663ffd141c8fb707</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape xsi:type="hexagon"/>
    </child>
    <child ID="3858" layerID="1" created="1336638487865" x="11757.042"
        y="535.46875" width="113.18359" height="39.73584"
        strokeWidth="1.0" strokeStyle="4" autoSized="false"
        controlCount="2" arrowState="3" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/35fa8d3b7f000101663ffd142dce9439</URIString>
        <point1 x="11757.542" y="535.96875"/>
        <point2 x="11869.726" y="574.7046"/>
        <ID1 xsi:type="node">3846</ID1>
        <ID2 xsi:type="node">3851</ID2>
        <ctrlPoint0 x="11791.079" y="564.9773" xsi:type="point"/>
        <ctrlPoint1 x="11804.223" y="563.0574" xsi:type="point"/>
    </child>
    <child ID="3859" layerID="1" created="1336638487865" x="11700.391"
        y="535.46875" width="21.248047" height="133.61719"
        strokeWidth="1.0" strokeStyle="4" autoSized="false"
        controlCount="2" arrowState="3" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/35fa8d3c7f000101663ffd148f53f000</URIString>
        <point1 x="11702.083" y="535.96875"/>
        <point2 x="11721.139" y="668.58594"/>
        <ID1 xsi:type="node">3846</ID1>
        <ID2 xsi:type="node">3855</ID2>
        <ctrlPoint0 x="11711.211" y="612.3513" xsi:type="point"/>
        <ctrlPoint1 x="11683.425" y="600.42896" xsi:type="point"/>
    </child>
    <child ID="3860" layerID="1" created="1336638487865" x="11549.565"
        y="552.9888" width="135.16895" height="138.14209"
        strokeWidth="1.0" strokeStyle="4" autoSized="false"
        controlCount="2" arrowState="3" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/35fa8d3c7f000101663ffd14e4057a6e</URIString>
        <point1 x="11550.065" y="553.4888"/>
        <point2 x="11684.234" y="690.63086"/>
        <ID1 xsi:type="node">3834</ID1>
        <ID2 xsi:type="node">3855</ID2>
        <ctrlPoint0 x="11660.66" y="604.282" xsi:type="point"/>
        <ctrlPoint1 x="11616.4" y="660.7468" xsi:type="point"/>
    </child>
    <child ID="3861" label="ticket_tickets" layerID="1"
        created="1336638487865" x="11390.865" y="715.41846"
        width="200.8" height="105.3" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#F4E5FF</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/35fa8d3d7f000101663ffd146e1c7fd0</URIString>
        <child ID="3862" label="source_ticket_id | INTEGER"
            created="1336638487876" x="53.4" y="43.4" width="163.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F4E5FF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fa8d3e7f000101663ffd1434ff22cf</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3863" label="target_ticket_id | INTEGER"
            created="1336638487877" x="53.4" y="63.65" width="156.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F4E5FF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fa8d3f7f000101663ffd14ab17b17b</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape xsi:type="hexagon"/>
    </child>
    <child ID="3864" layerID="1" created="1336638487865" x="11445.79"
        y="596.21875" width="33.62207" height="119.69922"
        strokeWidth="1.0" strokeStyle="4" autoSized="false"
        controlCount="2" arrowState="3" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/35fa8d3f7f000101663ffd14de61dc8e</URIString>
        <point1 x="11448.3" y="596.71875"/>
        <point2 x="11478.912" y="715.41797"/>
        <ID1 xsi:type="node">3834</ID1>
        <ID2 xsi:type="node">3861</ID2>
        <ctrlPoint0 x="11438.881" y="656.27026" xsi:type="point"/>
        <ctrlPoint1 x="11463.782" y="650.93774" xsi:type="point"/>
    </child>
    <child ID="3865" layerID="1" created="1336638487865" x="11500.055"
        y="596.21875" width="23.222656" height="119.69922"
        strokeWidth="1.0" strokeStyle="4" autoSized="false"
        controlCount="2" arrowState="3" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/35fa8d407f000101663ffd14dc0170fa</URIString>
        <point1 x="11507.829" y="596.71875"/>
        <point2 x="11500.555" y="715.41797"/>
        <ID1 xsi:type="node">3834</ID1>
        <ID2 xsi:type="node">3861</ID2>
        <ctrlPoint0 x="11540.185" y="655.3779" xsi:type="point"/>
        <ctrlPoint1 x="11512.05" y="650.25964" xsi:type="point"/>
    </child>
    <child ID="3866" label="release_tickets" layerID="1"
        created="1336638487865" x="11143.207" y="643.9021"
        width="173.25" height="101.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#F4E5FF</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/35fa8d417f000101663ffd14c9ff9af0</URIString>
        <child ID="3867" label="release_id | INTEGER"
            created="1336638487878" x="51.25" y="41.25" width="132.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F4E5FF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fa8d427f000101663ffd144441696d</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3868" label="ticket_id | INTEGER"
            created="1336638487879" x="51.25" y="61.5" width="118.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F4E5FF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fa8d427f000101663ffd14b37b7d18</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape xsi:type="hexagon"/>
    </child>
    <child ID="3869" layerID="1" created="1336638487865" x="11206.082"
        y="515.21875" width="36.143555" height="129.1836"
        strokeWidth="1.0" strokeStyle="4" autoSized="false"
        controlCount="2" arrowState="3" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/35fa8d447f000101663ffd14034c742d</URIString>
        <point1 x="11237.18" y="515.71875"/>
        <point2 x="11211.908" y="643.90234"/>
        <ID1 xsi:type="node">3842</ID1>
        <ID2 xsi:type="node">3866</ID2>
        <ctrlPoint0 x="11260.461" y="567.75757" xsi:type="point"/>
        <ctrlPoint1 x="11186.25" y="571.6116" xsi:type="point"/>
    </child>
    <child ID="3879" label="Ticket Statuses (StatusList)" layerID="1"
        created="1336638980471" x="10008.482" y="938.01013"
        width="197.0" height="127.25" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#83CEFF</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/35fa8d467f000101663ffd14b3fb0bee</URIString>
        <child ID="3890" label="New" created="1336639100878" x="34.0"
            y="23.0" width="36.0" height="23.0" strokeWidth="1.0"
            autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fa8d467f000101663ffd14eb6f869b</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3881" label="ReOpened" created="1336638989263"
            x="34.0" y="43.25" width="70.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fa8d477f000101663ffd143040c34c</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3884" label="Assigned" created="1336639009832"
            x="34.0" y="63.5" width="64.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fa8d477f000101663ffd14ebf5331e</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3886" label="Accepted" created="1336639027396"
            x="34.0" y="83.75" width="65.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fa8d497f000101663ffd145356577c</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3928" label="Closed" created="1336649098209" x="34.0"
            y="104.0" width="50.0" height="23.0" strokeWidth="1.0"
            autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/3680755a7f000101663ffd145d8f6033</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3880" layerID="1" created="1336638980480" x="9957.532"
        y="1064.7598" width="76.15234" height="65.79199"
        strokeWidth="1.0" autoSized="false" controlCount="0"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/35fa8d4a7f000101663ffd14b7708354</URIString>
        <point1 x="9958.032" y="1130.0518"/>
        <point2 x="10033.185" y="1065.2598"/>
        <ID1 xsi:type="node">3345</ID1>
        <ID2 xsi:type="node">3879</ID2>
    </child>
    <child ID="3893" label="Ticket Types (Type)" layerID="1"
        created="1336639200821" x="10062.615" y="1091.0936"
        width="149.0" height="66.5" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#8AEE95</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/35fa8d4b7f000101663ffd141146400f</URIString>
        <child ID="3894" label="Defect" created="1336639200821" x="34.0"
            y="23.0" width="48.0" height="23.0" strokeWidth="1.0"
            autoSized="true" xsi:type="node">
            <fillColor>#8AEE95</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fa8d4b7f000101663ffd142adf3f72</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3895" label="Enhancement" created="1336639200822"
            x="34.0" y="43.25" width="89.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#8AEE95</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/35fa8d4c7f000101663ffd14b437adda</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3900" layerID="1" created="1336639408389" x="9957.533"
        y="1146.6763" width="105.58301" height="33.05249"
        strokeWidth="1.0" autoSized="false" controlCount="0"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/35fa8d4d7f000101663ffd144d4a9602</URIString>
        <point1 x="9958.032" y="1179.2288"/>
        <point2 x="10062.615" y="1147.1763"/>
        <ID1 xsi:type="node">3345</ID1>
        <ID2 xsi:type="node">3893</ID2>
    </child>
    <child ID="3910"
        label="It is not possible&#xa;to store the users&#xa;in one database&#xa;and projects in another&#xa;because users will&#xa;be attached to another&#xa;session while projects&#xa;to another"
        layerID="1" created="1336640812078" x="5848.949" y="45.843506"
        width="266.0" height="239.0" strokeWidth="1.0" autoSized="false" xsi:type="node">
        <fillColor>#EA2218</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-18</font>
        <URIString>http://vue.tufts.edu/rdf/resource/3604033f7f000101663ffd145b469f06</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3912" layerID="1" created="1336640893817" x="5780.1895"
        y="207.59863" width="69.260254" height="22.943604"
        strokeWidth="1.0" autoSized="false" controlCount="0"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/360403407f000101663ffd1440a0efe3</URIString>
        <point1 x="5780.689" y="230.04224"/>
        <point2 x="5848.949" y="208.09863"/>
        <ID1 xsi:type="node">3638</ID1>
        <ID2 xsi:type="node">3910</ID2>
    </child>
    <child ID="3918" layerID="1" created="1336644686386" x="8774.722"
        y="751.1875" width="1095.9902" height="362.8125"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/363d47ef7f000101663ffd147aedb6bd</URIString>
        <point1 x="8775.222" y="751.6875"/>
        <point2 x="9870.212" y="1113.5"/>
        <ID1 xsi:type="node">1887</ID1>
        <ID2 xsi:type="node">3345</ID2>
        <ctrlPoint0 x="8767.172" y="1321.302" xsi:type="point"/>
        <ctrlPoint1 x="9871.733" y="905.60504" xsi:type="point"/>
    </child>
    <child ID="3921" label="Ticket Priorities" layerID="1"
        created="1336648644751" x="10066.114" y="1189.2603"
        width="129.0" height="127.25" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#E4E6D2</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/3678e9817f000101663ffd147e48304b</URIString>
        <child ID="3927" label="0 : trival" created="1336648704475"
            x="34.0" y="23.0" width="55.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#E4E6D2</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/3678e9837f000101663ffd145a0469c0</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3926" label="1 : minor" created="1336648697359"
            x="34.0" y="43.25" width="60.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#E4E6D2</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/3678e9827f000101663ffd144fb9e0f6</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3925" label="2 : major" created="1336648690011"
            x="34.0" y="63.5" width="60.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#E4E6D2</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/3678e9827f000101663ffd14e8afaa2a</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3923" label="3 : critical" created="1336648644752"
            x="34.0" y="83.75" width="66.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#E4E6D2</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/3678e9827f000101663ffd144ceb272f</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3922" label="4 : blocker" created="1336648644752"
            x="34.0" y="104.0" width="70.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#E4E6D2</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/3678e9817f000101663ffd149b7e6993</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3924" layerID="1" created="1336648648313" x="9957.532"
        y="1221.6255" width="109.08203" height="20.263794"
        strokeWidth="1.0" autoSized="false" controlCount="0"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/3678e9837f000101663ffd14af5fb80b</URIString>
        <point1 x="9958.032" y="1222.1254"/>
        <point2 x="10066.114" y="1241.3892"/>
        <ID1 xsi:type="node">3345</ID1>
        <ID2 xsi:type="node">3921</ID2>
    </child>
    <child ID="3933" label="TicketLog" layerID="1"
        created="1336649990599" x="10230.449" y="1163.9685"
        width="133.5" height="84.49998" strokeWidth="1.0"
        autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/36a21ef87f000101663ffd1411b60466</URIString>
        <child ID="3935" label="ticket | ONE | TICKET"
            created="1336649990599" x="34.0" y="23.0" width="125.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/36a21ef87f000101663ffd143ee68814</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3936" label="action | ENUM" created="1336649990600"
            x="34.0" y="43.25" width="88.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/36a21ef87f000101663ffd141c2db82d</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4048" created="1358112050934" x="34.0" y="63.5"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#30D643</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/3624aeb3c0a8000435fa8379e276ea76</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="3941" label="TicketLog_Operations" layerID="1"
        created="1336650037564" x="10470.449" y="1186.2185"
        width="283.0" height="128.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#E4E6D2</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/36a21ef97f000101663ffd1446677ed4</URIString>
        <child ID="3942" label="0 : accept" created="1336650037565"
            x="34.0" y="23.0" width="66.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#E4E6D2</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/36a21ef97f000101663ffd147086af45</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3943" label="1: reassign" created="1336650037565"
            x="34.0" y="43.25" width="74.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#E4E6D2</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/36a21ef97f000101663ffd14f97322da</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3944" label="2: resolve" created="1336650037565"
            x="34.0" y="63.5" width="66.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#E4E6D2</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/36a21ef97f000101663ffd14c8ebee7d</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3945" label="3 : reopen" created="1336650037566"
            x="34.0" y="83.75" width="67.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#E4E6D2</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/36a21efa7f000101663ffd145b9140d4</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3947"
            label="http://trac.edgewall.org/wiki/TracWorkflow"
            created="1336650124388" x="34.0" y="104.0" width="244.0"
            height="18.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
            <strokeColor>#404040</strokeColor>
            <textColor>#000000</textColor>
            <font>SansSerif-plain-14</font>
            <URIString>http://vue.tufts.edu/rdf/resource/36a21efa7f000101663ffd14cef1077e</URIString>
            <richText>&lt;html&gt;
  &lt;head style="color: #000000" color="#000000"&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      http://trac.edgewall.org/wiki/TracWorkflow
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
            <label>http://trac.edgewall.org/wiki/TracWorkflow</label>
        </child>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="3948" layerID="1" created="1336650146175" x="10363.449"
        y="1215.0498" width="107.5" height="15.887939" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/36a21efb7f000101663ffd1453cee0ae</URIString>
        <point1 x="10363.949" y="1215.5498"/>
        <point2 x="10470.449" y="1230.4377"/>
        <ID1 xsi:type="node">3933</ID1>
        <ID2 xsi:type="node">3941</ID2>
    </child>
    <child ID="3950" layerID="1" created="1336650182867" x="8808.297"
        y="486.59375" width="1637.0703" height="677.875"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/36a21efb7f000101663ffd1490b7da7d</URIString>
        <point1 x="8808.797" y="487.09375"/>
        <point2 x="10330.912" y="1163.9688"/>
        <ID1 xsi:type="node">1901</ID1>
        <ID2 xsi:type="node">3933</ID2>
        <ctrlPoint0 x="8813.76" y="648.2809" xsi:type="point"/>
        <ctrlPoint1 x="10954.907" y="381.96698" xsi:type="point"/>
    </child>
    <child ID="3952" label="ticket_related_tickets" layerID="1"
        created="1336650487013" x="9869.449" y="1344.0935" width="204.8"
        height="99.3" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#F4E5FF</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/36a21efb7f000101663ffd14e8a3c2c0</URIString>
        <child ID="3953" label="ticket_id" created="1336650487013"
            x="50.4" y="40.4" width="56.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F4E5FF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/36a21efb7f000101663ffd14a24b7fb3</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3954" label="related_ticket_id"
            created="1336650487014" x="50.4" y="60.65" width="101.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F4E5FF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/36a21efc7f000101663ffd14dc43d8ba</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape xsi:type="hexagon"/>
    </child>
    <child ID="3955" layerID="1" created="1336650498189" x="9839.641"
        y="1298.7266" width="96.32422" height="45.867188"
        strokeWidth="1.0" strokeStyle="4" autoSized="false"
        controlCount="2" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/36a21efc7f000101663ffd14e1a098c4</URIString>
        <point1 x="9842.318" y="1299.2266"/>
        <point2 x="9935.465" y="1344.0938"/>
        <ID1 xsi:type="node">3345</ID1>
        <ID2 xsi:type="node">3952</ID2>
        <ctrlPoint0 x="9826.693" y="1352.5485" xsi:type="point"/>
        <ctrlPoint1 x="9894.164" y="1287.735" xsi:type="point"/>
    </child>
    <child ID="3956" layerID="1" created="1336650521319" x="9957.532"
        y="1277.769" width="59.503906" height="66.82471"
        strokeWidth="1.0" strokeStyle="4" autoSized="false"
        controlCount="2" arrowState="1" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/36a21efc7f000101663ffd14b48e38a5</URIString>
        <point1 x="9958.032" y="1278.269"/>
        <point2 x="9999.215" y="1344.0938"/>
        <ID1 xsi:type="node">3345</ID1>
        <ID2 xsi:type="node">3952</ID2>
        <ctrlPoint0 x="9989.436" y="1303.7885" xsi:type="point"/>
        <ctrlPoint1 x="10045.057" y="1260.9203" xsi:type="point"/>
    </child>
    <child ID="3959" label="Base (declarative)" layerID="1"
        created="1337333910132" x="8749.698" y="46.818554" width="114.0"
        height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/5f526b557f0001011df075868f9d004d</URIString>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="3973" layerID="1" created="1337333931692" x="8805.149"
        y="69.3125" width="1.9042969" height="73.0625" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/5f526b567f0001011df0758616c29082</URIString>
        <point1 x="8806.554" y="69.8125"/>
        <point2 x="8805.649" y="141.875"/>
        <ID1 xsi:type="node">3959</ID1>
        <ID2 xsi:type="node">1901</ID2>
    </child>
    <child ID="3974" label="Permission" layerID="1"
        created="1337333960700" x="8149.949" y="155.94356"
        width="140.25" height="66.5" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/5f526b567f0001011df0758654c54329</URIString>
        <child ID="3976" label="action | STRING" created="1337333969654"
            x="34.0" y="23.0" width="98.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/5f526b577f0001011df07586a16bd21d</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3977" label="class_name | STRING"
            created="1337333981798" x="34.0" y="43.25" width="134.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/5f526b587f0001011df07586479df966</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="3975" layerID="1" created="1337333966973" x="8289.699"
        y="69.3186" width="465.95215" height="104.730225"
        strokeWidth="1.0" autoSized="false" controlCount="0"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/5f526b597f0001011df07586e10ce581</URIString>
        <point1 x="8755.151" y="69.8186"/>
        <point2 x="8290.199" y="173.54883"/>
        <ID1 xsi:type="node">3959</ID1>
        <ID2 xsi:type="node">3974</ID2>
    </child>
    <child ID="3978" label="ACL" layerID="1" created="1337333994306"
        x="8356.699" y="193.39357" width="176.25" height="66.5"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/5f526b5a7f0001011df075861aec2f3a</URIString>
        <child ID="3979" label="allowance | STRING"
            created="1337333994306" x="34.0" y="23.0" width="121.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/5f526b5b7f0001011df07586b2951812</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3980" label="permission | ONE | Permission"
            created="1337333994306" x="34.0" y="43.25" width="182.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/5f526b5c7f0001011df07586e0220ae9</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="3981" layerID="1" created="1337334052415" x="8515.807"
        y="69.3186" width="266.66797" height="124.57495"
        strokeWidth="1.0" autoSized="false" controlCount="0"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/5f526b5c7f0001011df07586103a1a93</URIString>
        <point1 x="8781.975" y="69.8186"/>
        <point2 x="8516.307" y="193.39355"/>
        <ID1 xsi:type="node">3959</ID1>
        <ID2 xsi:type="node">3978</ID2>
    </child>
    <child ID="3983" label="user_acls" layerID="1"
        created="1337334373492" x="7759.6743" y="1468.1436"
        width="148.95" height="96.2" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#F4E5FF</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/5f58e6257f0001011df0758673c76d34</URIString>
        <child ID="3984" label="uid | INTEGER" created="1337334373492"
            x="48.85" y="38.85" width="90.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F4E5FF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/5f58e6267f0001011df07586c5448109</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="3985" label="acl_id | INTEGER"
            created="1337334373492" x="48.85" y="59.1" width="106.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F4E5FF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/5f58e6267f0001011df07586a9836027</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape xsi:type="hexagon"/>
    </child>
    <child ID="3986" layerID="1" created="1337334391368" x="7903.3477"
        y="1508.908" width="184.63281" height="49.174072"
        strokeWidth="1.0" strokeStyle="4" autoSized="false"
        controlCount="2" arrowState="3" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/5f58e6277f0001011df07586777f0913</URIString>
        <point1 x="7903.8477" y="1509.408"/>
        <point2 x="8087.4805" y="1557.582"/>
        <ID1 xsi:type="node">3983</ID1>
        <ID2 xsi:type="node">1235</ID2>
        <ctrlPoint0 x="7973.189" y="1502.6073" xsi:type="point"/>
        <ctrlPoint1 x="7955.7944" y="1559.7931" xsi:type="point"/>
    </child>
    <child ID="3987" layerID="1" created="1337334391368" x="7892.0205"
        y="1538.7823" width="195.95996" height="56.41028"
        strokeWidth="1.0" strokeStyle="4" autoSized="false"
        controlCount="2" arrowState="3" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/5f58e6277f0001011df07586feb3e052</URIString>
        <point1 x="7892.5205" y="1539.2823"/>
        <point2 x="8087.4805" y="1581.8247"/>
        <ID1 xsi:type="node">3983</ID1>
        <ID2 xsi:type="node">1235</ID2>
        <ctrlPoint0 x="7952.594" y="1562.9932" xsi:type="point"/>
        <ctrlPoint1 x="7935.2" y="1620.1792" xsi:type="point"/>
    </child>
    <child ID="4009" label="CodeMixin" layerID="1"
        created="1358078140071" x="7051.2344" y="1680.6133"
        width="117.0" height="46.25" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/33fb2067c0a8000435fa8379ff5e5c4b</URIString>
        <child ID="4008" label="code | UNICODE" created="1358078127303"
            x="34.0" y="23.0" width="103.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/33fb2068c0a8000435fa8379dcbb914c</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="4012" label="CodeMixin" layerID="1"
        created="1358078140071" x="7074.7344" y="1652.6738" width="70.0"
        height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FF66CC</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/33fb2068c0a8000435fa8379b118788f</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="4017" layerID="1" created="1358111659745" x="9189.595"
        y="352.52405" width="29.0" height="19.99997" strokeWidth="1.0"
        autoSized="false" xsi:type="node">
        <fillColor>#30D643</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/35c8af4ac0a8000435fa8379819ac38b</URIString>
        <shape xsi:type="diamond"/>
    </child>
    <child ID="4018" label="__auto_name__ True" layerID="1"
        created="1358111659745" x="9222.637" y="354.02402" width="300.0"
        height="10.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
        <strokeColor>#404040</strokeColor>
        <textColor>#000000</textColor>
        <font>SansSerif-plain-14</font>
        <URIString>http://vue.tufts.edu/rdf/resource/35c8af4ac0a8000435fa83793900fa23</URIString>
        <richText>&lt;html&gt;
  &lt;head&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      __auto_name__ True
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
        <label>__auto_name__ True</label>
    </child>
    <child ID="4019" layerID="1" created="1358111748691" x="9189.595"
        y="382.92407" width="29.0" height="19.99997" strokeWidth="1.0"
        autoSized="false" xsi:type="node">
        <fillColor>#EA2218</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/35c8af4bc0a8000435fa837996a96163</URIString>
        <shape xsi:type="diamond"/>
    </child>
    <child ID="4020" label="strictly_named" layerID="1"
        created="1358111748691" x="9222.637" y="384.42404" width="300.0"
        height="10.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
        <strokeColor>#404040</strokeColor>
        <textColor>#000000</textColor>
        <font>SansSerif-plain-14</font>
        <URIString>http://vue.tufts.edu/rdf/resource/35c8af4bc0a8000435fa83797a0999cc</URIString>
        <richText>&lt;html&gt;
  &lt;head&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      __auto_name__ False
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
        <label>strictly_named</label>
    </child>
    <child ID="3870" layerID="1" created="1336638487865" x="11292.734"
        y="560.5132" width="80.831055" height="104.398926"
        strokeWidth="1.0" strokeStyle="4" autoSized="false"
        controlCount="2" arrowState="3" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/35fa8d457f000101663ffd14cd1b4218</URIString>
        <point1 x="11373.065" y="561.0132"/>
        <point2 x="11293.234" y="664.4121"/>
        <ID1 xsi:type="node">3834</ID1>
        <ID2 xsi:type="node">3866</ID2>
        <ctrlPoint0 x="11310.913" y="594.84155" xsi:type="point"/>
        <ctrlPoint1 x="11345.901" y="639.49976" xsi:type="point"/>
    </child>
    <child ID="4067" label="ACLMixin" layerID="1"
        created="1364475580000" x="7231.366" y="1680.6133" width="186.0"
        height="66.5" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/b118a18d4a976c7d56f44fb9c5dca92c</URIString>
        <child ID="4068" label="__acl__ | List of Tuples"
            created="1364475580002" x="34.0" y="23.0" width="141.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/b118a18e4a976c7d56f44fb9e3c74e42</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4070" label="permissions | MANY | Permission"
            created="1364475736939" x="34.0" y="43.25" width="195.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/b118a18e4a976c7d56f44fb94262c610</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="4069" label="ACLMixin" layerID="1"
        created="1364475580000" x="7292.366" y="1652.6738" width="64.0"
        height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FF66CC</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/b118a18f4a976c7d56f44fb9bbe83150</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="4092" label="Studio" layerID="1" created="1366071020306"
        x="8491.448" y="1348.2256" width="204.0" height="266.74997"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/103310bf1d61c1f706111f5d817e32c0</URIString>
        <child ID="4093" label="ScheduleMixin" created="1366071020308"
            x="34.0" y="23.0" width="94.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/103310c01d61c1f706111f5d68aee756</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4121" label="WorkingHoursMixin"
            created="1366071463871" x="34.0" y="43.25" width="123.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/10349fb61d61c1f706111f5da867a161</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4106" label="scheduler | SchedulerBase"
            created="1366071262667" x="34.0" y="63.5" width="160.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/103310c41d61c1f706111f5de62954c5</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4122" label="now | datetime.datetime"
            created="1366071497033" x="34.0" y="83.75" width="142.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/1035292f1d61c1f706111f5d98bf7781</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4099" label="projects | Project.query.all()"
            created="1366071047342" x="34.0" y="104.0" width="167.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/103310c11d61c1f706111f5dbb37702f</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4104" label="users | User.query.all()"
            created="1366071226855" x="34.0" y="124.25" width="139.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/103310c21d61c1f706111f5d381d3a8c</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4098" label="departments | Department.query.all()"
            created="1366071046441" x="34.0" y="144.5" width="219.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/103310c21d61c1f706111f5d370ce97a</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4102" label="active_project | Project.query..."
            created="1366071192280" x="34.0" y="164.75" width="186.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/103310c21d61c1f706111f5d668f6479</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4103" label="inactive_project | Project.query..."
            created="1366071219401" x="34.0" y="185.0" width="196.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/103310c31d61c1f706111f5ddfaa481b</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4105" label="schedule()" created="1366071245456"
            x="34.0" y="205.25" width="71.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/103310c31d61c1f706111f5df5b70542</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4096" created="1366071020312" x="34.0" y="225.5"
            width="29.0" height="23.0" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#FEFB03</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/103310c41d61c1f706111f5d83a80721</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <child ID="4097" created="1366071020313" x="34.0" y="245.75"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/103310c41d61c1f706111f5d1302ab7d</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="4101" layerID="1" created="1366071133748" x="8583.315"
        y="751.1875" width="194.16797" height="597.53906"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/103310c51d61c1f706111f5d43894860</URIString>
        <point1 x="8776.983" y="751.6875"/>
        <point2 x="8583.815" y="1348.2266"/>
        <ID1 xsi:type="node">1887</ID1>
        <ID2 xsi:type="node">4092</ID2>
        <ctrlPoint0 x="8783.341" y="1353.5111" xsi:type="point"/>
        <ctrlPoint1 x="8575.693" y="1235.7643" xsi:type="point"/>
    </child>
    <child ID="4107" label="SchedulerBase" layerID="1"
        created="1366071283725" x="8140.0244" y="274.64355"
        width="126.0" height="66.5" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/103310c51d61c1f706111f5dadc12882</URIString>
        <child ID="4109" label="studio" created="1366071283725" x="34.0"
            y="23.0" width="45.0" height="23.0" strokeWidth="1.0"
            autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/103310c51d61c1f706111f5d1c0e0a66</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4110" label="schedule()" created="1366071297632"
            x="34.0" y="43.25" width="71.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/103310c61d61c1f706111f5dc481d994</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="4111" label="TaskJugglerScheduler" layerID="1"
        created="1366071325565" x="8118.3496" y="405.84357"
        width="168.0" height="107.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/103310c61d61c1f706111f5dd4376894</URIString>
        <child ID="4112" label="tjp_file" created="1366071325566"
            x="34.0" y="23.0" width="47.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/103310c61d61c1f706111f5d2faedd65</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4115" label="csv_file" created="1366071355618"
            x="34.0" y="43.25" width="52.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/103310c71d61c1f706111f5dfad5796d</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4113" label="studio" created="1366071325566" x="34.0"
            y="63.5" width="46.0" height="23.0" strokeWidth="1.0"
            autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/103310c71d61c1f706111f5d1df97965</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4116" label="schedule()" created="1366071369168"
            x="34.0" y="83.75" width="71.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/103310c71d61c1f706111f5d1fa3fa02</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="4114" layerID="1" created="1366071328767" x="8202.088"
        y="340.625" width="1.2880859" height="65.625" strokeWidth="1.0"
        autoSized="false" controlCount="0" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/103310c81d61c1f706111f5d46f498e5</URIString>
        <point1 x="8202.876" y="341.125"/>
        <point2 x="8202.588" y="405.75"/>
        <ID1 xsi:type="node">4107</ID1>
        <ID2 xsi:type="node">4111</ID2>
    </child>
    <child ID="4117" label="WorkingHoursMixin" layerID="1"
        created="1366071433048" x="7451.5493" y="1680.6133"
        width="182.25" height="107.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/10349fba1d61c1f706111f5de58b0d92</URIString>
        <child ID="4119" label="working_hours | PICKLE"
            created="1366071433049" x="34.0" y="23.0" width="145.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/10349fba1d61c1f706111f5dd49712cc</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4123" label="daily_working_hours | PICKLE"
            created="1366071553456" x="34.0" y="43.25" width="177.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/10367d161d61c1f706111f5d5f945c46</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4124" label="weekly_working_hours | FLOAT"
            created="1366071569203" x="34.0" y="63.5" width="190.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/10367d161d61c1f706111f5df4afa051</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4125" label="yearly_working_hours | FLOAT"
            created="1366071584881" x="34.0" y="83.75" width="177.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/10367d161d61c1f706111f5df47d61a6</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="4120" label="WorkingHoursMixin" layerID="1"
        created="1366071433048" x="7481.1743" y="1652.6738"
        width="123.0" height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FF66CC</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/10349fba1d61c1f706111f5d52b2e31e</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="4127" label="Scene" layerID="1" created="1368620198891"
        x="8188.949" y="1858.0773" width="133.5" height="124.99998"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/a8223faca62df9af491657517acecb2b</URIString>
        <child ID="4137" label="ProjectMixin" created="1368620291689"
            x="34.0" y="23.0" width="82.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/a8223faca62df9af491657512cf9eb8b</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4128" label="CodeMixin" created="1368620198893"
            x="34.0" y="43.25" width="70.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/a8223fada62df9af491657510ea3a2ed</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4129" label="shots | MANY | SHOT"
            created="1368620198895" x="34.0" y="63.5" width="125.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/a8223fada62df9af49165751d2e649e2</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4134" created="1368620198904" x="34.0" y="83.75"
            width="29.0" height="23.0" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#FEFB03</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/a8223faea62df9af49165751a44b4527</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <child ID="4135" created="1368620198906" x="34.0" y="104.0"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/a8223faea62df9af49165751cba15620</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="4136" layerID="1" created="1368620229200" x="8261.573"
        y="751.2031" width="595.2168" height="1107.3828"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/a8223fafa62df9af49165751e9b56c77</URIString>
        <point1 x="8792.848" y="751.7031"/>
        <point2 x="8262.073" y="1858.0859"/>
        <ID1 xsi:type="node">1887</ID1>
        <ID2 xsi:type="node">4127</ID2>
        <ctrlPoint0 x="9082.37" y="1994.8195" xsi:type="point"/>
        <ctrlPoint1 x="8288.798" y="1596.0646" xsi:type="point"/>
    </child>
    <child ID="4159" label="Vacation" layerID="1"
        created="1382708793616" x="8428.048" y="498.0102" width="125.25"
        height="140.74994" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/efe0076bc8211d50048a37431a1c1633</URIString>
        <child ID="4160" label="ScheduleMixin" created="1382708793617"
            x="34.0" y="23.0" width="94.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#FF66CC</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/efe0076bc8211d50048a3743cecdca2f</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4161" label="user | ONE | USER"
            created="1382708793617" x="34.0" y="43.25" width="114.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/efe0076cc8211d50048a374306e29274</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4163" created="1382708793617" x="34.0" y="63.5"
            width="29.0" height="23.0" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#FEFB03</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/efe0076cc8211d50048a3743133d5bd2</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <child ID="4164" created="1382708793617" x="34.0" y="83.75"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/efe0076cc8211d50048a374340c4fe92</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <child ID="4213" created="1468653290985" x="34.0" y="101.74998"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#33A8F5</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/f28ffbffc0a8000f011bc52fd9c4fed2</URIString>
            <shape xsi:type="chevron"/>
        </child>
        <child ID="4214" created="1468653294154" x="34.0" y="119.749954"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#AF55F4</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/f28ffbffc0a8000f011bc52f22a80a94</URIString>
            <shape xsi:type="rhombus"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="4166" layerID="1" created="1382708864226" x="8496.098"
        y="470.6831" width="312.2793" height="44.147827"
        strokeWidth="1.0" autoSized="false" controlCount="2"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/efe0076cc8211d50048a37434fdaa095</URIString>
        <point1 x="8807.877" y="487.09375"/>
        <point2 x="8496.598" y="498.0"/>
        <ID1 xsi:type="node">1901</ID1>
        <ID2 xsi:type="node">4159</ID2>
        <ctrlPoint0 x="8810.178" y="577.50446" xsi:type="point"/>
        <ctrlPoint1 x="8504.1455" y="408.33646" xsi:type="point"/>
    </child>
    <child ID="4170" label="Media" layerID="1" created="1397835424148"
        x="9336.949" y="1027.2249" width="158.25" height="127.25"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/757fa42fc0a8004336c48f760e6c52fc</URIString>
        <child ID="4178" label="width | INTEGER" created="1397835476427"
            x="34.0" y="23.0" width="102.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/757fa42fc0a8004336c48f762d6b6b6d</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4179" label="height | INTEGER"
            created="1397835493677" x="34.0" y="43.25" width="107.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/757fa430c0a8004336c48f7684388152</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4183" label="duration | FLOAT"
            created="1397835571109" x="34.0" y="63.5" width="102.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/757fa431c0a8004336c48f76121a87ec</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4181" label="bitrate | INTEGER"
            created="1397835532870" x="34.0" y="83.75" width="107.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/757fa431c0a8004336c48f76ea9e7e28</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4186" label="other_sizes | MANY | Media"
            created="1397835683545" x="34.0" y="104.0" width="158.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/7581cfb5c0a8004336c48f7613cb7500</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="4171" layerID="1" created="1397835424158" x="9494.699"
        y="974.3784" width="103.32324" height="66.399414"
        strokeWidth="1.0" autoSized="false" controlCount="0"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/757fa430c0a8004336c48f76e74e4757</URIString>
        <point1 x="9597.522" y="974.8784"/>
        <point2 x="9495.199" y="1040.2778"/>
        <ID1 xsi:type="node">1145</ID1>
        <ID2 xsi:type="node">4170</ID2>
    </child>
    <child ID="4191" label="Roles" layerID="1" created="1459162812123"
        x="8715.449" y="1348.2256" width="71.0" height="64.24998"
        strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/bce315cdc0a82a4a01152d0e90f705d8</URIString>
        <child ID="4194" created="1459162812126" x="34.0" y="23.0"
            width="29.0" height="23.0" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/bce315cdc0a82a4a01152d0ebdecdd7d</URIString>
            <shape xsi:type="triangle"/>
        </child>
        <child ID="4195" created="1459162812127" x="34.0" y="43.25"
            width="29.0" height="19.99997" strokeWidth="1.0"
            autoSized="false" xsi:type="node">
            <fillColor>#EA2218</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/bce315cec0a82a4a01152d0e48b05e0b</URIString>
            <shape xsi:type="diamond"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="4196" layerID="1" created="1459162843527" x="8751.61"
        y="751.1875" width="22.542969" height="597.5156"
        strokeWidth="1.0" autoSized="false" controlCount="0"
        arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/bce315cec0a82a4a01152d0e6dc206c0</URIString>
        <point1 x="8773.653" y="751.6875"/>
        <point2 x="8752.11" y="1348.2031"/>
        <ID1 xsi:type="node">1887</ID1>
        <ID2 xsi:type="node">4191</ID2>
    </child>
    <child ID="4204" layerID="1" created="1468652962732" x="9189.595"
        y="418.3753" width="29.0" height="19.99997" strokeWidth="1.0"
        autoSized="false" xsi:type="node">
        <fillColor>#33A8F5</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/f28ffc00c0a8000f011bc52f338782cf</URIString>
        <shape xsi:type="chevron"/>
    </child>
    <child ID="4205"
        label="Implmented as RESTFul Service with unit tests"
        layerID="1" created="1468652962732" x="9222.637" y="419.87527"
        width="298.0" height="14.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
        <strokeColor>#404040</strokeColor>
        <textColor>#000000</textColor>
        <font>SansSerif-plain-14</font>
        <URIString>http://vue.tufts.edu/rdf/resource/f28ffc00c0a8000f011bc52f7f3ff996</URIString>
        <richText>&lt;html&gt;
  &lt;head&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Implmented as RESTFul Service with unit tests
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
        <label>Implmented as RESTFul Service with unit tests</label>
    </child>
    <child ID="4206" layerID="1" created="1468653091361" x="9187.595"
        y="452.82346" width="29.0" height="19.99997" strokeWidth="1.0"
        autoSized="false" xsi:type="node">
        <fillColor>#AF55F4</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-plain-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/f28ffc01c0a8000f011bc52f168f3afb</URIString>
        <shape xsi:type="rhombus"/>
    </child>
    <child ID="4207"
        label="Implemented as RESTFul Service with functional tests"
        layerID="1" created="1468653091361" x="9222.637" y="454.32343"
        width="297.0" height="17.0" strokeWidth="0.0" autoSized="false" xsi:type="text">
        <strokeColor>#404040</strokeColor>
        <textColor>#000000</textColor>
        <font>SansSerif-plain-14</font>
        <URIString>http://vue.tufts.edu/rdf/resource/f28ffc01c0a8000f011bc52fd0608bf1</URIString>
        <richText>&lt;html&gt;
  &lt;head&gt;
    &lt;style type="text/css"&gt;
      &lt;!--
        body { margin-left: 0px; font-size: 12; font-family: Arial; margin-top: 0px; margin-right: 0px; color: #000000; margin-bottom: 0px }
        ol { vertical-align: middle; margin-left: 30; list-style-position: outside; margin-top: 6; font-family: Arial; font-size: 12 }
        p { margin-left: 0; margin-top: 0; margin-right: 0; color: #000000; margin-bottom: 0 }
        ul { vertical-align: middle; margin-left: 30; list-style-position: outside; font-family: Arial; margin-top: 6; font-size: 12 }
      --&gt;
    &lt;/style&gt;
    
  &lt;/head&gt;
  &lt;body&gt;
    &lt;p style="color: #000000" color="#000000"&gt;
      Implemented as RESTFul Service with functional tests
    &lt;/p&gt;
  &lt;/body&gt;
&lt;/html&gt;
</richText>
        <label>Implemented as RESTFul Service with functional tests</label>
    </child>
    <child ID="4233" label="DAGMixin" layerID="1"
        created="1471422518912" x="6824.949" y="1870.0214"
        width="173.25" height="167.75" strokeWidth="1.0"
        autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/97ae2d15c0a82a4a019efdbd4499f5e0</URIString>
        <child ID="4234" label="parent | {CLASS}"
            created="1471422518913" x="34.0" y="23.0" width="102.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/97ae2d15c0a82a4a019efdbdd74739f1</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4236" label="children | {CLASS}"
            created="1471422664669" x="34.0" y="43.25" width="111.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/97ae2d15c0a82a4a019efdbdee3e5fcf</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4237" label="is_root | BOOL" created="1471422705938"
            x="34.0" y="63.5" width="93.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/97ae2d15c0a82a4a019efdbd580043fc</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4238" label="is_container | BOOL"
            created="1471422717295" x="34.0" y="83.75" width="124.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/97ae2d16c0a82a4a019efdbdb898570c</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4239" label="is_leaf | BOOL" created="1471422756262"
            x="34.0" y="104.0" width="91.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/97ae2d16c0a82a4a019efdbd49bb9763</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4240" label="parents | [{CLASS}]"
            created="1471422769228" x="34.0" y="124.25" width="121.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/97ae2d16c0a82a4a019efdbdb9a3c36c</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4241" label="walk_hierarchy | GENERATOR"
            created="1471422788467" x="34.0" y="144.5" width="178.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/97ae2d16c0a82a4a019efdbd21ac06fb</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="4235" label="DAGMixin" layerID="1"
        created="1471422518912" x="6879.074" y="1842.0819" width="65.0"
        height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FF66CC</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/97ae2d16c0a82a4a019efdbdcdc4ddd0</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="4252" label="DateRangeMixin" layerID="1"
        created="1471423301805" x="7051.2344" y="1870.0214"
        width="183.75" height="208.25" strokeWidth="1.0"
        autoSized="true" xsi:type="node">
        <fillColor>#FEFB03</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/97ae2d16c0a82a4a019efdbd8801c064</URIString>
        <child ID="4253" label="start | DATETIME"
            created="1471423301806" x="34.0" y="23.0" width="102.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/97ae2d16c0a82a4a019efdbda9b85014</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4254" label="end | DATETIME" created="1471423301806"
            x="34.0" y="43.25" width="99.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/97ae2d16c0a82a4a019efdbdd0362e2b</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4255" label="duration | TIMEDELTA"
            created="1471423301806" x="34.0" y="63.5" width="130.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/97ae2d16c0a82a4a019efdbd7680eb26</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4256" label="computed_start | DATETIME"
            created="1471423301806" x="34.0" y="83.75" width="164.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/97ae2d17c0a82a4a019efdbd0eab4c71</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4257" label="computed_end | DATETIME"
            created="1471423301807" x="34.0" y="104.0" width="161.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/97ae2d17c0a82a4a019efdbdf8cdb703</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4258" label="computed_duration | TIMEDELTA"
            created="1471423301807" x="34.0" y="124.25" width="192.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#F2AE45</fillColor>
            <strokeColor>#7F7F7F</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-plain-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/97ae2d17c0a82a4a019efdbd85fb5607</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4263" label="round_time" created="1471423301809"
            x="34.0" y="144.5" width="76.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/97ae2d17c0a82a4a019efdbd5c065e28</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4269" label="total_seconds" created="1471423499525"
            x="34.0" y="164.75" width="92.0" height="23.0"
            strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/97ae2d17c0a82a4a019efdbd9812cb2e</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <child ID="4271" label="computed_total_seconds"
            created="1471423513144" x="34.0" y="185.0" width="156.0"
            height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
            <fillColor>#83CEFF</fillColor>
            <strokeColor>#776D6D</strokeColor>
            <textColor>#000000</textColor>
            <font>Arial-bold-12</font>
            <URIString>http://vue.tufts.edu/rdf/resource/97ae2d17c0a82a4a019efdbde484861f</URIString>
            <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
        </child>
        <shape xsi:type="rectangle"/>
    </child>
    <child ID="4264" label="DateRangeMixin" layerID="1"
        created="1471423301805" x="7091.6094" y="1842.0819"
        width="103.0" height="23.0" strokeWidth="1.0" autoSized="true" xsi:type="node">
        <fillColor>#FF66CC</fillColor>
        <strokeColor>#776D6D</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-12</font>
        <URIString>http://vue.tufts.edu/rdf/resource/97ae2d17c0a82a4a019efdbde98bd4d5</URIString>
        <shape arcwidth="20.0" archeight="20.0" xsi:type="roundRect"/>
    </child>
    <child ID="4273" layerID="1" created="1471423799077" x="8775.2705"
        y="751.25" width="654.4365" height="1542.6562" strokeWidth="1.0"
        autoSized="false" controlCount="2" arrowState="2" xsi:type="link">
        <strokeColor>#000000</strokeColor>
        <textColor>#000000</textColor>
        <font>Arial-bold-11</font>
        <URIString>http://vue.tufts.edu/rdf/resource/97b7c5fac0a82a4a01081779ad8219f2</URIString>
        <point1 x="8775.7705" y="751.75"/>
        <point2 x="9429.207" y="2293.4062"/>
        <ID1 xsi:type="node">1887</ID1>
        <ID2 xsi:type="node">4225</ID2>
        <ctrlPoint0 x="8768.16" y="1934.3668" xsi:type="point"/>
        <ctrlPoint1 x="9433.276" y="2152.837" xsi:type="point"/>
    </child>
    <layer ID="1" label="Layer 1" created="0" x="0.0" y="0.0"
        width="1.4E-45" height="1.4E-45" strokeWidth="0.0" autoSized="false">
        <URIString>http://vue.tufts.edu/rdf/resource/5d104b32c00007d601b277f026b72230</URIString>
    </layer>
    <userZoom>1.3783363092809884</userZoom>
    <userOrigin x="6797.533" y="-1082.5613"/>
    <presentationBackground>#202020</presentationBackground>
    <PathwayList currentPathway="-1" revealerIndex="-1"/>
    <date>2009-05-20</date>
    <modelVersion>6</modelVersion>
    <saveLocation>C:\Users\eoyilmaz\Documents\development\stalker\stalker\docs\source\_static\images</saveLocation>
    <saveFile>C:\Users\eoyilmaz\Documents\development\stalker\stalker\docs\source\_static\images\stalker_design.vue</saveFile>
</LW-MAP>
